Package: GLmom
Type: Package
Title: Generalized L-Moments Estimation for Extreme Value Distributions
Version: 2.0.0
Authors@R: c(
      person(given = "Yonggwan",
             family = "Shin",
             role = c("aut", "cre"),
             email = "syg.stat@etri.re.kr",
             comment = c(ORCID = "0000-0001-6966-6511")),
      person(given = "Seokkap",
             family = "Ko",
             role = c("aut", "ctb"),
             email = "softgear@etri.re.kr",
             comment = c(ORCID = "0000-0001-9218-7236")),
       person(given = "Jihong",
             family = "Park",
             role = "ctb",
             email="jihong8090@gmail.com",
             comment = c(ORCID = "0009-0003-3191-9968")),
      person(given = "Yire",
             family = "Shin",
             role = c("aut", "dtc"),
             email = "shinyire87@gmail.com",
             comment = c(ORCID = "0000-0003-1297-5430")),
      person(given = "Jeong-Soo",
             family = "Park",
             role = c("aut", "ths"),
             email = "jspark@chonnam.ac.kr",
             comment = c(ORCID = "0000-0002-8460-4869")))
Description: Provides generalized L-moments estimation methods for the
    generalized extreme value ('GEV') distribution. Implements both stationary
    'GEV' and non-stationary 'GEV11' models where location and scale parameters
    vary with time. Includes various penalty functions (Martins-Stedinger,
    Park, Cannon, Coles-Dixon) for shape parameter regularization.
    Also provides model averaging estimation ('ma.gev') that combines MLE and
    L-moment methods with multiple weighting schemes for robust high quantile
    estimation. The 'GLME' methodology is described in Shin et al. (2025a)
    <doi:10.48550/arXiv.2512.20385>. The non-stationary L-moment method
    is based on Shin et al. (2025b) <doi:10.1007/s42952-025-00325-3>.
    The model averaging method is described in Shin et al. (2026)
    <doi:10.1007/s00477-025-03167-x>.
    See also Hosking (1990) <doi:10.1111/j.2517-6161.1990.tb01775.x> for
    L-moments theory and Martins and Stedinger (2000)
    <doi:10.1029/1999WR900330> for penalized likelihood methods.
License: GPL (>= 3)
URL: https://github.com/sygstat/GLmom
BugReports: https://github.com/sygstat/GLmom/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Imports: lmomco, nleqslv, stats, methods, robustbase, ismev, Rsolnp,
        zoo, graphics
Depends: R (>= 3.5.0)
NeedsCompilation: no
Packaged: 2026-07-05 13:01:28 UTC; yg
Author: Yonggwan Shin [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6966-6511>),
  Seokkap Ko [aut, ctb] (ORCID: <https://orcid.org/0000-0001-9218-7236>),
  Jihong Park [ctb] (ORCID: <https://orcid.org/0009-0003-3191-9968>),
  Yire Shin [aut, dtc] (ORCID: <https://orcid.org/0000-0003-1297-5430>),
  Jeong-Soo Park [aut, ths] (ORCID:
    <https://orcid.org/0000-0002-8460-4869>)
Maintainer: Yonggwan Shin <syg.stat@etri.re.kr>
Repository: CRAN
Date/Publication: 2026-07-05 15:50:17 UTC

# Export main functions
export(glme.gev)
export(glme.gev11)
export(glme.like)
export(init.glme)
export(pargev.kfix)
export(MS_pk)
export(new_pf_norm)
export(pk.norm.stnary)
export(pk.beta)
export(pk.beta.stnary)

# Non-stationary GEV11 methods (new in v2.0.0)
export(lme.gev11)
export(strup.gev11)
export(GN16.gev11)
export(ran.gev_all)

# S3 methods for fitted objects (v2.0.0)
S3method(print, glme)
S3method(summary, glme)
S3method(plot, glme)
S3method(print, glme11)
S3method(summary, glme11)
S3method(plot, glme11)
S3method(print, lme11)
S3method(summary, lme11)
S3method(plot, lme11)
S3method(print, magev)
S3method(summary, magev)
S3method(plot, magev)

# Shin et al. (2025b) compatibility functions
export(nsgev)
export(gado.prop_11)
export(quagev.NS)

# Shin et al. (2026) Model Averaging function
export(ma.gev)

# MAGEV Plotting functions
export(magev.ksensplot)
export(magev.qqplot)
export(magev.rlplot)

# Imports from lmomco
importFrom("lmomco", "lmoms", "pargev", "lmoms.cov", "vec2par", "lmomgev", "lmorph", "are.lmom.valid", "TLmoms", "lmomgum", "quagev", "pargum", "rlmomco")

# Imports from stats
importFrom("stats", "optim", "rnorm", "runif", "integrate", "dnorm", "lm", "var", "sd", "cov", "median", "quantile", "cor", "spline", "qchisq", "na.omit", "ppoints")

# Imports from methods
importFrom("methods", "is")

# Imports from nleqslv
importFrom("nleqslv", "nleqslv")

# Imports from robustbase
importFrom("robustbase", "lmrob")

# Imports for MAGEV
importFrom("ismev", "gev.fit")
importFrom("Rsolnp", "solnp")
importFrom("zoo", "rollmean")
importFrom("graphics", "plot", "abline", "text", "par", "axis", "lines", "points")

# Model Averaging for GEV High Quantile Estimation
#
# Reference: Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with
# mixed criteria for estimating high quantiles of extreme values: Application
# to heavy rainfall. Stochastic Environmental Research and Risk Assessment,
# 40(2), 47. https://doi.org/10.1007/s00477-025-03167-x


#' Model Averaging for GEV High Quantile Estimation
#'
#' @description
#' This function estimates high quantiles of the Generalized Extreme Value (GEV)
#' distribution using model averaging with mixed criteria. It combines Maximum
#' Likelihood Estimation (MLE) and L-moment Estimation (LME) to construct
#' candidate submodels and assign weights effectively.
#'
#' @param data A numeric vector of data to be fitted (e.g., annual maxima).
#' @param quant The probabilities corresponding to high quantiles to be estimated.
#'   Default is c(0.98, 0.99, 0.995).
#' @param weight The weighting method name. Options are:
#'   \itemize{
#'     \item 'like', 'like0', 'like1' (default): Likelihood-based weights (AIC)
#'     \item 'gLd', 'gLd0', 'gLd1', 'gLd2': Generalized L-moment distance weights
#'     \item 'med', 'med1', 'med2': Median-based weights
#'     \item 'cvt': Conventional AIC weights
#'   }
#'   Variants with numbers indicate left trimming level (0, 1, or 2).
#' @param numk The number of candidate submodels K. Default is 12.
#' @param B The number of bootstrap samples. Default is 200.
#' @param varcom Logical. Whether to compute variance of quantile estimates.
#'   Default is TRUE.
#' @param trim The number of left trimming for L-moments. Usually 0 (default), 1, or 2.
#' @param fig Logical. Whether to produce diagnostic plots. Default is FALSE.
#' @param bma Logical. Whether to use Bayesian Model Averaging. Default is FALSE.
#' @param pen Penalty type for BMA prior: 'norm' (normal, default) or 'beta'.
#' @param CD Logical. Whether to compute Coles-Dixon penalized MLE. Default is FALSE.
#' @param remle Logical. Whether to compute restricted MLE. Default is FALSE.
#'
#' @details
#' The model averaging approach works as follows:
#' \enumerate{
#'   \item MLE and LME of GEV parameters are computed
#'   \item K candidate shape parameters (xi) are selected from profile likelihood CI
#'   \item For each candidate xi, MLE with fixed xi is computed
#'   \item Weights are assigned based on the selected method
#'   \item Final quantile estimates are weighted averages across submodels
#' }
#'
#' The weighting schemes include:
#' \itemize{
#'   \item 'like': AIC-based weights using likelihood with fixed xi
#'   \item 'gLd': Weights based on generalized L-moment distance
#'   \item 'med': Weights based on median and L-moment distance
#'   \item 'cvt': Conventional AIC weights
#' }
#'
#' When bma=TRUE, Bayesian model averaging is applied with prior specified by pen.
#'
#' @return A list containing:
#' \itemize{
#'   \item mle.hosking - MLE estimates in Hosking style (mu, sigma, xi)
#'   \item qua.mle - Quantile estimates from MLE
#'   \item mle.cov3 - Covariance matrix of MLE (3x3)
#'   \item qua.se.mle.delta - Standard errors of MLE quantiles (delta method)
#'   \item lme - L-moment estimates (mu, sigma, xi)
#'   \item lme.cov3 - Covariance matrix of LME (bootstrap)
#'   \item qua.lme - Quantile estimates from LME
#'   \item qua.se.lme.boots - Standard errors of LME quantiles (bootstrap)
#'   \item qua.ma - Model-averaged quantile estimates
#'   \item w.ma - Weights used for model averaging
#'   \item fixw.se.ma - Asymptotic SE under fixed weights
#'   \item ranw.se.ma - Asymptotic SE under random weights
#'   \item surr - Surrogate model parameters (mu, sigma, xi)
#'   \item pick_xi - Selected xi values for K submodels
#'   \item qua.bma - (if bma=TRUE) BMA quantile estimates
#'   \item w.bma - (if bma=TRUE) BMA weights
#'   \item mle.CD - (if CD=TRUE) Coles-Dixon penalized MLE
#'   \item qua.CD - (if CD=TRUE) Quantile estimates from CD-MLE
#'   \item remle1 - (if remle=TRUE) Restricted MLE (first constraint)
#'   \item qua.remle1 - (if remle=TRUE) Quantile estimates from remle1
#'   \item remle2 - (if remle=TRUE) Restricted MLE (second constraint)
#'   \item qua.remle2 - (if remle=TRUE) Quantile estimates from remle2
#'   \item quant - The quantile probabilities used
#' }
#'
#' @references
#' Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
#' for estimating high quantiles of extreme values: Application to heavy rainfall.
#' \emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
#' \doi{10.1007/s00477-025-03167-x}
#'
#' @seealso \code{\link{glme.gev}} for stationary GLME estimation,
#'   \code{\link{magev.ksensplot}} for K sensitivity analysis,
#'   \code{\link{magev.qqplot}} for Q-Q diagnostic plots,
#'   \code{\link{magev.rlplot}} for return level plots.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' # Load example data
#' data(haenam)
#' x <- haenam$X1
#'
#' # Basic usage with likelihood weights
#' result <- ma.gev(x, quant = c(0.95, 0.99), weight = 'like1', B = 100)
#' print(result$qua.ma)     # Model-averaged quantiles
#' print(result$qua.mle)    # MLE quantiles for comparison
#' print(result$qua.lme)    # LME quantiles for comparison
#'
#' # Using generalized L-moment distance weights
#' result2 <- ma.gev(x, quant = c(0.95, 0.99), weight = 'gLd', B = 100)
#' print(result2$w.ma)      # Model weights
#'
#' @export
ma.gev <- function(data = NULL, quant = c(0.98, 0.99, 0.995),
                   weight = 'like1', numk = 12, B = 200, varcom = TRUE,
                   trim = 0, fig = FALSE, bma = FALSE, pen = "norm",
                   CD = FALSE, remle = FALSE) {

  zx <- list()
  SMALL <- 1e-05
  numq <- length(quant)
  zx$weight <- weight
  start <- "mle"
  zx$start <- start
  zx$quant <- quant

  # Internal defaults
  pick <- 0.95
  pertr <- 0.85
  surr <- TRUE
  numom <- 3
  order <- 3
  boot.lme <- TRUE
  cov.lme <- NULL
  type <- 'full'
  cov.type <- 'lambda'
  fix.kpar <- FALSE
  pick.xi <- NULL

  if (fix.kpar == FALSE) {
    pick.xi <- NULL
  } else if (fix.kpar == TRUE) {
    numk <- length(pick.xi)
  }
  if (surr == FALSE) fig <- FALSE
  if (fig == TRUE) {
    surr <- TRUE
    boot.lme <- TRUE
    varcom <- TRUE
  }

  # Normalize weight names
  if (weight == 'gld') weight <- 'gLd'
  if (cov.type == 'r') cov.type <- 'ratio'
  if (cov.type == 'l') cov.type <- 'lambda'
  if (weight == 'like0') {
    weight <- 'like'
    trim <- 0
  }
  if (weight == 'like1') {
    weight <- 'like'
    trim <- 1
  }
  if (weight == 'gld0' | weight == 'gLd0') {
    weight <- 'gLd'
    trim <- 0
  }
  if (weight == 'gld1' | weight == 'gLd1') {
    weight <- 'gLd'
    trim <- 1
  }
  if (weight == 'gld2' | weight == 'gLd2') {
    weight <- 'gLd'
    trim <- 2
  }
  if (weight == 'med1') {
    weight <- 'med'
    trim <- 1
  }
  if (weight == 'med2') {
    weight <- 'med'
    trim <- 2
  }

  zx$data <- data
  org.numk <- numk

  para3 <- matrix(NA, nrow = numk, ncol = 3)
  wtgd <- rep(NA, numk)
  zp <- matrix(NA, numq, numk)
  zpf <- rep(NA, numq)

  nsample <- length(data)
  delta <- list()

  # ------- MLE and SE by delta method -------------------
  delta <- gev.rl.delta(data = data, ntry = 10, quant = quant)

  zx$mle.hosking <- delta$mle
  zx$qua.mle <- delta$qua.mle
  zx$mle.cov3 <- delta$cov

  if (varcom == TRUE & !is.null(delta$cov)) {
    qua.se.mle <- rep(NA, numq)
    for (i in 1:numq) {
      qua.se.mle[i] <- delta.gev(gevf3 = delta, quant = quant[i], d3yes = TRUE)$v3
    }
    zx$qua.se.mle.delta <- sqrt(qua.se.mle)
  }

  # ----- LME and SE by bootstrap --------------------
  if (is.null(cov.lme)) {
    if (start == 'lme' | weight == 'med' | start == 'mix' |
        weight == 'lcv' | weight == 'opt') {
      boot.lme <- TRUE
    }
  }

  hosking <- list()
  hosking <- lme.boots(data = data, B = B, quant = quant,
                           boot = boot.lme, trim = trim)

  if (boot.lme == TRUE) {
    zx$lme.cov3 <- hosking$cov.par
  } else if (boot.lme == FALSE) {
    zx$lme.cov3 <- cov.lme$cov.par
  }

  zx$lme <- hosking$lme
  zx$qua.lme <- hosking$qua.lme
  zx$leftrim <- trim
  hosking$mle <- delta$mle

  xi.hat <- (delta$mle[3] + hosking$lme[3]) / 2

  # ---- CD penalized MLE and restricted MLE ------------------------------------------
  if (CD == TRUE) {
    CD.mle <- rep(NA, 3)
    CD.mle <- mle.gev.CD(xdat = data, ntry = 5)$mle  # hosking style xi

    if (any(is.na(CD.mle))) {
      zx$qua.CD <- NA
      zx$mle.CD <- NA
      CD.mle <- NULL
    } else {
      zx$qua.CD <- quagev(quant, vec2par(CD.mle, 'gev'))
      zx$mle.CD <- CD.mle
    }
  }

  if (remle == TRUE) {
    if (CD != TRUE) CD.mle <- hosking$lme
    rem.try <- list()
    rem.try <- remle.gev(xdat = data, ntry = 5, rest = 'mean', quant = quant,
                         CD.mle = CD.mle, mle = delta$mle,
                         second = TRUE, w.mpse = FALSE)

    zx$remle1 <- rem.try$para.remle1
    zx$qua.remle1 <- rem.try$qua.remle1
    zx$remle2 <- rem.try$para.remle2
    zx$qua.remle2 <- rem.try$qua.remle2
  }
  # --------------------------------------------------------------

  hosking$start <- start
  hosking$numk <- numk
  hosking$quant <- quant
  hosking$weight <- weight
  hosking$B <- B
  hosking$trim <- trim
  hosking$data <- data
  hosking$boot.lme <- boot.lme

  # --- xi_k picking for candidate submodels ---------------------------
  kpar <- rep(NA, numk)

  if (fix.kpar == FALSE) {
    kpar.list <- list()

    kpar.list <- cand.xi(data, hosking = hosking, mle = zx$mle.hosking,
                                   pick0 = pick, nint = 256, start = start,
                                   numk = numk, figure = fig, bma = bma, pen = pen)

    kpar <- kpar.list$kpar
    hosking$start <- kpar.list$start
  } else if (fix.kpar == TRUE) {
    kpar <- pick.xi
    hosking$start <- start
  }
  hosking$kpar <- kpar
  hosking$cov.type <- cov.type

  # ------- Zp and weights computing -----------------------------------
  run <- 1

  xqa <- c(0.5, .65, .8, .85, .9, .925, .95, .965, .98,
           .985, .99, .9925, .995, .9965, .998, .999)
  mywt <- list()
  hosking$pertr <- pertr
  bmaw <- rep(0, numk)
  wtgd <- rep(NA, numk)

  mywt <- weight.com(data, numk = numk, hosking = hosking,
                         kpar = kpar, numom = numom,
                         xqa = xqa, varcom = varcom, boot.lme = boot.lme,
                         cov.lme = cov.lme, surr = surr,
                         type = type, trim = trim, cov.type = cov.type,
                         bma = bma, pen = pen)

  para3 <- mywt$prob.call$mle3
  npara3 <- na.omit(para3)
  numk <- nrow(npara3)
  if (mywt$numk != numk) message("Note: numk adjusted after weight computation")
  notid <- which(!is.na(para3[, 1]))
  mywt$data <- data
  para3 <- npara3
  mywt$prob.call$mle3 <- npara3
  kpar <- kpar[notid]

  zp <- mywt$zp[, notid, drop = FALSE]
  xzp <- mywt$xzp[, notid, drop = FALSE]
  wtgd <- mywt$wtgd[notid]
  mywt$wtgd <- wtgd
  bmaw <- mywt$bmaw[notid]

  if (bma != TRUE) {
    newk <- new.kpar2(wtgd = wtgd, numk = numk, kpar = kpar)
  } else if (bma == TRUE) {
    newk <- new.kpar2(wtgd = bmaw, numk = numk, kpar = kpar)
  }

  numk <- newk$numk
  aw <- newk$aw

  while (aw >= 1) {
    run <- run + 1
    kpar <- newk$kpar2
    hosking$kpar <- newk$kpar2
    hosking$numk <- numk

    mywt <- list()
    bmaw <- rep(0, numk)
    wtgd <- rep(NA, numk)
    para3 <- matrix(NA, nrow = numk, ncol = 3)
    zp <- matrix(NA, numq, numk)

    mywt <- weight.com(data, numk = numk, hosking = hosking,
                           kpar = newk$kpar2, numom = numom,
                           xqa = xqa, varcom = varcom, boot.lme = boot.lme,
                           cov.lme = cov.lme, surr = surr,
                           type = type, trim = trim, cov.type = cov.type,
                           bma = bma, pen = pen)

    para3 <- mywt$prob.call$mle3
    npara3 <- na.omit(para3)
    numk <- nrow(npara3)
    if (mywt$numk != numk) message("Note: numk adjusted in iteration")
    notid <- which(!is.na(para3[, 1]))
    mywt$data <- data
    para3 <- npara3
    mywt$prob.call$mle3 <- npara3
    kpar <- mywt$kpar
    kpar <- kpar[notid]

    zp <- mywt$zp[, notid, drop = FALSE]
    xzp <- mywt$xzp[, notid, drop = FALSE]
    wtgd <- mywt$wtgd[notid]
    mywt$wtgd <- wtgd
    bmaw <- mywt$bmaw[notid]
    bmaw2 <- if (!is.null(mywt$bmaw2)) mywt$bmaw2[notid] else NULL

    if (run >= 5) break
    mxid <- which.max(wtgd)
    bmxid <- which.max(bmaw)

    tre1 <- 0.1
    tre8 <- 0.5

    if (bma != TRUE) {
      if (wtgd[1] > tre1 | wtgd[numk] > tre1 | wtgd[mxid] > tre8) {
        newk <- new.kpar2(wtgd = wtgd, numk = numk, kpar = kpar)
        numk <- newk$numk
        aw <- newk$aw
      } else {
        aw <- 0
        break
      }
    }

    if (bma == TRUE) {
      if (bmaw[1] > tre1 | bmaw[numk] > tre1 | bmaw[bmxid] > tre8) {
        newk <- new.kpar2(wtgd = bmaw, numk = numk, kpar = kpar)
        numk <- newk$numk
        aw <- newk$aw
      } else {
        aw <- 0
        break
      }
    }
  }

  idp3 <- which(is.na(para3[, 1]))
  if (length(idp3) > 0) {
    para3[idp3, ] <- 0
    wtgd[idp3] <- 0
    bmaw[idp3] <- 0
  }

  # Handle NA in zp - different handling for numq==1 vs numq>1
  if (numq > 1) {
    id <- which(is.na(t(zp)[, 1]))
    if (length(id) > 0) {
      zp[, id] <- 0
      wtgd[id] <- 0
      bmaw[id] <- 0
    }
  } else if (numq == 1) {
    id <- which(is.na(zp))
    if (length(id) > 0) {
      zp[id] <- 0
      wtgd[id] <- 0
      bmaw[id] <- 0
    }
  }

  if (all(wtgd == 0)) {
    message("All return levels are NA, returning LME quantiles")
    zx$qua.ma <- zx$qua.lme
    zx$qua.bma <- zx$qua.lme
    class(zx) <- "magev"
    return(zx)
  }

  bmaw[which(is.na(bmaw))] <- 0.0
  wtgd[which(is.na(wtgd))] <- 0.0

  zpf <- rep(NA, numq)
  zpf.bma <- rep(NA, numq)
  if (numq > 1) {
    for (iq in 1:numq) {
      zpf[iq] <- t(wtgd) %*% t(zp)[1:numk, iq]
      if (bma == TRUE) zpf.bma[iq] <- t(bmaw) %*% t(zp)[1:numk, iq]
    }
  } else if (numq == 1) {
    zpf[1] <- sum(wtgd * as.vector(zp))
    if (bma == TRUE) zpf.bma[1] <- sum(bmaw * as.vector(zp))
  }

  # ---- calculating asymptotic SE by delta method -----------------
  if (varcom == TRUE) {
    covint <- cov.interp(numk, para3, cov2 = mywt$prob.call$cov2)
    avar <- asymp.var(mywt, covint, qqq = quant, order)
  }

  # ----- finding surrogate for MA using GEV -----------
  if (surr == TRUE) {
    zpf.surr <- t(wtgd) %*% t(xzp)
    para.ma <- t(wtgd) %*% para3
    surro <- surrogate(zpf.surr, xqa, init.surr = para.ma)
    zx$surr <- list()
    zx$surr$par <- as.vector(surro$par)
  }

  # ----- Predictive posterior variance ----------------
  if (varcom == TRUE & bma == TRUE) {
    zpdiff <- matrix(NA, numq, numk)
    loc.var <- matrix(NA, numq, numk)
    for (ip in 1:numk) {
      if (numq > 1) {
        zpdiff[1:numq, ip] <- (zp[1:numq, ip] - zpf.bma[1:numq])^2
      } else if (numq == 1) {
        zpdiff[1:numq, ip] <- (zp[ip] - zpf.bma[1:numq])^2
      }
      loc.var[1:numq, ip] <- avar$MatC[ip, ip, 1:numq]
    }
    msm <- t(bmaw) %*% t(zpdiff)
    mse <- t(bmaw) %*% t(loc.var)
  }

  if (varcom == TRUE) {
    zx$fixw.se.ma <- avar$fin.se.MA.qua
    zx$ranw.se.ma <- avar$adj.se.MA.qua
    if (bma == TRUE) {
      zx$pred.se.bma <- as.vector(sqrt(msm + mse))
      zx$bma.se.between <- sqrt(msm)
      zx$bma.se.within <- sqrt(mse)
    }
  }

  if (boot.lme == TRUE) {
    zx$qua.se.lme.boots <- hosking$qua.lme.se
  } else {
    zx$qua.se.lme.boots <- cov.lme$qua.lme.se
  }

  # ------- finalization ---------------------------
  if (bma != TRUE) {
    zx$zp <- zp
    zx$qua.ma <- zpf
    zx$w.ma <- wtgd
    zx$pick_xi <- kpar
    zx$run.numk <- numk
  }

  if (bma == TRUE) {
    zx$qua.bma <- zpf.bma
    zx$pen <- pen
    zx$w.bma <- bmaw

    set <- set.prior(pen = pen, numk = numk, xi_lme = hosking$lme[3],
                     kpar = kpar, weight = weight)
    if (pen == "norm") zx$prior_mu_std <- set$prior_mu_std
    if (pen == "beta") zx$p_q_beta <- set$p_q_beta

    zx$pick_xi <- kpar
    zx$run.numk <- numk
  }

  zx$run_kpar <- run
  zx$original.numk <- org.numk

  class(zx) <- "magev"
  return(zx)
}

# -----------------------------------------------------------------
# Building NS gev11 model using L-moments
# See the paper by Shin, Shin, and Park at JKSS (2025)
# -----------------------------------------------------------------

#' L-moment parameter estimation for the non-stationary GEV11 model
#'
#' @description
#' Estimates the parameters of the non-stationary GEV11 model
#' (mu(t) = mu0 + mu1*t, sigma(t) = exp(sigma0 + sigma1*t), constant xi)
#' by the pure L-moment method of Shin et al. (2025b, JKSS).
#' Starting from the WLS pre-estimate of \code{\link{strup.gev11}},
#' the three L-moment equations (matching the sample L-moments of the
#' Gumbel-transformed data with the population L-moments of the standard
#' Gumbel distribution) are solved by Broyden's method for
#' (mu0, sigma0, xi), and the best solution is chosen by a goodness-of-fit
#' measure.
#'
#' This function replaces \code{\link{gado.prop_11}} of GLmom v1.x
#' (renamed in v2.0.0, with slightly changed input/output).
#' It is also equivalent to \code{glme.gev11(xdat, pen="no")}.
#'
#' @param xdat A numeric vector of data to be fitted.
#' @param ntry Number of attempts (initial points) for the nonlinear solver
#'   (default 5).
#' @param ftol Function tolerance for the solver (default 1e-5).
#' @param show If TRUE, prints the precision and parameters of each try.
#'
#' @return A list containing:
#' \itemize{
#'   \item lme.gev11 - The L-moment estimates (mu0, mu1, sigma0, sigma1, xi).
#'     If no solution is found, the WLS pre-estimate is returned.
#'   \item precis - Precision (mean absolute residual of the L-moment
#'     equations) of the selected solution.
#'   \item data - The input data (used by the plot method).
#' }
#' The object has class \code{"lme11"}; see \code{\link{GLmom-methods}}.
#'
#' @references
#' Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
#' model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
#' \doi{10.1007/s42952-025-00325-3}
#'
#' @seealso \code{\link{glme.gev11}} for the penalized (GLME) version,
#'   \code{\link{strup.gev11}} for the WLS pre-estimation,
#'   \code{\link{GN16.gev11}} for the GN16 method,
#'   \code{\link{nsgev}} and \code{\link{gado.prop_11}} for the v1.x
#'   compatibility wrappers.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' \donttest{
#' data(PhliuAgromet)
#' result <- lme.gev11(PhliuAgromet$prec, ntry = 5)
#' print(result$lme.gev11)
#' }
#'
#' @export
lme.gev11 = function(xdat, ntry=5, ftol=1e-5, show=FALSE){

  z <- list()
  model="gev11"
  init=matrix(NA, ntry,3)

  pretheta= strup.gev11(xdat, init.rob=TRUE)$strup.mdfy

  init[1,]= pretheta[c(1,3,5)]+0.02
  if(ntry >= 2){
   sinit = init.glme(xdat, ntry=ntry, pretheta=pretheta,
                     model=model)
   init[2:ntry,]= sinit[1:(ntry-1),]
  }

  precis=rep(1000, ntry)
  para.sel=matrix(NA,ntry,ncol=5)

  for(i in 1:ntry) {

    kt <- try(
            nleqslv(x=as.vector(init[i,1:3]),
                    fn= fun.lme.gev11,
                    method="Broyden",
                    control=list(ftol=ftol, xtol=ftol*10),
                    xdat=xdat, pretheta=pretheta),
            silent=TRUE)

    if(inherits(kt, "try-error")) next   # precis stays 1000

    precis[i]= mean(abs(kt$fvec))
    if(is.na(precis[i])) precis[i]= 1000

    if( precis[i] < ftol) {
      para.sel[i,1:5]= c(kt$x[1], pretheta[2],
                         kt$x[2], pretheta[4], kt$x[3])
    }

    if( abs( kt$termcd ) > 3 ) {
      precis[i]=1000
      para.sel[i,]=NA
    }

    if(show==TRUE){
     cat("--- itry, precision=",i, precis[i],"\n")
     cat("--- itry, para=",i, round(para.sel[i,1:5],4),"\n","\n")
    }
  } #end for

  sel = sel.para_all(xdat, para.sel, model,
                                     obj.fun=abs(precis))
  z$lme.gev11= sel$para
  z$precis = precis[sel$min.itry]

  if(z$precis > ftol) { z$lme.gev11 = pretheta
    message("no solution found for the L-moment method; ",
            "WLS pre-estimate returned") }

  names(z$lme.gev11) <- c("mu0","mu1","sigma0","sigma1","xi")
  z$data = xdat
  class(z) = "lme11"
  return(z)
}

# MAGEV Plotting Functions
#
# Reference: Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with
# mixed criteria for estimating high quantiles of extreme values: Application
# to heavy rainfall. Stochastic Environmental Research and Risk Assessment,
# 40(2), 47. https://doi.org/10.1007/s00477-025-03167-x


#' K Sensitivity Plot for MAGEV
#'
#' @description
#' Plots return level estimates, standard errors, and first-order differences
#' across different numbers of candidate submodels K. This helps identify
#' stable regions where estimates converge and select an optimal K value.
#'
#' @param data A numeric vector of data to be fitted (e.g., annual maxima).
#' @param q.cut Quantile cutoff for determining stability (default 0.6).
#' @param mink Minimum number of candidate submodels to test (default 4).
#' @param maxk Maximum number of candidate submodels to test (default 20).
#' @param quant The probabilities for high quantile estimation.
#'   Default is c(0.99, 0.995).
#'
#' @details
#' The function computes MAGEV estimates for K ranging from \code{mink} to
#' \code{maxk}. For each K, it calculates:
#' \itemize{
#'   \item Return level estimates (black points)
#'   \item Normalized standard errors (blue line)
#'   \item First-order differences (red line with triangles)
#' }
#'
#' The optimal K is selected as the smallest value where both the normalized
#' standard error and first-order difference are below their respective
#' \code{q.cut} quantile cutoffs. The selected K is indicated by a purple
#' vertical line.
#'
#' @return The optimal K value (integer) selected by the algorithm.
#'
#' @references
#' Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
#' for estimating high quantiles of extreme values: Application to heavy rainfall.
#' \emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
#' \doi{10.1007/s00477-025-03167-x}
#'
#' @seealso \code{\link{ma.gev}} for the main model averaging function.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' \donttest{
#' data(haenam)
#' optimal_k <- magev.ksensplot(haenam$X1)
#' print(optimal_k)
#' }
#'
#' @export
magev.ksensplot <- function(data = NULL, q.cut = 0.6, mink = 4,
                            maxk = 20, quant = c(0.99, 0.995)) {

  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar))

  totk <- maxk - mink + 1
  test <- list()
  numq <- length(quant)
  diff <- rt <- se <- matrix(NA, maxk, numq)

  qq <- quant
  id12 <- seq(1, numq)

  bma <- FALSE
  weight <- "like1"
  start <- "mle"

  for (k in mink:maxk) {

    test[[k]] <- ma.gev(data = data, quant = qq, weight = weight,
                        numk = k, B = 200, varcom = TRUE, trim = 0,
                        fig = FALSE, bma = bma, pen = "norm", remle = FALSE)

    if (bma != TRUE) {
      rt[k, id12] <- test[[k]]$qua.ma[id12]
      se[k, id12] <- test[[k]]$ranw.se.ma[id12]
    } else if (bma == TRUE) {
      rt[k, id12] <- test[[k]]$qua.bma[id12]
      se[k, id12] <- test[[k]]$pred.se.bma[id12]
    }

    message("k = ", k, ", zp.ma = ", paste(round(rt[k, id12], 2), collapse = ", "),
            ", se = ", paste(round(se[k, id12], 2), collapse = ", "))

  } # end for k

  tr.diff <- rep(NA, maxk)
  for (i in (mink + 1):(maxk - 1)) {
    diff[i, ] <- abs(rt[i + 1, ] - rt[i, ]) + abs(rt[i, ] - rt[i - 1, ])
  }

  nina <- which(!is.na(rt[, 1]))
  U <- (se[, 1] - min(se[, 1], na.rm = TRUE)) /
       (max(se[, 1], na.rm = TRUE) - min(se[, 1], na.rm = TRUE))
  tr.se <- U * (max(rt[, 1], na.rm = TRUE) - min(rt[, 1], na.rm = TRUE)) +
           min(rt[, 1], na.rm = TRUE)

  dina <- which(!is.na(diff[, 1]))

  Ub <- (diff[dina, 1] - min(diff[dina, 1], na.rm = TRUE)) /
        (max(diff[dina, 1], na.rm = TRUE) - min(diff[dina, 1], na.rm = TRUE))
  tr.diff[(mink + 1):(maxk - 1)] <- Ub * (max(rt[, 1], na.rm = TRUE) -
                                          min(rt[, 1], na.rm = TRUE)) +
                                    min(rt[, 1], na.rm = TRUE)

  diff.cut <- stats::quantile(tr.diff[dina], probs = q.cut, na.rm = TRUE)
  id <- which(tr.diff <= diff.cut)
  se.cut <- stats::quantile(tr.se[nina], probs = q.cut, na.rm = TRUE)

  tid <- which(tr.se <= se.cut)
  jd <- sort(intersect(id, tid))
  selTk <- min(jd)

  max.selTk <- min(maxk, selTk + 2)
  min.selTk <- max(mink, selTk - 2)
  iid <- intersect(jd, seq(min.selTk, max.selTk))
  star <- iid[which.min(se[iid, 1])]

  my <- min(rt[, 1], na.rm = TRUE) * 0.997
  My <- max(rt[, 1], na.rm = TRUE) * 1.003
  graphics::plot(seq(mink, maxk), rt[nina, 1], xlim = c(mink - 2, maxk),
                 ylim = c(my, My),
                 xlab = "K", ylab = "return level", pch = 16, cex = 1)

  graphics::axis(side = 1, at = seq(mink - 1, maxk, 1))

  graphics::par(new = TRUE)
  graphics::plot(seq(mink, maxk), tr.se[nina], xlim = c(mink - 2, maxk),
                 ylim = c(my, My),
                 type = "l", xlab = "K", ylab = "return level", col = "blue")
  graphics::points(seq(mink, maxk), tr.se[nina], col = "blue")

  graphics::par(new = TRUE)
  graphics::plot(seq(mink + 1, maxk - 1), tr.diff[(mink + 1):(maxk - 1)],
                 xlim = c(mink - 2, maxk), ylim = c(my, My),
                 type = "l", col = "red",
                 xlab = "K", ylab = "return level")
  graphics::points(seq(mink + 1, maxk - 1), tr.diff[(mink + 1):(maxk - 1)],
                   col = "red", pch = 2, cex = 0.8)

  graphics::abline(h = diff.cut, lty = 3, lwd = 2, col = "darkorange")
  graphics::abline(h = se.cut, lty = 3, lwd = 2, col = "darkgreen")
  graphics::abline(v = star, col = "purple")

  graphics::text(mink - 1.3, se.cut * 1.0012, "q.6(SE_K)", cex = 0.8, col = "darkgreen")
  graphics::text(mink - 1.5, diff.cut * 1.0012, "q.6(d_K)", col = "darkorange", cex = 0.8)

  return(star)
}


#' Q-Q Diagnostic Plot for MAGEV
#'
#' @description
#' Creates a 2x2 panel of Q-Q plots comparing observed vs. fitted quantiles
#' for different estimation methods: MLE, LME, surrogate (MA), and REMLE.
#'
#' @param data A numeric vector of observed data.
#' @param zx A list object returned by \code{\link{ma.gev}} with \code{remle = TRUE}.
#'
#' @details
#' The function creates four Q-Q plots:
#' \itemize{
#'   \item Upper left: MLE (Maximum Likelihood Estimation)
#'   \item Upper right: LME (L-moment Estimation)
#'   \item Lower left: Surrogate MA (Model Averaging surrogate)
#'   \item Lower right: REMLE (Restricted MLE, if available)
#' }
#'
#' Points close to the 45-degree diagonal line indicate good model fit.
#'
#' @return NULL. The function produces a plot as a side effect.
#'
#' @references
#' Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
#' for estimating high quantiles of extreme values: Application to heavy rainfall.
#' \emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
#' \doi{10.1007/s00477-025-03167-x}
#'
#' @seealso \code{\link{ma.gev}} for the main model averaging function,
#'   \code{\link{magev.rlplot}} for return level plots.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' data(haenam)
#' qq <- c(seq(0.01, 0.99, by = 0.01), 0.995, 0.999)
#' zx <- ma.gev(haenam$X1, quant = qq, weight = 'like1',
#'              numk = 9, varcom = FALSE, remle = TRUE)
#' magev.qqplot(data = haenam$X1, zx = zx)
#'
#' @export
magev.qqplot <- function(data = NULL, zx = NULL) {

  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar))

  dat <- data
  qorg <- ((1:length(dat)) - 0.35) / (length(dat))

  graphics::par(mfrow = c(2, 2))
  minx <- min(dat) * 0.95

  maxp <- lmomco::quagev(max(qorg), lmomco::vec2par(zx$mle.hosking, 'gev'))
  maxb <- lmomco::quagev(max(qorg), lmomco::vec2par(zx$surr$par, 'gev'))
  maxx <- max(dat, maxp, maxb) * 1.2

  graphics::plot(lmomco::quagev(qorg, lmomco::vec2par(zx$mle.hosking, 'gev')),
                 sort(dat),
                 ylab = "Empirical", xlab = "model GEV", main = "mle",
                 xlim = c(minx, maxx), ylim = c(minx, maxx), pch = 16)
  graphics::abline(0, 1, col = 4)

  graphics::plot(lmomco::quagev(qorg, lmomco::vec2par(zx$lme, 'gev')),
                 sort(dat),
                 ylab = "Empirical", xlab = "model GEV", main = "lme",
                 xlim = c(minx, maxx), ylim = c(minx, maxx), pch = 16)
  graphics::abline(0, 1, col = 4)

  graphics::plot(lmomco::quagev(qorg, lmomco::vec2par(zx$surr$par, 'gev')),
                 sort(dat),
                 ylab = "Empirical", xlab = "model GEV", main = "surr.ma",
                 xlim = c(minx, maxx), ylim = c(minx, maxx), pch = 16)
  graphics::abline(0, 1, col = 4)

  if (!is.null(zx$remle1)) {
    graphics::plot(lmomco::quagev(qorg, lmomco::vec2par(zx$remle1, 'gev')),
                   sort(dat),
                   ylab = "Empirical", xlab = "model GEV", main = "remle1",
                   xlim = c(minx, maxx), ylim = c(minx, maxx), pch = 16)
    graphics::abline(0, 1, col = 4)
  }

  invisible(NULL)
}


#' Return Level Plot for MAGEV
#'
#' @description
#' Displays fitted return levels with 95% confidence intervals against
#' return period on a log scale.
#'
#' @param par A numeric vector of GEV parameters (mu, sigma, xi) in Hosking style.
#'   Typically from \code{zx$surr$par} where \code{zx} is the output of
#'   \code{\link{ma.gev}}.
#' @param se.vec A numeric vector of standard errors for the quantile estimates
#'   corresponding to the plotting positions. Typically from \code{zx$ranw.se.ma}.
#' @param data A numeric vector of observed data (annual maxima).
#'
#' @details
#' The plot shows:
#' \itemize{
#'   \item Black line: Fitted return level curve
#'   \item Blue lines: 95% confidence interval (mean +/- 1.96 * SE)
#'   \item Black points: Observed data at empirical return periods
#' }
#'
#' The x-axis (return period) is on a log scale, ranging from 0.1 to 900 years.
#'
#' @return NULL. The function produces a plot as a side effect.
#'
#' @references
#' Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
#' for estimating high quantiles of extreme values: Application to heavy rainfall.
#' \emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
#' \doi{10.1007/s00477-025-03167-x}
#'
#' @seealso \code{\link{ma.gev}} for the main model averaging function,
#'   \code{\link{magev.qqplot}} for Q-Q diagnostic plots.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' data(haenam)
#' ff <- c(seq(0.01, 0.09, by = 0.01), 0.1, 0.2, 0.3, 0.4, 0.5,
#'         0.6, 0.7, 0.8, 0.9, 0.93, 0.95, 0.98, 0.99,
#'         0.993, 0.995, 0.998, 0.999)
#' zx <- ma.gev(haenam$X1, quant = ff, weight = 'like1',
#'              numk = 9, varcom = TRUE)
#' magev.rlplot(par = zx$surr$par, se.vec = zx$ranw.se.ma, data = haenam$X1)
#'
#' @export
magev.rlplot <- function(par = NULL, se.vec = NULL, data = NULL) {

  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar))

  a <- par  # a[3] is Hosking-style xi
  f <- c(seq(0.01, 0.09, by = 0.01), 0.1, 0.2, 0.3, 0.4, 0.5,
         0.6, 0.7, 0.8, 0.9, 0.93, 0.95, 0.98, 0.99,
         0.993, 0.995, 0.998, 0.999)
  q <- lmomco::quagev(f, lmomco::vec2par(a, 'gev'))

  v <- se.vec  # variance vector for f-quantiles
  maxq <- max(data) * 2.0
  ylima <- c(min(data, q), maxq)

  graphics::plot(-1 / log(f), q, log = "x", type = "n", xlim = c(0.1, 900),
                 ylim = ylima, xlab = "Return Period", ylab = "Return Level")

  qorg <- ((1:length(data)) - 0.35) / (length(data))

  graphics::lines(-1 / log(f), q)
  graphics::lines(-1 / log(f), q + 1.96 * v, col = 4)
  graphics::lines(-1 / log(f), q - 1.96 * v, col = 4)

  graphics::points(-1 / log(qorg), sort(data), pch = 16)

  invisible(NULL)
}

# Penalty (preference) functions for the GEV shape parameter
# Shared by the stationary (glme.gev) and non-stationary (glme.gev11) methods.

#' Beta function integrand for penalty calculation
#'
#' @param w Integration variable.
#' @param al Lower bound.
#' @param bl Upper bound.
#' @param p Shape parameter p.
#' @param q Shape parameter q.
#' @return Integrand value.
#' @keywords internal
Befun <- function(w, al=al, bl=bl, p=p, q=q) {
  ((-al+w)^(p-1)) * ((bl-w)^(q-1))
}


#' Martins-Stedinger prior function
#'
#' @description
#' Computes the Martins-Stedinger Beta(6,9) prior probability for the GEV
#' shape parameter on the interval \eqn{[-0.5, 0.5]}.
#'
#' @param para A vector of GEV parameters (location, scale, shape).
#' @param p Shape parameter for beta distribution (default 6).
#' @param q Shape parameter for beta distribution (default 9).
#' @return Prior probability value (scalar).
#'
#' @references
#' Martins, E. S. & Stedinger, J. R. (2000). Generalized maximum-likelihood
#' generalized extreme-value quantile estimators for hydrologic data.
#' Water Resources Research, 36(3), 737-744.
#' \doi{10.1029/1999WR900330}
#'
#' @seealso \code{\link{pk.beta}} for the adaptive beta penalty,
#'   \code{\link{glme.gev}} which uses these penalty functions.
#'
#' @examples
#' # Evaluate MS prior at xi = -0.2
#' MS_pk(para = c(100, 20, -0.2))
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#' @export
MS_pk = function(para=para, p=6, q=9){

  Bef <- function(x) { ((0.5+x)^(p-1)) * ((0.5-x)^(q-1)) }
  Be  <- integrate(Bef, lower=-0.5, upper=0.5)[1]$value

  if( abs(para[3]) < 0.5 ){
    pk.ms <- ((0.5+para[3])^(p-1))*((0.5-para[3])^(q-1))/ Be
  }else if ( abs(para[3]) >= 0.5 ) {
    pk.ms = 1e-20
  }

  pk.ms
}


#' Normal preference function for shape parameter
#'
#' @description
#' Computes a normal distribution-based preference (penalty) function value
#' for the GEV shape parameter. The alias \code{new_pf_norm} is provided
#' for backward compatibility.
#'
#' @param para A vector of GEV parameters.
#' @param mu Mean for normal distribution.
#' @param std Standard deviation for normal distribution.
#' @return Preference function value (scalar).
#'
#' @references
#' Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
#' L-moment estimation for stationary and nonstationary extreme value models.
#' arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
#'
#' @seealso \code{\link{pk.beta}} for the beta penalty,
#'   \code{\link{MS_pk}} for the Martins-Stedinger penalty,
#'   \code{\link{glme.gev}} which uses these penalty functions.
#'
#' @examples
#' # Normal preference with mean=-0.5, sd=0.2 at xi=-0.2
#' pk.norm.stnary(para = c(100, 20, -0.2), mu = -0.5, std = 0.2)
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#' @export
pk.norm.stnary = function(para=NULL, mu=NULL, std=NULL){
  1 + dnorm(para[3], mean= mu, sd=std)
}

#' @rdname pk.norm.stnary
#' @export
new_pf_norm = pk.norm.stnary


#' Beta preference function for the GEV shape parameter
#'
#' @description
#' Computes the data-adaptive Beta preference (penalty) function for the GEV
#' shape parameter, used by both the stationary (\code{\link{glme.gev}}) and
#' the non-stationary (\code{\link{glme.gev11}}) GLME methods (v2.0.0 unified
#' implementation).
#'
#' When \code{q} is \code{NULL} (adaptive mode), the support is
#' \eqn{(\max(-1, \hat\xi - c_0),\; \min(0.3, \hat\xi + c_0))} where
#' \eqn{\hat\xi} is \code{lme.center[3]}, and
#' \eqn{q = p + \min(|\hat\xi| c_1, c_2)} for \eqn{\hat\xi \le -0.05}
#' (otherwise \eqn{q = p}). When \code{q} is given (fixed mode), the support
#' is \code{(min.xi, max.xi)}; this mode is used for the fixed literature
#' penalties ("ms", "park", "cannon").
#'
#' \code{pk.beta.stnary} is kept as a backward-compatible alias. Note that
#' since v2.0.0 both names use this unified implementation (the v1.x default
#' hyperparameters \code{c0=0.3, c2=5} changed to \code{c0=0.35, c2=2}, and
#' out-of-support values now return \code{1e-100} instead of \code{1}).
#'
#' @param para A vector of GEV parameters; only \code{para[3]} (shape) is used.
#' @param lme.center L-moment estimates used as the center (adaptive mode).
#' @param p First shape parameter of the beta distribution.
#' @param q Second shape parameter (optional; if provided, fixed mode is used).
#' @param min.xi Lower support limit in fixed mode (default -1).
#' @param max.xi Upper support limit in fixed mode (default 0.5).
#' @param c0 Half-width of the adaptive support (default 0.35).
#' @param c1 Scaling hyperparameter for q (default 10).
#' @param c2 Upper limit hyperparameter for q (default 2).
#' @return A list containing:
#' \describe{
#'   \item{pk.one}{Preference function value (scalar)}
#'   \item{p}{Shape parameter p used}
#'   \item{q}{Shape parameter q used}
#' }
#'
#' @references
#' Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
#' L-moment estimation for stationary and nonstationary extreme value models.
#' arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
#'
#' @seealso \code{\link{pk.norm.stnary}} for the normal penalty,
#'   \code{\link{MS_pk}} for the Martins-Stedinger penalty,
#'   \code{\link{glme.gev}}, \code{\link{glme.gev11}}.
#'
#' @examples
#' # Adaptive beta preference for xi = -0.2 centered at LME xi = -0.15
#' pk.beta(para = c(100, 20, -0.2),
#'         lme.center = c(100, 20, -0.15), p = 6, c1 = 3, c2 = 1)
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#' @export
pk.beta = function(para=NULL, lme.center=NULL, p=NULL,
                   q=NULL, min.xi=-1.0, max.xi=0.5,
                   c0=0.35, c1=10, c2=2){

  pk=list()

  if(is.null(q)){

    aa= max(-1, lme.center[3]-c0)
    bb= min(0.3, lme.center[3]+c0)
    al=min(aa,bb)
    bl=max(aa,bb)

    if(lme.center[3] <= -0.05) {
      cut= min(lme.center[3], -0.05)
      qlim= min(abs(cut)*c1, c2)
    }else{ qlim =0.0 }

    q=p+qlim

  }else if(!is.null(q)){
    lme.center= c(0,0,0)
    al= min.xi; bl=max.xi
  }

  Be  <- integrate(Befun, lower=al, upper=bl,
                   al=al, bl=bl, p=p, q=q)[1]$value

  pk$pk.one=1e-100
  if(lme.center[3] < 0.5){
    if( (para[3] > al) & (para[3] < bl) ) {
      pk$pk.one <- ((-al+para[3])^(p-1))*((bl-para[3])^(q-1))/ Be
    }else{
      pk$pk.one = 1e-100
    }
  }
  if(is.na(pk$pk.one)) pk$pk.one=1e-100
  pk$p=p
  pk$q=q
  return(pk)
}

#' @rdname pk.beta
#' @export
pk.beta.stnary = pk.beta


#' Validate and resolve penalty settings
#'
#' @description
#' Normalizes the penalty name (e.g., "Beta" to "beta") and, when
#' \code{pen.choice} is given, resolves the preset hyperparameters:
#' for \code{pen="beta"}, choices 1-6 give (p, c1, c2) =
#' (6,3,1), (6,5,2), (6,7,3), (2,3,0.5), (2,5,1), (2,7,1.5);
#' for \code{pen="norm"}, choices 1-4 give (mu, std) =
#' (-0.5,0.25), (-0.5,0.15), (-0.6,0.25), (-0.6,0.15).
#' These presets follow Table 1 and Eq. (15) of Shin et al. (2025).
#'
#' @param pen Penalty name (character).
#' @param pen.choice Preset number or NULL.
#' @param p,c1,c2 Beta penalty hyperparameters (used when pen.choice is NULL).
#' @param mu,std Normal penalty hyperparameters (used when pen.choice is NULL).
#' @return A list with elements pen, p, c1, c2, mu, std.
#' @keywords internal
check.penalty = function(pen=NULL,pen.choice=NULL,p=NULL,
                         c1=NULL,c2=NULL,mu=NULL,std=NULL){

  if(is.null(pen) | !is.character(pen)) {
    stop("pen should be given as a character")}
  if(pen=="Beta" ) pen="beta"  ;  if(pen=="CD") pen="cd"
  if(pen=="MS") pen="ms"       ;  if(pen=="Park") pen="park"
  if(pen=="normal") pen="norm" ;  if(pen=="Cannon") pen="cannon"

  if(pen=='beta' & is.null(pen.choice)){
    if(is.null(p) | is.null(c1) | is.null(c2)){
      stop("if pen.choice is null, p, c1, c2 should be specified")}
  }
  if(pen=='norm' & is.null(pen.choice)){
    if(is.null(mu) | is.null(std)){
      stop("if pen.choice is null, mu, std should be specified")}
  }

  if(pen=='beta' & !is.null(pen.choice)){
    if(pen.choice %% 1 != 0 | pen.choice < 1 | pen.choice > 6 ){
      stop("pen.choice for beta should be an integer: 1~6")
    }
    pc1c2= matrix(c(6,6,6, 2,2,2, 3,5,7, 3,5,7,
                    1,2,3, 0.5,1,1.5), 6,3, byrow=FALSE)
    p= pc1c2[pen.choice,1]
    c1=pc1c2[pen.choice,2]
    c2=pc1c2[pen.choice,3]
  }
  if(pen=='norm' & !is.null(pen.choice)){
    if(pen.choice %% 1 != 0 | pen.choice < 1 | pen.choice > 4 ){
      stop("pen.choice for norm should be an integer: 1~4")
    }
    mustd= matrix(c(-.5,.25, -.5,.15,
                    -.6,.25, -.6,.15), 4,2, byrow=TRUE)
    mu= mustd[pen.choice,1]
    std=mustd[pen.choice,2]
  }
  result=list(pen=pen, p=p, c1=c1, c2=c2, mu=mu, std=std)
  return(result)
}


#' Penalty dispatcher (negative log prior)
#'
#' @description
#' Returns the negative log of the preference (prior) function for the
#' selected penalty type. Dispatches to \code{\link{pk.norm.stnary}}
#' ("norm"), the adaptive \code{\link{pk.beta}} ("beta"), the fixed
#' literature beta penalties ("ms": Beta(6,9); "park": Beta(2.5,2.5);
#' "cannon": Beta(2,3.3)), the Coles-Dixon exponential penalty ("cd"),
#' or zero ("no").
#'
#' @param par Parameter vector; only \code{par[3]} (shape) is penalized.
#' @param mu,std Hyperparameters for the normal penalty.
#' @param lme Center (L-moment pre-estimate) for the adaptive beta penalty.
#' @param pen Penalty type.
#' @param p,c0,c1,c2,q Hyperparameters for the beta penalty.
#' @return Negative log prior value (scalar).
#' @keywords internal
penalty.fun = function(par, mu=NULL, std=NULL, lme=NULL, pen='beta',
                       p=NULL, c0=0.35, c1=NULL, c2=NULL, q=NULL){

  if(pen=='norm' | pen=="normal"){

    pk = -log( pk.norm.stnary(para=par, mu=mu, std=std) )

  }else if(pen=='beta' | pen=="Beta"){

    pk = -log( pk.beta(para=par, lme.center=lme, p=p, q=q,
                       c0=c0, c1=c1, c2=c2)$pk.one )

  }else if(pen=='ms' | pen=="MS"){

    pk = -log( pk.beta(para=par, p=6, q=9)$pk.one )

  }else if(pen=="park" | pen=="Park"){

    pk = -log( pk.beta(para=par, p=2.5, q=2.5)$pk.one )

  }else if(pen=="cannon" | pen=="Cannon"){

    pk = -log( pk.beta(para=par, p=2, q=3.3)$pk.one )

  }else if(pen=="cd" | pen=="CD"){

    if (par[3] >= 0) { pk = 0
    }else if (par[3] > -1 & par[3] < 0) {
      pk = (1/(1 + par[3])) - 1
    }else{ pk = 10^6 }

  }else if(pen=='no'){
    pk = 0
  }else{
    stop("unknown penalty type: ", pen)
  }
  pk
}

# S3 methods (print, summary, plot) for the fitted objects returned by
# glme.gev() ["glme"], glme.gev11() ["glme11"], lme.gev11() ["lme11"],
# and ma.gev() ["magev"]. All methods return their first argument invisibly.

# one-line description of the penalty settings of a fit -------------------
penalty.label <- function(x) {
  pen <- x$pen
  if (is.null(pen)) return("Penalty: unknown")
  choice <- if (length(x$pen_pen.choice) > 1) x$pen_pen.choice[2] else "custom"
  if (pen == "beta") {
    sprintf("Penalty: adaptive beta (choice %s), p = %.4g, q = %.4g",
            choice, x$p_q[1], x$p_q[2])
  } else if (pen == "norm") {
    sprintf("Penalty: adaptive normal (choice %s), mu = %.4g, sd = %.4g",
            choice, x$mu_std[1], x$mu_std[2])
  } else if (pen == "no") {
    "Penalty: none (pure L-moment fit)"
  } else {
    sprintf("Penalty: %s (fixed)", pen)
  }
}

convergence.label <- function(x) {
  if (is.null(x$convergence)) return("")
  if (identical(unname(x$convergence), 0)) "(converged)" else "(NOT converged)"
}

# Gumbel-scale Q-Q plot shared by the GEV11 fits --------------------------
qq.gum01 <- function(xdat, para, main, ...) {
  z <- trans.gum01(xdat, para = para, na.ok = TRUE)
  z <- sort(z[!is.na(z)])
  th <- -log(-log(ppoints(length(z))))
  plot(th, z, xlab = "Standard Gumbel quantiles",
       ylab = "Transformed observations", main = main, ...)
  abline(0, 1, lty = 2)
  invisible(NULL)
}

#' Methods for fitted GLmom objects
#'
#' @description
#' The estimation functions return classed objects: \code{\link{glme.gev}}
#' returns an object of class \code{"glme"}, \code{\link{glme.gev11}} of
#' class \code{"glme11"}, \code{\link{lme.gev11}} of class \code{"lme11"},
#' and \code{\link{ma.gev}} of class \code{"magev"}. The objects remain
#' ordinary lists, so all documented fields (e.g., \code{$para.glme}) stay
#' accessible as before.
#'
#' \code{print} shows the parameter estimates and the penalty settings;
#' \code{summary} additionally reports the auxiliary estimates stored in
#' the object; \code{plot} draws a quantile-quantile diagnostic: against
#' the fitted GEV distribution for \code{"glme"}, on the standard Gumbel
#' scale (after the transformation of Eq. 8 in the manuscript) for
#' \code{"glme11"} and \code{"lme11"}, and via \code{\link{magev.qqplot}}
#' for \code{"magev"}.
#'
#' @param x,object A fitted object.
#' @param ... Further arguments passed to plotting functions; ignored by
#'   \code{print} and \code{summary}.
#' @return \code{x} (or \code{object}), invisibly.
#'
#' @examples
#' data(haenam)
#' fit <- glme.gev(haenam$X1)
#' fit
#' summary(fit)
#' plot(fit)
#'
#' @name GLmom-methods
NULL

#' @rdname GLmom-methods
#' @export
print.glme <- function(x, ...) {
  cat("Generalized L-moment fit of the stationary GEV distribution\n")
  cat(penalty.label(x), "\n\n")
  cat("Estimates (para.glme):\n")
  print(round(x$para.glme, 4))
  cat("\nL-moment estimates (para.lme):\n")
  print(round(x$para.lme, 4))
  if (!is.null(x$nllh.glme))
    cat("\nPenalized criterion:", round(x$nllh.glme, 4),
        convergence.label(x), "\n")
  invisible(x)
}

#' @rdname GLmom-methods
#' @export
summary.glme <- function(object, ...) {
  print(object)
  if (!is.null(object$c0_c1_c2))
    cat("Hyperparameters (c0, c1, c2):",
        paste(object$c0_c1_c2, collapse = ", "), "\n")
  cat("Sample size:", length(object$data), "\n")
  invisible(object)
}

#' @rdname GLmom-methods
#' @export
plot.glme <- function(x, ...) {
  n <- length(x$data)
  th <- quagev(ppoints(n), vec2par(x$para.glme, "gev"))
  plot(th, sort(x$data), xlab = "Fitted GEV quantiles",
       ylab = "Empirical quantiles", main = "Q-Q plot of the GLME fit", ...)
  abline(0, 1, lty = 2)
  invisible(x)
}

#' @rdname GLmom-methods
#' @export
print.glme11 <- function(x, ...) {
  cat("Generalized L-moment fit of the nonstationary GEV11 model\n")
  cat("  mu(t) = mu0 + mu1 t,  sigma(t) = exp(sigma0 + sigma1 t)\n")
  cat(penalty.label(x), "\n\n")
  cat("Estimates (para.glme; mu1 and sigma1 held at the WLS pre-estimates):\n")
  print(round(x$para.glme, 4))
  if (!is.null(x$nllh.glme))
    cat("\nPenalized criterion:", round(x$nllh.glme, 4),
        convergence.label(x), "\n")
  if (!is.null(x$precis))
    cat("Precision of the L-moment equations:", signif(x$precis, 3), "\n")
  invisible(x)
}

#' @rdname GLmom-methods
#' @export
summary.glme11 <- function(object, ...) {
  print(object)
  cat("\nAuxiliary estimates:\n")
  aux <- rbind(WLS = object$para.wls, "WLS (original)" = object$strup.org)
  if (!is.null(object$para.gado)) aux <- rbind(aux, GN16 = object$para.gado)
  print(round(aux, 4))
  cat("\nStationary L-moment estimates (lme.sta):\n")
  print(round(object$lme.sta, 4))
  invisible(object)
}

#' @rdname GLmom-methods
#' @export
plot.glme11 <- function(x, ...) {
  qq.gum01(x$data, x$para.glme,
           main = "Gumbel-scale Q-Q plot of the GEV11 fit", ...)
  invisible(x)
}

#' @rdname GLmom-methods
#' @export
print.lme11 <- function(x, ...) {
  cat("L-moment fit of the nonstationary GEV11 model (Shin et al. 2025b)\n")
  cat("  mu(t) = mu0 + mu1 t,  sigma(t) = exp(sigma0 + sigma1 t)\n\n")
  cat("Estimates (lme.gev11):\n")
  print(round(x$lme.gev11, 4))
  cat("\nPrecision of the L-moment equations:", signif(x$precis, 3), "\n")
  invisible(x)
}

#' @rdname GLmom-methods
#' @export
summary.lme11 <- function(object, ...) {
  print(object)
  cat("Sample size:", length(object$data), "\n")
  invisible(object)
}

#' @rdname GLmom-methods
#' @export
plot.lme11 <- function(x, ...) {
  qq.gum01(x$data, x$lme.gev11,
           main = "Gumbel-scale Q-Q plot of the GEV11 fit", ...)
  invisible(x)
}

#' @rdname GLmom-methods
#' @export
print.magev <- function(x, ...) {
  cat("Model-averaged GEV quantile estimates\n")
  cat(sprintf("Weights: '%s' over %d candidate shape parameters\n\n",
              x$weight, x$run.numk))
  est <- rbind(MLE = x$qua.mle, LME = x$qua.lme, "Model average" = x$qua.ma)
  colnames(est) <- paste0("q", x$quant)
  print(round(est, 2))
  if (!is.null(x$fixw.se.ma)) {
    cat("\nSE of the model average (fixed weights):\n")
    print(round(x$fixw.se.ma, 2))
  }
  invisible(x)
}

#' @rdname GLmom-methods
#' @export
summary.magev <- function(object, ...) {
  print(object)
  cat("\nCandidate shape parameters (pick_xi):\n")
  print(round(object$pick_xi, 4))
  cat("\nWeights (w.ma):\n")
  print(round(object$w.ma, 4))
  invisible(object)
}

#' @rdname GLmom-methods
#' @export
plot.magev <- function(x, ...) {
  magev.qqplot(data = x$data, zx = x)
  invisible(x)
}

# R code for Non-Stationary GEV11 model using generalized L-moments
# GEV11: mu(t) = mu0 + mu1*t, sigma(t) = exp(sigma0 + sigma1*t), xi constant
# v2.0.0: rewritten following the revised code of Shin et al. (Apr 2026)

#----------------------------------------------------------------------
#' Generalized L-moments estimation for non-stationary GEV11 model
#'
#' @description
#' This function estimates parameters of the non-stationary GEV11 model
#' where mu(t) = mu0 + mu1*t and sigma(t) = exp(sigma0 + sigma1*t),
#' by the generalized method of L-moment estimation (GLME).
#'
#' Starting from the WLS pre-estimate of \code{\link{strup.gev11}},
#' the penalized objective (generalized L-moment distance based on the
#' asymptotic covariance of the sample L-moments, plus penalty) is
#' minimized over (mu0, sigma0, xi) with mu1 and sigma1 held at their
#' pre-estimated values. With \code{pen="no"} the result equals the pure
#' L-moment method \code{\link{lme.gev11}} of Shin et al. (2025b).
#'
#' @param xdat A numeric vector of data to be fitted.
#' @param ntry Number of attempts for parameter estimation (default 5).
#' @param ftol Tolerance for convergence (default 1e-6).
#' @param opt.choose Selection criterion among the multi-start solutions:
#'   "nllh" (default, penalized negative log-likelihood) or "gof"
#'   (goodness-of-fit by the number-of-exceedances measure).
#' @param pen Type of penalty function: "norm", "beta" (default), "ms",
#'   "park", "cannon", "cd", or "no".
#' @param pen.choice Choice number for penalty hyperparameters (default 1).
#'   See \code{\link{glme.gev}} for the preset values. Set to NULL to use
#'   \code{p, c1, c2} (beta) or \code{mu, std} (norm) directly.
#' @param mu Mean for normal penalty (default -0.55).
#' @param std Std for normal penalty (default 0.2).
#' @param p Shape for beta penalty (default 6).
#' @param c1 Scaling for beta penalty (default 5).
#' @param c2 Limit for beta penalty (default 2).
#' @param c0 Half-width of the adaptive beta penalty support (default 0.35).
#' @param q Optional fixed second shape parameter for the beta penalty.
#' @param show If TRUE, prints the objective value of each try.
#' @param init.rob Use robust regression for the WLS initialization
#'   (default TRUE).
#' @param glme.pre Deprecated (ignored). The WLS pre-estimate is always used.
#'
#' @return A list containing:
#' \itemize{
#'   \item para.glme - Proposed GLME estimates (mu0, mu1, sigma0, sigma1, xi).
#'   \item nllh.glme - Penalized negative log-likelihood of the solution.
#'   \item convergence - 0 if converged, 5 otherwise.
#'   \item pen, pen_pen.choice - Penalty method (and choice) used.
#'   \item p_q, c1_c2, c0_c1_c2 - (for beta) hyperparameters used.
#'   \item mu_std - (for norm) hyperparameters used.
#'   \item strup.org - WLSE by the Strupczewski method (\code{strup.para}).
#'   \item para.wls - Modified WLS estimates (\code{strup.mdfy}).
#'   \item para.gado - GN16 method estimates.
#'   \item lme.sta - Stationary L-moment estimates.
#'   \item para.lme - Pure L-moment estimates; returned only when
#'     \code{pen="no"} (then identical to para.glme). For other penalties
#'     use \code{\link{lme.gev11}} directly.
#'   \item precis - (only when \code{pen="no"}) precision of the L-moment
#'     equation solution.
#'   \item data - The input data (used by the plot method).
#' }
#' The object has class \code{"glme11"}; see \code{\link{GLmom-methods}}.
#'
#' @references
#' Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
#' L-moment estimation for stationary and nonstationary extreme value models.
#' arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
#'
#' Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
#' model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
#' \doi{10.1007/s42952-025-00325-3}
#'
#' @seealso \code{\link{glme.gev}} for stationary GEV estimation,
#'   \code{\link{lme.gev11}} for the pure L-moment method (no penalty),
#'   \code{\link{strup.gev11}} for the WLS pre-estimation,
#'   \code{\link{GN16.gev11}} for the GN16 method,
#'   \code{\link{quagev.NS}} for non-stationary quantile computation.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' data(PhliuAgromet)
#' x <- PhliuAgromet$prec
#'
#' \donttest{
#' # Estimate non-stationary GEV11 parameters
#' result <- glme.gev11(x, ntry = 5)
#' print(result$para.glme)  # Proposed GLME estimates
#' }
#'
#' @export
glme.gev11= function(xdat, ntry=5, ftol=1e-6,
                     opt.choose="nllh", pen='beta', pen.choice=1,
                     mu= -0.55, std=0.2, p=6, c1=5, c2=2,
                     c0=0.35, q=NULL, show=FALSE,
                     init.rob=TRUE, glme.pre=NULL){

  if(!is.null(glme.pre)){
    warning("'glme.pre' is deprecated and ignored; ",
            "the WLS (Strupczewski) pre-estimate is always used.")
  }

  z <- list()
  model='gev11'
  name_gev11_ns  =c("mu0","mu1","sigma0","sigma1","xi")
  name_gev00_sta =c("mu_sta","sigma_sta","xi_sta")

  ch= check.penalty(pen=pen, pen.choice=pen.choice, p=p,
                   c1=c1,c2=c2,mu=mu,std=std)
  pen= ch$pen
  if(!is.null(pen.choice)){
   p= ch$p ; c1 =ch$c1;  c2= ch$c2
   mu= ch$mu ; std= ch$std
  }

#---- proposed method with pen="no" or "norm" or "beta" or etc.--
# z$para.glme = proposed glme

  strup = strup.gev11(xdat, init.rob=init.rob)
  pretheta= strup$strup.mdfy

  z = opt.glme.gev11(xdat, ntry=ntry, ftol=ftol,
                    pretheta=pretheta, model=model,
                    pen=pen, mu=mu, std=std,
                    p=p, c0=c0, c1=c1, c2=c2, q=q,
                    opt.choose=opt.choose, show=show)

  if(z$convergence > 0) { z$para.glme = pretheta
     message("no result was found for proposed glme; ",
             "WLS pre-estimate returned") }

  names(z$para.glme) <- name_gev11_ns
  z$pen = pen                          # kept for v1.x compatibility
  z$pen_pen.choice= c(pen, pen.choice)

  if(pen=='beta'){
    ww= pk.beta(para=z$para.glme[c(1,3,5)],
                lme.center=pretheta[c(1,3,5)],
                p=p, c0=c0, c1=c1, c2=c2, q=q)
    z$p_q= c(ww$p,ww$q)
    z$c1_c2= c(c1,c2)                  # kept for v1.x compatibility
    z$c0_c1_c2=c(c0,c1,c2)

  }else if(pen=="norm"){
    z$mu_std =c(mu,std)
  }

  #---- v1.x compatibility fields (inexpensive) ---------------------
  z$strup.org = strup$strup.para       # WLSE by Strupczewski
  z$para.wls  = strup$strup.mdfy       # modified WLSE
  names(z$strup.org) <- names(z$para.wls) <- name_gev11_ns

  gado <- try(GN16.gev11(xdat), silent=TRUE)
  if(!inherits(gado, "try-error")){
    z$para.gado = gado$para.gado.org   # GN16 original estimates
    names(z$para.gado) <- name_gev11_ns
  }

  z$lme.sta = pargev(lmoms(xdat,nmom=3))$para   # stationary L-ME
  names(z$lme.sta) <- name_gev00_sta

  if(pen=="no") z$para.lme = z$para.glme

  z$data = xdat
  class(z) = "glme11"
  return(z)
}

#---------------------------------------------------------------------
#' GLME objective function for GEV11 model (mu0, sigma0, xi optimization)
#'
#' @param a Parameter vector (mu0, sigma0, xi).
#' @param xdat Data vector.
#' @param newtheta Full parameter vector (mu0, mu1, sigma0, sigma1, xi).
#' @param covinv Inverse covariance matrix of the sample L-moments.
#' @param lcovdet Log determinant of the covariance matrix.
#' @param pen Penalty type.
#' @param mu Normal penalty mean.
#' @param std Normal penalty std.
#' @param p Beta penalty shape.
#' @param c0 Beta penalty support half-width.
#' @param c1 Beta penalty scaling.
#' @param c2 Beta penalty limit.
#' @param q Optional fixed second beta shape parameter.
#' @return Penalized negative log-likelihood value.
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#' @keywords internal
nllh.glme.gev11 = function(a, xdat=xdat, newtheta=newtheta,
                           covinv=covinv, lcovdet=lcovdet,
                           pen=pen, mu=mu, std=std,
                           p=p, c0=c0, c1=c1, c2=c2, q=q)
{
  # updated 18Apr26
  ldist= fun.lme.gev11(a, xdat=xdat, pretheta=newtheta)

  if(any(ldist >= 10^5)) return(1e6)

  gld= t(ldist) %*% covinv %*% ldist

  nllh.norm =  gld/2 + (3/2)*log(2*pi) + lcovdet

  pk.ns= penalty.fun(par=a, mu=mu, std=std, lme=newtheta[c(1,3,5)],
                     pen=pen, p=p, c0=c0, c1=c1, c2=c2, q=q)
  as.numeric(nllh.norm + pk.ns)
}

#-----------------------------------------------------------------
#' L-moment distance function for GEV11 model
#'
#' @description Computes the vector of differences between the population
#' L-moments of the standard Gumbel distribution and the sample L-moments
#' of the data transformed by \code{\link{trans.gum01}} under the current
#' GEV11 parameters. Solving this system equal to zero gives the L-moment
#' estimates (Shin et al., 2025b).
#'
#' @param a Numeric vector (mu0, sigma0, xi) to optimize.
#' @param xdat Numeric vector of data.
#' @param pretheta Full parameter vector (mu0, mu1, sigma0, sigma1, xi)
#'   from pre-estimation; mu1 and sigma1 are held fixed.
#'
#' @return Numeric vector of length 3 of L-moment equation residuals.
#' @keywords internal
fun.lme.gev11 <- function(a, xdat=xdat, pretheta=pretheta)
{
  if( abs(a[3]) >= 1) return(rep(10^6,3))
  par=c(a[1],pretheta[2],a[2],pretheta[4],a[3])

  newg2= trans.gum01(xdat,para=par, na.ok=TRUE)
  newg=newg2[!is.na(newg2)]

  if( length(newg) < length(xdat)/2 ) return(rep(10^6,3))

  # population L-moments of the standard Gumbel:
  # lambda1 = Euler's constant, lambda2 = log(2), lambda3 = 0.169925*log(2)

  lgum= lmoms.md.park(newg, nmom=3, mtrim=FALSE, no.stop=TRUE)
  if(lgum$ifail == 1) return(rep(10^6,3))

  c(0.57721566, 0.69314718, 0.11778304) - lgum$lambdas[1:3]
}

#-----------------------------------------------------------------------
#' Multi-start penalized optimization for the GEV11 model
#'
#' @description Internal driver of \code{\link{glme.gev11}}. For
#' \code{pen="no"} it delegates to \code{\link{lme.gev11}}. Otherwise it
#' minimizes \code{\link{nllh.glme.gev11}} from several starting points,
#' using the fixed asymptotic covariance matrix of the first three sample
#' L-moments (computed for n=300 and rescaled by 300/n; see Eq. (26) of
#' Shin et al., 2025).
#'
#' @param xdat Numeric data vector.
#' @param ntry Number of starting points.
#' @param ftol Convergence tolerance.
#' @param show If TRUE, prints progress.
#' @param pretheta Pre-estimated parameter vector.
#' @param model Model name ("gev11").
#' @param pen,mu,std,p,q,c0,c1,c2 Penalty settings.
#' @param opt.choose "nllh" or "gof".
#' @return A list with para.glme, nllh.glme, convergence
#'   (and precis when \code{pen="no"}).
#' @keywords internal
opt.glme.gev11= function(xdat, ntry=5, ftol=1e-5, show=FALSE,
                        pretheta=NULL, model="gev11",
                        pen='beta', mu=NULL, std=NULL, p=2, q=NULL,
                        c0=0.35, c1=3, c2=2, opt.choose="nllh"){
  zm=list()

 if(pen=="no"){

   lme= lme.gev11(xdat, ntry=ntry, ftol=ftol)
   zm$precis = lme$precis

   if(lme$precis <= ftol){
     zm$para.glme = lme$lme.gev11
     zm$convergence =0
   }else{
     zm$para.glme = pretheta
     zm$convergence =5
   }
   return(zm)
 }

 # pen != "no" : perform glme -----------------------------------

  cov_300 = matrix(c( 0.0053986209, 0.0015941445, 0.0004516406,
                      0.0015941445, 0.0012886186, 0.0005222434,
                      0.0004516406, 0.0005222434, 0.0005117520),
                   3,3)

  cov_n0 = cov_300 *300/length(xdat)

  covinv= solve(cov_n0)
  lcovdet=log(det(cov_n0))

  newtheta = pretheta
  init = matrix(0, nrow=ntry, ncol=3)
  init[1,] = pretheta[c(1,3,5)]

  if(ntry >= 2){
    max_pk= find_max_beta.pk(newtheta[c(1,3,5)],p=p,
                             c0=c0,c1=c1,c2=c2)
    init[2,]= c(newtheta[c(1,3)]+0.2, max_pk)
  }
  if(ntry >= 3){
    sinit = init.glme(xdat, ntry=ntry, model="gev11",
                            pretheta=pretheta)
    init[3:ntry,]= sinit[1:(ntry-2),]
  }

  conv= my.nllh= rep(1e6,ntry)
  para.sel=matrix(NA,ntry,ncol=5)

  for(i in 1:ntry){

    kt <- try(
          optim(par=as.vector(init[i,1:3]),
                fn= nllh.glme.gev11,
                method="BFGS",
                control=list(abstol=ftol, reltol=ftol,
                               maxit=80),
                xdat=xdat, newtheta=newtheta, covinv=covinv,
                lcovdet=lcovdet, pen=pen, mu=mu,
                std=std, p=p, c0=c0, c1=c1, c2=c2, q=q),
      silent=TRUE)

    if(inherits(kt, "try-error")){
      my.nllh[i]= 10^6
      conv[i]= 5
      next
    }

    if(show==TRUE) cat("---itry, fvec=",i,kt$value,"\n")

    if( kt$convergence != 0) {
      my.nllh[i]=10^6
      conv[i] =5
    }else{
      my.nllh[i] = kt$value
      conv[i]= 0
      para.sel[i,1:5]=c(kt$par[1], newtheta[2],
                        kt$par[2], newtheta[4], kt$par[3])
    } #if
  } #for

  if(all(my.nllh==10^6)) {
    message("-- No solution found in optim for glme.gev11")
    zm$para.glme = pretheta
    zm$convergence= 5
    return(zm)
   }

  if(opt.choose=="nllh"){

      id= which.min(my.nllh)
      zm$para.glme = para.sel[id,]
      zm$nllh.glme = my.nllh[id]
      zm$convergence = conv[id]

  }else if(opt.choose=="gof"){

      sel = sel.para_all(xdat,    #para est. SSP JKSS(2025)
                         para.sel, model, obj.fun=my.nllh)

      zm$para.glme = sel$para
      zm$nllh.glme = my.nllh[sel$min.itry]
      zm$convergence = conv[sel$min.itry]
  } #if opt

 return(zm)
}

#-------------------------------------------------------------
#' Find the shape parameter maximizing the beta penalty
#'
#' @description Finds the value of xi (over a grid within the adaptive
#' beta penalty support) which maximizes the beta penalty function.
#' Used as one of the initial points for optimization.
#'
#' @param lme.center Center vector (mu0, sigma0, xi) of the penalty.
#' @param p,c0,c1,c2 Beta penalty hyperparameters.
#' @return The xi value maximizing the penalty.
#' @keywords internal
find_max_beta.pk=function(lme.center,p=p,
                          c0=c0,c1=c1,c2=c2){

  lower= max(-1, lme.center[3]-c0)
  upper= min(0.3, lme.center[3]+c0)
  xpk= seq(lower,upper,by=(upper-lower)/70)
  pk=rep(NA,length(xpk))

  for(i in 1:length(xpk)){
    a= c(0,1,xpk[i])
    pk[i]= pk.beta(para= a, lme.center=lme.center,
                   p=p, c0=c0,c1=c1,c2=c2)$pk.one
  }
  xpk[which.max(pk)]
}

#-----------------------------------------------------------
#' Select parameters using goodness-of-fit criterion
#'
#' @description Internal function that selects the best parameter set among
#' candidate solutions using a goodness-of-fit measure based on the expected
#' vs. observed number of exceedances (see Shin et al., 2025b, JKSS).
#'
#' @param xdat Numeric vector of data.
#' @param para.sel Matrix of candidate parameter vectors (one per row).
#' @param model Model name string (e.g., "gev11").
#' @param obj.fun Numeric vector of objective values used for tie-breaking.
#'
#' @return A list containing:
#' \describe{
#'   \item{para}{Selected parameter vector}
#'   \item{min.itry}{Index of selected try}
#'   \item{gof}{Goodness-of-fit values}
#'   \item{obj.fun}{Objective function values}
#' }
#' @keywords internal
sel.para_all =function(xdat, para.sel=NULL, model=NULL,
                       obj.fun=NULL){
  z=list()
  upara.sel = para.sel

  gof=rep(NA,nrow(upara.sel))
  ns=length(xdat)
  npar=ncol(upara.sel)

  vecT=c(5,10,20,40,60)
  if(ns >= 100) vecT=c(5,10,20,40,80,120)
  if(ns <= 30) vecT=c(5,10,20,40)

  for(i in 1:nrow(upara.sel) ){
    par.vec = as.vector(upara.sel[i,1:npar])
    gof[i]  = gof.ene_all(xdat, vecT, par.vec, model)
  }

  if(length(unique(gof))==1) {
    z$para =upara.sel[which.min(obj.fun),]
    z$min.itry = which.min(obj.fun)
  }else{
    z$para =upara.sel[which.min(gof),]
    z$min.itry = which.min(gof)
  }

  z$gof=gof
  z$obj.fun= obj.fun
  return(z)
}

#-----------------------------------------------------------
#' Goodness-of-fit by number of exceedances
#'
#' @description Internal function computing a goodness-of-fit measure
#' comparing the expected and observed numbers of exceedances of the
#' T-year levels for a set of return periods.
#'
#' @param xdat Numeric vector of data.
#' @param vecT Vector of return periods.
#' @param para Parameter vector.
#' @param model Model name string.
#' @return Sum of the relative absolute differences.
#' @keywords internal
gof.ene_all = function(xdat, vecT=c(5,10,20,40,80),
                       para=NULL, model=NULL){

  ns=length(xdat)
  nT = length(vecT)
  chi=rep(NA,nT)

  for(i in 1:nT){
    qt = quagev.NS(f=1-(1/vecT[i]), para,
                   nsample=ns, model)
    ene = ns/vecT
    sne = sum(xdat >= qt)
    chi[i] = abs(ene[i]-sne) /ene[i]
  }
  sum(chi)
}

#-------------------------------------------------------------
#' Modified L-moments computation with failure handling
#'
#' @description A slightly modified version of \code{lmomco::lmoms} (by
#' J. Park) which returns an \code{ifail} flag instead of stopping when
#' the L-moments cannot be computed.
#'
#' @param x Numeric vector.
#' @param nmom Number of L-moments.
#' @param mtrim If TRUE, uses left-trimmed TL-moments (leftrim=5).
#' @param no.stop If TRUE, returns \code{ifail=1} instead of stopping on error.
#' @return A list of L-moments with an additional \code{ifail} element.
#' @keywords internal
lmoms.md.park = function (x, nmom = 5, mtrim=FALSE,
                          no.stop = FALSE){
    z=list()
    n <- length(x)
    if (nmom > n) {
      if (no.stop) { z$ifail=1; return(z) }
      stop("More L-moments requested by parameter 'nmom' than data points available in 'x'")
    }
    if (length(unique(x)) == 1) {
      if (no.stop) { z$ifail=1; return(z) }
      stop("all values are equal--Lmoments can not be computed")
    }

    if(mtrim==FALSE){
      z <- TLmoms(x, nmom = nmom)
    }else{
      z <- TLmoms(x, nmom = nmom, leftrim=5)
    }
    z$source <- "lmoms"
    z$ifail=0
    return(z)
}

#---------------------------------------------------------------
#' Transform non-stationary GEV data to standard Gumbel
#'
#' @description Transforms observations from a non-stationary GEV11 model
#' with parameters \code{para} = (mu0, mu1, sigma0, sigma1, xi) to the
#' standard Gumbel scale.
#'
#' @param xdat Numeric vector of data.
#' @param para GEV11 parameter vector (mu0, mu1, sigma0, sigma1, xi).
#' @param na.ok If TRUE, invalid (out-of-support) values are returned as NA;
#'   if FALSE, they are clipped.
#' @return Numeric vector on the standard Gumbel scale.
#' @keywords internal
trans.gum01= function(xdat, para=NULL, na.ok=FALSE){

  ns= length(xdat)
  ngum01= rep(0,ns)

  ngum.dat= (xdat-(para[1]+ para[2]*seq(1,ns)))/exp(para[3]
                          + para[4]*seq(1,ns))
  work= 1 - para[5]*ngum.dat

  if(na.ok == FALSE){
    work[which(work <= 0)] = 1e-100
    ngum01= log(work)/(-para[5])
  }else{
    ngum01[which(work <= 0)] = NA
    id= which(work > 0)
    ngum01[id] = log(work[id])/(-para[5])
  }
  return(ngum01)
}

# --------------------------------------------------------------
# Random number generation and quantile functions for stationary
# and non-stationary GEV models
# --------------------------------------------------------------

#' Set parameters based on non-stationary model type
#'
#' @description Internal function that maps a flat parameter vector to named
#' location (mu), scale (sig), and shape (xi) components depending on the
#' model specification (GEV00, GEV10, GEV11, GEV20, GEV01).
#'
#' @param para Parameter vector whose length and meaning depend on \code{model}.
#' @param model Model type: "gev11", "gev10", "gev20", "gev01", or "gev00"/"gev".
#'
#' @return A list containing:
#' \describe{
#'   \item{mu}{Numeric vector c(mu0, mu1, mu2) for location}
#'   \item{sig}{Numeric vector c(sigma0, sigma1) for log-scale}
#'   \item{xi}{Shape parameter (scalar)}
#' }
#'
#' @keywords internal
set.para.model = function(para, model=NULL){

  z=list()
  mu0=para[1]
  mu1=para[2]

  if(model=='gev10'){
    sig0=para[3]
    xi=para[4]
    mu2=0
    sig1=0
  }else if(model=='gev20'){
    mu2=para[3]
    sig0=para[4]
    xi=para[5]
    sig1=0
  }else if(model=='gev11'){
    sig0=para[3]
    sig1=para[4]
    xi=para[5]
    mu2=0
  }else if(model=='gev00' | model=='gev'){
    mu1=mu2=sig1=0
    sig0=para[2]
    xi=para[3]
  }else if(model=='gev01'){
    mu1=mu2=0
    sig0=para[2]
    sig1=para[3]
    xi=para[4]
  }

  z$mu=c(mu0,mu1,mu2)
  z$sig=c(sig0,sig1)
  z$xi=xi
  return(z)
}

#---------------------------------------------------------
#' Random number generation for non-stationary GEV models
#'
#' @description
#' Generates a random sample from a stationary or non-stationary GEV model.
#' For non-stationary models the location and/or scale vary with time
#' t = 1, ..., nsample:
#' mu(t) = mu0 + mu1*t + mu2*t^2 and sigma(t) = exp(sigma0 + sigma1*t).
#'
#' @param nsample Sample size (number of time points).
#' @param para Parameter vector; its length and meaning depend on
#'   \code{model} (see \code{\link{quagev.NS}}). For "gev11":
#'   (mu0, mu1, sigma0, sigma1, xi).
#' @param model Model type: "gev", "gev00", "gev01", "gev10", "gev11",
#'   or "gev20".
#'
#' @return A numeric vector of length \code{nsample}.
#'
#' @references
#' Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
#' model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
#' \doi{10.1007/s42952-025-00325-3}
#'
#' @seealso \code{\link{quagev.NS}} for the corresponding quantile function,
#'   \code{\link{glme.gev11}}, \code{\link{lme.gev11}}.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' set.seed(1)
#' # Non-stationary GEV11 sample of size 50
#' x <- ran.gev_all(50, para = c(100, 0.5, 3, 0.005, -0.2), model = "gev11")
#' plot(x, type = "l")
#'
#' @export
ran.gev_all= function(nsample, para, model=NULL){

  rns=rep(NA, nsample)

  # model=c("gev","gev01","gev10", "gev11", "gev20")

  sp=set.para.model(para,model)

  for (t in 1:nsample){
    mut=sp$mu[1]+ sp$mu[2]*t +sp$mu[3]*(t^2)
    sigt=exp(sp$sig[1] +sp$sig[2]*t)

    savens=vec2par(c(mut,sigt,sp$xi),'gev')
    rns[t]=rlmomco(1,savens)
  }

  return(rns)
}

#----------------------------------------------------
#' Quantile function for non-stationary GEV models
#'
#' @description
#' Calculates quantiles for non-stationary GEV models including GEV11,
#' GEV10, GEV20, and stationary GEV00.
#'
#' @param f Probability (or vector of probabilities) for quantile calculation.
#' @param para Parameter vector. For GEV11: (mu0, mu1, sigma0, sigma1, xi).
#' @param nsample Number of time points (sample size).
#' @param model Model type: "gev11", "gev10", "gev20", "gev01", or "gev00"/"gev".
#'
#' @return A matrix of quantiles (nsample x length(f)) or a vector if f is scalar.
#'
#' @references
#' Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
#' L-moment estimation for stationary and nonstationary extreme value models.
#' arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
#'
#' @seealso \code{\link{glme.gev11}} for non-stationary GEV estimation,
#'   \code{\link{glme.gev}} for stationary GEV estimation,
#'   \code{\link{ran.gev_all}} for random number generation.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' # GEV11 model: time-varying quantiles
#' para <- c(84.55, 1.03, 2.91, 0.009, -0.08)  # mu0, mu1, sigma0, sigma1, xi
#' q99 <- quagev.NS(f = 0.99, para = para, nsample = 53, model = "gev11")
#' print(q99)
#'
#' @export
quagev.NS= function(f=NULL, para=NULL, nsample=NULL, model=NULL){

  if(model=='gev00' | model=='gev'){
    zpT = quagev(f,vec2par(para,'gev'))
    return(zpT)

  }else{  # if(model !='gev00')

    ns=nsample
    year=seq(1:ns)
    numq=length(f)
    zpT= matrix(NA,ns,numq)

    sp=set.para.model(para,model)

    xi= sp$xi
    zpc= (1- (-log(f))^xi) /xi
    vec = sp$mu[1] + sp$mu[2]*year[1:ns] +sp$mu[3]*(year[1:ns]^2)

    for(iq in 1:numq){
      zpT[1:ns,iq] = vec[1:ns]+ zpc[iq]* exp( sp$sig[1]
                                               +sp$sig[2]*year[1:ns] )
    }
    if(numq==1) zpT= as.vector(zpT)
    zpT
  }
}

#----------------------------------------------------
#' Quantile function for GEV11 model
#'
#' @description Internal wrapper that computes time-varying quantiles for the
#' GEV11 model given a return period.
#'
#' @param Tp Return period.
#' @param para Parameter vector (mu0, mu1, sigma0, sigma1, xi).
#' @param year Time vector.
#' @return Numeric vector of quantile values (one per time point).
#' @keywords internal
qns.gev11= function(Tp=NULL, para=NULL, year=NULL){

  model="gev11"
  f=1-1/Tp
  quagev.NS(f,para,nsample=length(year),model)
}

#----------------------------------------------------
#' Quantile function for non-stationary GEV (return period input)
#'
#' @description Internal function that computes time-varying quantiles for
#' non-stationary GEV models given a return period.
#'
#' @param Tp Return period (scalar).
#' @param para Parameter vector for the non-stationary model.
#' @param year Numeric vector of time points.
#' @param model Model type: "gev11", "gev10", "gev20", or "gev01".
#'
#' @return Numeric vector of quantile values (one per time point).
#'
#' @keywords internal
qns.gev_all= function(Tp=NULL, para=NULL, year=NULL, model=NULL){

  nsample=length(year)
  ns=nsample
  zpT=rep(NA, nsample)
  year2=year^2

  sp=set.para.model(para,model)

  xi= sp$xi
  zpc= (1- ( -log(1-(1/Tp) ) )^xi ) /xi
  zpT[1:ns] = sp$mu[1] + sp$mu[2]*year[1:ns] +sp$mu[3]*year2[1:ns]
  zpT[1:ns] = zpT[1:ns]+ zpc* exp( sp$sig[1] +sp$sig[2]*year[1:ns] )

  return(zpT)
}

# ---------------------------------------------------------------
# Generalized L-moment estimation (GLME) for the stationary GEV
# L-me under a preference (prior) function for stationary GEV
# ---------------------------------------------------------------

#' Initialize parameters for GEV MLE estimation
#'
#' @description
#' This function initializes parameters for GEV maximum likelihood estimation.
#'
#' @param data A numeric vector of data to be fitted.
#' @param ntry Number of initial parameter sets to generate.
#'
#' @details
#' The function generates `ntry` sets of initial parameters for the GEV distribution.
#' It uses L-moment estimates as a starting point and then generates additional
#' sets of parameters using random perturbations.
#'
#' @return A matrix with `ntry` rows and 3 columns, where each row represents
#' a set of initial parameters (location, scale, shape) for the GEV distribution.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#' @keywords internal
init.gevmax <-function(data=NULL, ntry=NULL){

  if(ntry < 3) ntry=3
  init <-matrix(NA, ntry, 3)

  lmom_init = lmoms(data, nmom=3)
  init[1,] <- pargev(lmom_init)$para

  sd1= max(abs(init[1,1])*0.1, 5)
  sd2= max(init[1,2]*2, 2)

  maxm2=ntry-1
  init[2:ntry,1] <- init[1,1]+rnorm(n=maxm2, mean=0, sd = sd1)
  init[2:ntry,2] <- init[1,2]+rnorm(n=maxm2, mean=4, sd = sd2)
  init[2:ntry,3] <- runif(n=maxm2, min= -0.45, max=0.4)
  init[2:ntry,2] = pmax(0.1, init[2:ntry,2])

  return(init)
}


#' Initialize starting values for GLME optimization
#'
#' @description
#' Generates multiple starting parameter sets for multi-start optimization
#' in GLME estimation, for both the stationary GEV (\code{model="gev00"})
#' and the non-stationary GEV11 (\code{model="gev11"}). Uses L-moment
#' estimates as a base and adds random perturbations.
#'
#' For \code{model="gev11"} the second column is returned on the log scale
#' (matching sigma0 of the GEV11 parameterization), and when \code{pretheta}
#' is supplied the first candidate is a perturbation of the pre-estimate.
#'
#' @param data A numeric vector of data to be fitted.
#' @param ntry Number of initial parameter sets to generate (minimum 2).
#' @param model Either "gev00" (stationary, default) or "gev11".
#' @param pretheta Optional pre-estimate (mu0, mu1, sigma0, sigma1, xi)
#'   used for the first candidate when \code{model="gev11"}.
#' @param xdat Deprecated alias of \code{data} (v1.x compatibility).
#'
#' @return A matrix with \code{ntry} rows and 3 columns,
#'   where each row is a candidate starting point for optimization.
#'
#' @seealso \code{\link{glme.gev}}, \code{\link{glme.gev11}} which use this
#'   function internally.
#'
#' @examples
#' data(haenam)
#' inits <- init.glme(haenam$X1, ntry = 5)
#' print(inits)
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#' @export
init.glme = function(data = NULL, ntry = NULL, model="gev00",
                     pretheta=NULL, xdat=NULL)
{
  if(is.null(data) & !is.null(xdat)) data <- xdat  # v1.x compatibility

  if (ntry < 2) ntry <- 2
  init <- matrix(NA, ntry, 3)

  init[2, 1:3] <- pargev(lmoms(data, nmom = 3))$para[1:3]
  init[1, 1:3] = c(mean(data)*0.75, sd(data)*0.5,
                   max(init[2,3]-0.03,-0.99))

  if(ntry >= 3){
    maxm2 <- ntry - 2
    init[3:ntry, 1] <- init[2, 1] + rnorm(n = maxm2, mean = 0,
                                       sd = 5)
    init[3:ntry, 2] <- pmax(0.1, init[2, 2] + rnorm(n = maxm2,
                                      mean = 3, sd = 3))
    init[3:ntry, 3] <- runif(n = maxm2, min = -0.49, max = 0.4)
  }

  if(model=="gev11"){
    init[,2] <- log(init[,2])

    if(!is.null(pretheta)){
      init[1, 1:3] = c( pretheta[1]*0.98, max(pretheta[3]-0.1, 0.1),
                        max(pretheta[5]-0.03, -0.99) )
    }
  }
  return(init)
}


#' Bootstrap covariance of sample L-moments
#'
#' @description Computes the covariance matrix of the first three sample
#' L-moments by nonparametric bootstrap. Used as a fallback when the
#' direct covariance estimate is singular.
#'
#' @param xdat Numeric data vector.
#' @param BB Number of bootstrap replicates.
#' @return A 3x3 covariance matrix.
#' @keywords internal
boot.cov = function(xdat,BB){
  sam.lmom= matrix(NA,BB,3)
  ns= length(xdat)

  for(ib in 1:BB){
    idx= sample.int(ns,ns,replace=TRUE)
    sam.lmom[ib,1:3]= lmoms(xdat[idx],nmom=3)$lambdas
  }
  cov(sam.lmom)
}


#' Calculate the likelihood for Generalized L-moments estimation of GEV distribution
#'
#' @description
#' This function calculates the likelihood (or more precisely, a penalized negative log-likelihood)
#' for the Generalized L-moments estimation of the Generalized Extreme Value (GEV) distribution.
#'
#' @param par A vector of GEV parameters (location, scale, shape).
#' @param xdat A numeric vector of data.
#' @param slmgev Sample L-moments of the data.
#' @param covinv Inverse of the covariance matrix of the sample L-moments.
#' @param lcovdet Log determinant of the covariance matrix.
#' @param mu Mean for the normal penalization (used when pen='norm').
#' @param std Standard deviation for the normal penalization (used when pen='norm').
#' @param lme L-moment estimates of the parameters.
#' @param pen Penalization method: 'norm', 'beta', 'ms', 'park', 'cannon', 'cd', or 'no'.
#' @param p Shape parameter for beta penalty.
#' @param c1 Scaling parameter for beta penalty.
#' @param c2 Upper limit parameter for beta penalty.
#' @param c0 Half-width of the adaptive beta penalty support (default 0.35).
#' @param q Optional fixed second shape parameter for the beta penalty.
#'
#' @details
#' The function performs the following steps:
#' 1. Checks if the parameters are within valid ranges.
#' 2. Calculates the expected L-moments based on the current parameters.
#' 3. Computes the difference between expected and sample L-moments.
#' 4. Calculates the generalized L-moments distance.
#' 5. Applies a penalization term based on the specified method.
#' 6. Returns the sum of the L-moments distance and the penalization term.
#'
#' @return A numeric value representing the penalized negative log-likelihood.
#' A lower value indicates a better fit.
#'
#' @references
#' Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
#' L-moment estimation for stationary and nonstationary extreme value models.
#' arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
#'
#' @seealso \code{\link{glme.gev}} which calls this function for optimization.
#'
#' @examples
#' data(haenam)
#' x <- haenam$X1
#' slm <- lmomco::lmoms(x, nmom = 3)
#' cov_mat <- lmomco::lmoms.cov(x, nmom = 3)
#' lme_par <- lmomco::pargev(slm)$para
#' glme.like(par = lme_par, xdat = x, slmgev = slm,
#'           covinv = solve(cov_mat), lcovdet = log(det(cov_mat)),
#'           mu = -0.5, std = 0.2, lme = lme_par, pen = "beta",
#'           p = 6, c1 = 3, c2 = 1)
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#' @export
glme.like = function(par, xdat=xdat, slmgev=slmgev, covinv=covinv,
                     lcovdet=lcovdet, mu=mu, std=std, lme=lme,
                     pen=pen, p=p, c1=c1, c2=c2, c0=0.35, q=NULL){

  if( par[2] <= 0) return(10^8)
  if( abs(par[3]) >= 1) return(10^8)
  if( abs(par[3]) <= 1e-5) par[3]= -1e-5

  emom= lmomgev( vec2par(par,type='gev') )$lambdas[1:3]
  if( any(is.na(emom[1:3])) ) return(10^8)

  zvec= emom - slmgev$lambdas[1:3]

  gld= t(zvec) %*% covinv %*% zvec
  prob.norm =  gld/2  + (3/2)*log(2*pi) + lcovdet

  pk_beta = penalty.fun(par=par, mu=mu, std=std, lme=lme,
                    pen=pen, p=p, c0=c0, c1=c1, c2=c2, q=q)
  as.numeric(prob.norm + pk_beta)
}


#' Generalized L-moments estimation for generalized extreme value distribution
#'
#' @description
#' This function estimates the parameters of the Generalized Extreme Value
#' distribution by the generalized method of L-moment estimation (GLME),
#' which minimizes the generalized L-moment distance plus a penalty
#' (preference) function on the shape parameter.
#'
#' @param xdat A numeric vector of data to be fitted.
#' @param ntry Number of attempts for parameter estimation (default 5). Higher
#'   values increase the chance of finding the global optimum by trying
#'   different initial conditions.
#' @param pen Type of penalty function: Choose among "norm", "beta" (default),
#'   "ms", "park", "cannon", "cd", and "no" (without penalty function).
#' @param pen.choice Choice number of penalty function specifying hyperparameters
#'   (default 1). For "beta": 1-6 correspond to (p, c1, c2) =
#'   (6,3,1), (6,5,2), (6,7,3), (2,3,0.5), (2,5,1), (2,7,1.5).
#'   For "norm": 1-4 correspond to (mu, std) =
#'   (-0.5,0.25), (-0.5,0.15), (-0.6,0.25), (-0.6,0.15).
#'   Set \code{pen.choice=NULL} to use the hyperparameters given by
#'   \code{p, c1, c2} (beta) or \code{mu, std} (norm) directly.
#' @param mu Mean hyperparameter for "norm" penalty function (default -0.5).
#' @param std Standard deviation hyperparameter for "norm" penalty function (default 0.2).
#' @param p Shape hyperparameter for "beta" penalty function (default 6).
#' @param c1 Scaling hyperparameter for "beta" penalty function (default 3).
#' @param c2 Upper limit hyperparameter for "beta" penalty function (default 1).
#' @param c0 Half-width of the adaptive beta penalty support (default 0.35).
#' @param q Optional fixed second shape parameter for the beta penalty. If
#'   given, the beta penalty support is fixed instead of data-adaptive.
#' @param show If TRUE, prints the objective value and parameters of each try.
#' @param method Optimization method passed to \code{\link[stats]{optim}}
#'   (default "BFGS").
#' @param maxit Maximum number of iterations for \code{optim} (default 70).
#' @param abstol Absolute and relative convergence tolerance for \code{optim}
#'   (default 1e-5).
#'
#' @details
#' The equations for the L-moments for LME of the GEVD are
#' \deqn{ \underline{\bf \lambda} - \underline{\bf l} = \underline{\bf 0},}
#' where \eqn{ \underline{\bf \lambda} =(\lambda_1,\; \lambda_2,\; \lambda_3)^t } and \eqn{\underline{\bf l} =(l_1,\; l_2,\; l_3)^t}.
#' Next, we define the generalized L-moments distance (GLD) as;
#' \deqn{(\underline{\bf \lambda} -\underline{\bf l})^t V^{-1} (\underline{\bf \lambda} -\underline{\bf l}),}
#' where \eqn{V} is the variance-covariance matrix of the sample L-moments up to the third order.
#'
#' @return The glme.gev function returns a list containing the following elements:
#' \itemize{
#'  \item para.glme - The estimated parameters of the Generalized Extreme Value distribution.
#'  \item para.lme - The L-moment estimates of the parameters.
#'  \item nllh.glme - The penalized negative log-likelihood of the GLME solution.
#'  \item convergence - 0 if the optimization converged, 5 otherwise.
#'  \item pen, pen_pen.choice - The penalization method (and choice) used.
#'  \item p_q - (for beta penalty) The p and q values used.
#'  \item c1_c2, c0_c1_c2 - (for beta penalty) The hyperparameters used.
#'  \item mu_std - (for norm penalty) The mu and std values used.
#'  \item covinv.lmom - The inverse of the covariance matrix of the L-moments.
#'  \item lcovdet - The log determinant of the covariance matrix.
#'  \item data - The input data (used by the plot method).
#' }
#' The object has class \code{"glme"}; see \code{\link{GLmom-methods}}.
#'
#' @references
#' Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
#' L-moment estimation for stationary and nonstationary extreme value models.
#' arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
#'
#' @seealso \code{\link{glme.gev11}} for non-stationary GEV estimation,
#'   \code{\link{ma.gev}} for model averaging estimation,
#'   \code{\link{glme.like}} for the objective function,
#'   \code{\link{quagev.NS}} for quantile computation.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' # Load example heavy-rainfall data
#' data(haenam)
#' x <- haenam$X1
#'
#' # Estimate GEV parameters using beta penalty (default)
#' result <- glme.gev(x, ntry = 5)
#' print(result$para.glme)
#'
#' # Using Martins-Stedinger penalty
#' result_ms <- glme.gev(x, ntry = 5, pen = "ms")
#' print(result_ms$para.glme)
#'
#' @export
glme.gev= function(xdat, ntry=5, pen='beta', pen.choice=1,
                   mu= -0.5, std= 0.2, p=6, c1=3, c2=1,
                   c0=0.35, q=NULL, show=FALSE,
                   method="BFGS", maxit=70, abstol=1e-5){
  # updated on 17Apr26 (v2.0.0, revised code of Shin et al.)

  z=list()

  ch= check.penalty(pen=pen, pen.choice=pen.choice, p=p,
                   c1=c1,c2=c2,mu=mu,std=std)
  pen= ch$pen
  if(!is.null(pen.choice)){
    p= ch$p ; c1 =ch$c1;  c2= ch$c2
    mu= ch$mu ; std= ch$std
  }

  sinit <- init.glme(xdat, ntry=ntry)
  slmgev = lmoms(xdat, nmom=3)

  z$para.lme= lme= pargev(slmgev)$para
  conv= nllh= rep(1e6, ntry)
  para= matrix(NA,ntry,3)

  cov= lmoms.cov(xdat, nmom=3)
  detc = det(cov)

  if(detc <= 0){    # Bootstrap to calculate cov
    cov= boot.cov(xdat,BB=100)
    detc= det(cov)
  }
  lcovdet= log(detc)
  covinv = solve(cov)

  z$covinv.lmom = covinv   # kept for v1.x compatibility
  z$lcovdet = lcovdet

  # estimating paras using optim
  for(i in 1:ntry){

    kt <- try(
      optim(par=as.vector(sinit[i,1:3]), fn=glme.like,
            method=method,
            control=list(abstol=abstol, reltol=abstol,
                         maxit=maxit),
            xdat=xdat, slmgev=slmgev, covinv=covinv,
            lcovdet=lcovdet, mu=mu, std=std, lme=lme,
            pen=pen, p=p, c1=c1, c2=c2, c0=c0, q=q),
      silent=TRUE)

    if(inherits(kt, "try-error")){
      nllh[i]= 1e6
      conv[i]= 5
      next
    }

    if( kt$convergence != 0){
      nllh[i]=10^6
      conv[i]=5
    }else{
      nllh[i]= kt$value
      conv[i]= 0
      para[i,]= kt$par
    }

    if(show==TRUE){
      cat("--- itry, nllh=",i, nllh[i],"\n")
      cat("--- itry, para=",i, round(kt$par,4),"\n","\n")
    }
  } #for

  if(all(nllh >= 10^5)) {
    message("-- No solution found in optim at glme.gev")
    z$para.glme = z$para.lme
    z$convergence = 5
    z$pen = pen
    z$data = xdat
    class(z) = "glme"
    return(z)
  }
  istar= which.min(nllh)
  z$para.glme = para[istar,]
  z$nllh.glme = nllh[istar]
  z$convergence = conv[istar]

  names(z$para.lme) <- names(z$para.glme) <- c("mu","sig","xi")
  z$pen = pen                          # kept for v1.x compatibility
  z$pen_pen.choice = c(pen, pen.choice)

  if(pen=="beta"){
    ww = pk.beta(para= z$para.glme, lme.center=lme, p=p,
                 c0=c0, c1=c1, c2=c2, q=q)
    z$p_q= c(ww$p, ww$q)
    z$c1_c2=c(c1,c2)                   # kept for v1.x compatibility
    z$c0_c1_c2=c(c0,c1,c2)

  }else if(pen=="norm"){
    z$mu_std = c(mu, std)
  }
  z$data = xdat
  class(z) = "glme"
  return(z)
}


#' GEV parameter estimation with fixed shape parameter
#'
#' @description
#' Estimates GEV location and scale parameters from L-moments while keeping
#' the shape parameter fixed at a user-specified value. Modified from
#' \code{lmomco::pargev()}.
#'
#' @param lmom L-moments object.
#' @param kfix Fixed shape parameter value.
#' @param checklmom Whether to check L-moment validity.
#' @param ... Additional arguments.
#' @return A list with components:
#' \describe{
#'   \item{type}{Character "gev"}
#'   \item{para}{Numeric vector of GEV parameters (xi=location, alpha=scale, kappa=shape)}
#'   \item{source}{Character "pargev"}
#' }
#'
#' @references
#' Hosking, J. R. M. (1990). L-moments: Analysis and estimation of
#' distributions using linear combinations of order statistics.
#' Journal of the Royal Statistical Society, Series B, 52(1), 105-124.
#' \doi{10.1111/j.2517-6161.1990.tb01775.x}
#'
#' @seealso \code{\link{glme.gev}} for GLME estimation,
#'   \code{\link[lmomco]{pargev}} for the original L-moment GEV fitting.
#'
#' @examples
#' data(haenam)
#' lmom <- lmomco::lmoms(haenam$X1, nmom = 3)
#' pargev.kfix(lmom, kfix = -0.1)
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#' @export
pargev.kfix= function (lmom, kfix= 0.1, checklmom = TRUE, ...)
{

  # modified from 'pargev' function in lmomco package

  para <- rep(NA, 3)
  names(para) <- c("xi", "alpha", "kappa")
  SMALL <- 1e-05
  EPS <- 1e-06
  MAXIT <- 20
  EU <- 0.57721566
  DL2 <- 0.69314718
  DL3 <- 1.0986123
  A0 <- 0.2837753
  A1 <- -1.21096399
  A2 <- -2.50728214
  A3 <- -1.13455566
  A4 <- -0.07138022
  B1 <- 2.06189696
  B2 <- 1.31912239
  B3 <- 0.25077104
  C1 <- 1.59921491
  C2 <- -0.48832213
  C3 <- 0.01573152
  D1 <- -0.64363929
  D2 <- 0.08985247
  if (length(lmom$L1) == 0) {
    lmom <- lmorph(lmom)
  }
  if (checklmom & !are.lmom.valid(lmom)) {
    warning("L-moments are invalid")
    return()
  }
  T3 <- lmom$TAU3
  if (T3 > 0) {
    Z <- 1 - T3
    G <- (-1 + Z * (C1 + Z * (C2 + Z * C3)))/(1 + Z * (D1 +
                                                         Z * D2))
    if (abs(G) < SMALL) {
      para[3] <- 0
      para[2] <- lmom$L2/DL2
      para[1] <- lmom$L1 - EU * para[2]
      return(list(type = "gev", para = para))
    }
  }
  else {
    G <- (A0 + T3 * (A1 + T3 * (A2 + T3 * (A3 + T3 * A4))))/(1 +
                                                               T3 * (B1 + T3 * (B2 + T3 * B3)))
    if (T3 >= -0.8) {
    }
    else {
      if (T3 <= -0.97)
        G <- 1 - log(1 + T3)/DL2
      T0 <- (T3 + 3) * 0.5
      CONVERGE <- FALSE
      for (it in seq(1, MAXIT)) {
        X2 <- 2^-G
        X3 <- 3^-G
        XX2 <- 1 - X2
        XX3 <- 1 - X3
        T <- XX3/XX2
        DERIV <- (XX2 * X3 * DL3 - XX3 * X2 * DL2)/(XX2 *
                                                      XX2)
        GOLD <- G
        G <- G - (T - T0)/DERIV
        if (abs(G - GOLD) <= EPS * G)
          CONVERGE <- TRUE
      }
      if (CONVERGE == FALSE) {
        warning("Noconvergence---results might be unreliable")
      }
    }
  }

  para[3]=kfix
  G = kfix

  GAM <- exp(lgamma(1 + G))
  para[2] <- lmom$L2 * G/(GAM * (1 - 2^(-G)))
  para[1] <- lmom$L1 - para[2] * (1 - GAM)/G
  return(list(type = "gev", para = para, source = "pargev"))
}

# ------------------------------------------------------------------
# Estimate parameters of NS GEV11 by the Weighted Least Squares
# method of Strupczewski and Kaczmarek (2001)
# ------------------------------------------------------------------

#' Weighted least squares estimation for the non-stationary GEV11 model
#'
#' @description
#' Estimates the parameters of the non-stationary GEV11 model by the
#' weighted least squares (WLS) method of Strupczewski and Kaczmarek (2001),
#' with the final-specification modification described in Shin et al.
#' (2025b, JKSS). The result is used as the pre-estimate (starting value)
#' by \code{\link{glme.gev11}} and \code{\link{lme.gev11}}.
#'
#' @param xdat A numeric vector of data to be fitted.
#' @param init.rob If TRUE (default), the initial location trend is fitted
#'   by robust regression (\code{\link[robustbase]{lmrob}}); otherwise by
#'   ordinary least squares.
#' @param wls.rob If TRUE, the log-scale regression uses robust regression
#'   (default FALSE).
#' @param strup.fin.rob If TRUE (default), the final location trend
#'   re-specification uses robust regression.
#'
#' @return A list containing:
#' \itemize{
#'   \item strup.para - Original WLS estimates (mu0, mu1, sigma0, sigma1, xi).
#'   \item strup.mdfy - Modified (re-specified) WLS estimates.
#' }
#'
#' @references
#' Strupczewski, W. G. & Kaczmarek, Z. (2001). Non-stationary approach to
#' at-site flood frequency modelling II. Weighted least squares estimation.
#' Journal of Hydrology, 248(1-4), 143-151.
#'
#' Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
#' model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
#' \doi{10.1007/s42952-025-00325-3}
#'
#' @seealso \code{\link{glme.gev11}}, \code{\link{lme.gev11}},
#'   \code{\link{GN16.gev11}}.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' data(PhliuAgromet)
#' result <- strup.gev11(PhliuAgromet$prec)
#' print(result$strup.mdfy)
#'
#' @export
strup.gev11 = function(xdat, init.rob=TRUE,
                       wls.rob=FALSE, strup.fin.rob=TRUE){
  w=list()
  ns=length(xdat)
  year=seq(1,ns)

  reg.dat=data.frame( cbind(year, xdat) )

  if(init.rob==FALSE){
    mu.init= lm(xdat~year, reg.dat)$coefficients  # regression
  }else{
    mu.init= lmrob(xdat~year, reg.dat)$coefficients  # robust regression
  }
  res= xdat -(mu.init[1]+mu.init[2]*year)

  wls= wls.gev11(xdat, res=res, rob=wls.rob)  # log-scale regression

  wls$res = res/exp(wls$sig[1] + wls$sig[2]*year)
  strup.sta = pargev(lmoms(wls$res), checklmom=FALSE)$para

  strup.para = c(mu.init[1:2], wls$sig, strup.sta[3])
  w$strup.para = strup.para

  #------to specify final parameter values ----------
  # See JKSS(2025) paper by Shin, Shin, Park

  yt=rep(0,ns+1)
  mu_st=strup.sta[1]
  year2=seq(0,ns)

  for (ka in 1:(ns+1) ) {
    yt[ka]= ( strup.para[1] + strup.para[2]*(ka-1)
            + mu_st*exp(strup.para[3]+ strup.para[4]*(ka-1)) )
  }
  nh=round((ns/2))
  yt[nh-1] = yt[nh-1] + 0.02;   yt[nh-2] = yt[nh-2] - 0.02
  yt[nh+1] = yt[nh+1] - 0.01;   yt[nh+2] = yt[nh+2] + 0.01

  reg.dat=data.frame( cbind(year2, yt) )

  if(strup.fin.rob==FALSE){
    mu.f= lm(yt ~ year2, reg.dat)$coefficients
  }else{
    mu.f= lmrob(yt ~ year2, reg.dat)$coefficients
  }
  sigmaf_0 = strup.para[3] + log(strup.sta[2])

  w$strup.mdfy= c(mu.f, sigmaf_0, strup.para[4], strup.sta[3])
  names(w$strup.para) <- names(w$strup.mdfy) <- c("mu0",
                               "mu1","sigma0","sigma1","xi")
  w
}

#------------------------------------------------------------------
#' Log-scale regression step of the Strupczewski WLS method
#'
#' @description Internal step of \code{\link{strup.gev11}}: regresses the
#' log absolute residuals on time to estimate the log-scale coefficients
#' (sigma0, sigma1).
#'
#' @param xdat Numeric data vector.
#' @param res Residuals from the location trend fit.
#' @param rob If TRUE, uses robust regression (setting "KS2014").
#' @return A list with element \code{sig} (the two coefficients).
#' @keywords internal
wls.gev11= function(xdat, res=NULL, rob=NULL){

  z=list()
  ns=length(res)
  year=seq(1,ns)

  lres.pr=log(abs(res))
  sig.dat=data.frame( cbind(year, lres.pr) )

  if(rob==FALSE){
    z$sig= lm(lres.pr~year, sig.dat)$coefficients
  }else{
    z$sig= lmrob(lres.pr~year, data=sig.dat,
                 setting="KS2014")$coefficients
  }
  return(z)
}

# GLmom package - basic tests
library(GLmom)

set.seed(777)

# --- 1. Data loading ---
data(PhliuAgromet)
data(bangkok)
data(haenam)

stopifnot(is.data.frame(PhliuAgromet), nrow(PhliuAgromet) == 40)
stopifnot(is.data.frame(bangkok), nrow(bangkok) == 58)
stopifnot(is.data.frame(haenam), nrow(haenam) == 52)
stopifnot("prec" %in% names(PhliuAgromet), "X1" %in% names(haenam))

# --- 2. Stationary GEV (glme.gev) ---
res1 <- glme.gev(haenam$X1)

stopifnot(is.list(res1))
stopifnot("para.glme" %in% names(res1))
stopifnot("para.lme" %in% names(res1))
stopifnot("nllh.glme" %in% names(res1))
stopifnot("convergence" %in% names(res1))
stopifnot(length(res1$para.glme) == 3)
stopifnot(length(res1$para.lme) == 3)
stopifnot(is.numeric(res1$para.glme))
stopifnot(res1$para.glme[2] > 0)  # sigma > 0
# v1.x compatibility fields still present
stopifnot(all(c("pen", "c1_c2", "covinv.lmom", "lcovdet") %in% names(res1)))

# --- 3. Penalty functions ---
for (pen in c("beta", "norm", "ms", "park", "cannon", "cd", "no")) {
  r <- glme.gev(haenam$X1, pen = pen)
  stopifnot(is.numeric(r$para.glme), length(r$para.glme) == 3)
}

# All preset choices run
for (ch in 1:6) {
  r <- glme.gev(haenam$X1, ntry = 3, pen = "beta", pen.choice = ch)
  stopifnot(is.numeric(r$para.glme))
}
for (ch in 1:4) {
  r <- glme.gev(haenam$X1, ntry = 3, pen = "norm", pen.choice = ch)
  stopifnot(is.numeric(r$para.glme))
}

# v1.x-style call with explicit hyperparameters (pen.choice = NULL)
r_old <- glme.gev(haenam$X1, ntry = 3, pen = "beta", pen.choice = NULL,
                  p = 6, c1 = 10, c2 = 5)
stopifnot(is.numeric(r_old$para.glme))

# pk.beta / pk.beta.stnary are the same implementation
pb1 <- pk.beta(para = c(100, 20, -0.2), lme.center = c(100, 20, -0.15),
               p = 6, c1 = 3, c2 = 1)
pb2 <- pk.beta.stnary(para = c(100, 20, -0.2), lme.center = c(100, 20, -0.15),
                      p = 6, c1 = 3, c2 = 1)
stopifnot(identical(pb1, pb2), pb1$pk.one > 0)

# glme.gev with pen="no" reproduces the plain L-moment estimates
set.seed(2)
r_no <- glme.gev(haenam$X1, pen = "no")
stopifnot(max(abs(r_no$para.glme - r_no$para.lme)) < 0.1)

# --- 4. Non-stationary GEV11 (glme.gev11) ---
res2 <- glme.gev11(PhliuAgromet$prec, ntry = 5)

stopifnot(is.list(res2))
stopifnot("para.glme" %in% names(res2))
stopifnot("convergence" %in% names(res2))
stopifnot(length(res2$para.glme) == 5)
# v1.x compatibility fields
stopifnot(all(c("strup.org", "para.wls", "para.gado", "lme.sta") %in% names(res2)))
stopifnot(length(res2$para.wls) == 5, length(res2$lme.sta) == 3)

# pen="no" returns para.lme and equals lme.gev11() (same RNG state)
set.seed(123)
res2n <- glme.gev11(PhliuAgromet$prec, ntry = 5, pen = "no")
stopifnot("para.lme" %in% names(res2n))
set.seed(123)
lg <- lme.gev11(PhliuAgromet$prec, ntry = 10)
# same L-moment solution up to solver/RNG tolerance (lmrob consumes RNG)
stopifnot(max(abs(res2n$para.glme - lg$lme.gev11)) < 1e-2)

# Regression baseline: v2.0.0 snapshot for the PhliuAgromet L-moment fit
# (values computed at release; generous tolerances allow for platform and
# RNG differences in the robust-regression pre-estimation step)
stopifnot(abs(lg$lme.gev11[1] - 126.77) < 1.0)     # mu0
stopifnot(abs(lg$lme.gev11[2] - 0.570) < 0.05)     # mu1
stopifnot(abs(lg$lme.gev11[3] - 2.759) < 0.10)     # sigma0
stopifnot(abs(lg$lme.gev11[4] - 0.041) < 0.01)     # sigma1
stopifnot(abs(lg$lme.gev11[5] - (-0.030)) < 0.05)  # xi
stopifnot(lg$precis < 1e-4)

# opt.choose = "gof" also runs
res2g <- glme.gev11(PhliuAgromet$prec, ntry = 5, opt.choose = "gof")
stopifnot(length(res2g$para.glme) == 5)

# --- 5. Component methods (new in v2.0.0) ---
s <- strup.gev11(PhliuAgromet$prec)
stopifnot(all(c("strup.para", "strup.mdfy") %in% names(s)))
stopifnot(length(s$strup.mdfy) == 5)

gn <- GN16.gev11(PhliuAgromet$prec)
stopifnot(all(c("para.gado.org", "para.gado.mdfy") %in% names(gn)))
stopifnot(length(gn$para.gado.org) == 5)

rr <- ran.gev_all(30, para = c(100, 0.5, 3, 0.005, -0.2), model = "gev11")
stopifnot(is.numeric(rr), length(rr) == 30, all(is.finite(rr)))

# --- 6. Compatibility wrappers (nsgev, gado.prop_11) ---
res3 <- nsgev(PhliuAgromet$prec, ntry = 5)

stopifnot(is.list(res3))
stopifnot("para.prop" %in% names(res3))
stopifnot(length(res3$para.prop) == 5)

res3b <- suppressWarnings(gado.prop_11(PhliuAgromet$prec, ntry = 5))
stopifnot(all(c("para.prop", "para.gado", "para.wls", "strup.org", "lme.sta")
              %in% names(res3b)))

# --- 7. Model averaging (ma.gev) ---
res4 <- ma.gev(haenam$X1, quant = c(0.99, 0.995), numk = 7)

stopifnot(is.list(res4))
stopifnot("qua.ma" %in% names(res4))
stopifnot("qua.mle" %in% names(res4))
stopifnot("qua.lme" %in% names(res4))
stopifnot("w.ma" %in% names(res4))
stopifnot("pick_xi" %in% names(res4))
stopifnot(length(res4$qua.ma) == 2)
stopifnot(all(res4$qua.ma > 0))

# --- 8. Auxiliary functions ---
# pargev.kfix
lmom <- lmomco::lmoms(haenam$X1)
pk <- pargev.kfix(lmom, kfix = -0.1)
stopifnot(is.list(pk), "para" %in% names(pk))

# quagev.NS (returns one quantile per time point)
q <- quagev.NS(f = 0.99, para = res2$para.glme, nsample = 40, model = "gev11")
stopifnot(is.numeric(q), length(q) == 40, all(q > 0))

# --- 9. S3 classes and methods (new in v2.0.0) ---
stopifnot(inherits(res1, "glme"), inherits(res2, "glme11"),
          inherits(lg, "lme11"), inherits(res4, "magev"))
stopifnot(is.list(res1), !is.null(res1$data))   # still a plain list underneath

out <- utils::capture.output(print(res1));    stopifnot(length(out) > 3)
out <- utils::capture.output(summary(res2));  stopifnot(length(out) > 5)
out <- utils::capture.output(print(lg));      stopifnot(length(out) > 3)
out <- utils::capture.output(summary(res4));  stopifnot(length(out) > 5)

grDevices::pdf(NULL)
plot(res1); plot(res2); plot(lg); plot(res4)
grDevices::dev.off()

# init.glme: new interface and v1.x-style named xdat=
i1 <- init.glme(haenam$X1, ntry = 4)
i2 <- init.glme(xdat = haenam$X1, ntry = 4)
stopifnot(is.matrix(i1), nrow(i1) == 4, is.matrix(i2), nrow(i2) == 4)
i3 <- init.glme(PhliuAgromet$prec, ntry = 4, model = "gev11",
                pretheta = s$strup.mdfy)
stopifnot(is.matrix(i3), nrow(i3) == 4)

message("All tests passed.")

# --------------------------------------------------------------
# Compute GN16 parameter estimates for the NS GEV11 model
# --------------------------------------------------------------

#' GN16 estimation for the non-stationary GEV11 model
#'
#' @description
#' Estimates the parameters of the non-stationary GEV11 model by the
#' quantile-based GN16 method (Gilleland and Katz, 2016; as adapted in
#' Shin et al., 2025b), with both the original and the modified
#' specification returned.
#'
#' @param xdat A numeric vector of data to be fitted.
#' @param rob If TRUE, uses robust regression in the internal steps
#'   (default FALSE).
#'
#' @return A list containing:
#' \itemize{
#'   \item para.gado.org - Original GN16 estimates
#'     (mu0, mu1, sigma0, sigma1, xi).
#'   \item para.gado.mdfy - Modified GN16 estimates.
#' }
#'
#' @references
#' Gilleland, E. & Katz, R. W. (2016). extRemes 2.0: An extreme value
#' analysis package in R. Journal of Statistical Software, 72(8), 1-39.
#'
#' Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
#' model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
#' \doi{10.1007/s42952-025-00325-3}
#'
#' @seealso \code{\link{glme.gev11}}, \code{\link{lme.gev11}},
#'   \code{\link{strup.gev11}}.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' data(PhliuAgromet)
#' result <- GN16.gev11(PhliuAgromet$prec)
#' print(result$para.gado.org)
#'
#' @export
GN16.gev11 = function(xdat, rob=FALSE){

 year=seq(1,length(xdat))
 reg.dat=data.frame( cbind(year, xdat) )

 mu.init= lm(xdat~year, reg.dat)$coefficients

 orig.para=c(mu.init[1],mu.init[2], 1,.1,0)

 qlist= make.qmax.gev11(xdat, orig.para=orig.para, rob=rob)

 orig.para=c(mu.init[1],mu.init[2], qlist$sig0, qlist$sig1, 0)

 time.m.gev11(qmax=qlist$qmax, orig.para=orig.para,
              rob=rob)
}

#-----------------------------------------------------
#' Construct pseudo-maxima for the GN16 method
#'
#' @description Internal step of \code{\link{GN16.gev11}}: builds the
#' adjusted residual series (qmax) and estimates the log-scale trend
#' coefficients.
#'
#' @param xdat Numeric data vector.
#' @param orig.para Initial parameter vector (mu0, mu1, sigma0, sigma1, xi).
#' @param rob If TRUE, uses robust regression.
#' @return A list with sig0, sig1, and qmax.
#' @keywords internal
make.qmax.gev11 =function(xdat=NULL, orig.para=NULL, rob=FALSE)
{
  z=list()
  ns = length(xdat)
  year= seq(1,ns)

  res = xdat - (orig.para[1] + orig.para[2]* year)
  mres= mean(res)
  res.pr = abs(res - mres)

  lres.pr=log(res.pr)
  sig.dat=data.frame( cbind(year, lres.pr) )

  if(rob==FALSE){
    sig.lm= lm(lres.pr~year, sig.dat)$coefficients

  }else{
    sig.lm= lmrob(lres.pr~year, sig.dat)$coefficients
  }

  z$sig0 = sig.lm[1]
  z$sig1 = sig.lm[2]
  sigt = exp(z$sig0 + z$sig1* year)

  qmax= rep(NA, ns)

  for(i in 1:ns){
    if(z$sig1 >= 0){
      if(res[i] >= mres ) {
        qmax[i] = res[i] - sigt[i]
      }else{
        qmax[i] = res[i] + sigt[i]
      }
    }else{
      if(res[i] >= mres ) {
        qmax[i] = res[i] + sigt[i]
      }else{
        qmax[i] = res[i] - sigt[i]
      }
    }
  }
  z$qmax=qmax
  return(z)
}

#------------------------------------------------------------------
#' Time-varying moment step of the GN16 method
#'
#' @description Internal step of \code{\link{GN16.gev11}}: fits a
#' stationary GEV to the pseudo-maxima and converts the result to the
#' GEV11 parameterization (both original and modified specification).
#'
#' @param qmax Pseudo-maxima series from \code{\link{make.qmax.gev11}}.
#' @param orig.para Parameter vector (mu0, mu1, sigma0, sigma1, xi).
#' @param rob If TRUE, uses robust regression for the location trend.
#' @return A list with para.gado.org and para.gado.mdfy.
#' @keywords internal
time.m.gev11 = function(qmax=NULL, orig.para=NULL, rob=FALSE){

  z=list()
  ns=length(qmax)
  year=seq(1,ns)

  sig0=orig.para[3]
  sig1=orig.para[4]

  q.sta = pargev(lmoms(qmax, nmom=3), checklmom=FALSE)$para
  xi= q.sta[3]

  if( xi <= -0.5) xi = -0.499

  cd = sqrt( (xi^2) /( gamma(1+2*xi) - gamma(1+xi)^2 ) )
  alpha_t = exp(sig0 +sig1*year) * cd
  mu.gado = ( -( 1-gamma(1+ xi) )*alpha_t/xi
              + orig.para[1]+ orig.para[2]*year )

  #------------------------------
  nh=round((ns/2))
  mu.gado[nh-1] = mu.gado[nh-1] + 0.02
  mu.gado[nh+1] = mu.gado[nh+1] - 0.02

  mu.data= data.frame( cbind(year, mu.gado) )
  if(rob==TRUE){
    loc.gado =lmrob(mu.gado~year, mu.data)$coefficients

  }else{
    loc.gado =lm(mu.gado~year, mu.data)$coefficients
  }

  alpha0 = log(cd) + sig0

  z$para.gado.org= c(loc.gado, alpha0, sig1, xi)

  # modify GN16 ---------------

  alpha0_up = log(q.sta[2])- sig1*nh
  z$para.gado.mdfy = c(loc.gado, alpha0_up, sig1, xi)

  names(z$para.gado.org) <- names(z$para.gado.mdfy) <- c("mu0",
                                 "mu1","sigma0","sigma1","xi")
  return(z)
}

# Internal helper functions for MAGEV (Model Averaging GEV)
# These functions are not exported and are used internally by ma.gev()
#
# Reference: Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with
# mixed criteria for estimating high quantiles of extreme values: Application
# to heavy rainfall. Stochastic Environmental Research and Risk Assessment,
# 40(2), 47. https://doi.org/10.1007/s00477-025-03167-x


#' MLE for GEV distribution using constrained optimization
#'
#' @description Internal function that computes maximum likelihood estimates
#' of GEV parameters using the Rsolnp constrained optimizer with multiple
#' random starting points.
#'
#' @param xdat Numeric vector of data.
#' @param ntry Number of random starting points for optimization. Default is 5.
#'
#' @return A list containing:
#' \describe{
#'   \item{nsample}{Sample size}
#'   \item{conv}{Convergence status (0 = success)}
#'   \item{nllh}{Negative log-likelihood at the optimum}
#'   \item{mle}{MLE estimates (mu, sigma, xi) in Hosking style}
#' }
#'
#' @keywords internal
gev.max <- function(xdat, ntry = 5) {
  z <- list()
  k <- list()
  n <- ntry

  nsample <- length(xdat)
  z$nsample <- nsample

  init <- matrix(0, nrow = ntry, ncol = 3)
  init <- init.gevmax(xdat, ntry)
  upsig <- sqrt(var(xdat)) * 5
  upmu <- abs(mean(xdat)) * 3

  gev.lik.max <- function(a) {
    mu <- a[1]
    sc <- a[2]
    xi <- a[3]

    y <- (xdat - mu) / sc
    y <- 1 - xi * y  # Hosking style

    for (i in 1:nsample) {
      y[i] <- max(0, y[i], na.rm = TRUE)
    }

    if (any(y <= 0) || any(sc <= 0))
      return(10^6)

    if (abs(xi) >= 10^(-5)) {
      ooxi <- 1 / xi
    } else {
      ooxi <- sign(xi) * 10^5
    }

    zz <- nsample * (log(sc)) + sum(exp(ooxi * log(y))) + sum(log(y) * (1 - (ooxi)))
    return(zz)
  }

  tryCatch(
    for (i in 1:nrow(init)) {
      value <- try(solnp(init[i, ], fun = gev.lik.max,
                         LB = c(-upmu, 0, -1), UB = c(upmu, upsig, 1),
                         control = list(trace = 0, outer.iter = 10,
                                        delta = 1.e-6, inner.iter = 200, tol = 1.e-5)))

      if (is(value)[1] == "try-error") {
        k[[i]] <- list(value = 10^6)
      } else {
        k[[i]] <- value
      }
    }
  )

  optim_value <- data.frame(num = 1:n, value = sapply(k, function(x) x$value[which.min(x$value)]))
  optim_table1 <- optim_value[order(optim_value$value), ]
  selc_num <- optim_table1[1, "num"]

  x <- k[[selc_num]]

  z$conv <- x$convergence
  z$nllh <- x$value[which.min(x$value)]
  z$mle <- x$par

  return(z)
}


#' @noRd
init.gevmax <- function(data = NULL, ntry = NULL) {
  if (ntry < 2) ntry <- 2
  n <- ntry
  init <- matrix(NA, ntry, 3)

  lmom_init <- lmoms(data, nmom = 3)
  lmom_est <- pargev(lmom_init)

  init[1, 1] <- lmom_est$para[1]
  init[1, 2] <- lmom_est$para[2]
  init[1, 3] <- lmom_est$para[3]

  maxm1 <- ntry
  maxm2 <- maxm1 - 1
  init[2:maxm1, 1] <- init[1, 1] + rnorm(n = maxm2, mean = 0, sd = 5)
  init[2:maxm1, 2] <- init[1, 2] + rnorm(n = maxm2, mean = 3, sd = 3)
  init[2:maxm1, 3] <- runif(n = maxm2, min = -0.45, max = 0.4)
  init[2:maxm1, 2] <- pmax(0.1, init[2:maxm1, 2])

  return(init)
}


#' GEV parameter estimation with fixed shape parameter (MAGEV internal)
#'
#' @description Internal function that estimates GEV location and scale
#' parameters given a fixed shape parameter, using L-moment equations.
#'
#' @param lmom L-moments object from \code{lmomco::lmoms()}.
#' @param xifix Fixed shape parameter value. Default is 0.1.
#' @param checklmom Logical. Whether to check L-moment validity. Default is TRUE.
#' @param ... Additional arguments (unused).
#'
#' @return A list with components:
#' \describe{
#'   \item{type}{Character "gev"}
#'   \item{para}{Numeric vector of GEV parameters (mu, sigma, xi)}
#'   \item{source}{Character "pargev"}
#' }
#'
#' @keywords internal
pargev.xifix <- function(lmom, xifix = 0.1, checklmom = TRUE, ...) {
  para <- rep(NA, 3)
  G <- xifix
  para[3] <- G

  if (length(lmom$lambdas[1]) == 0) {
    lmom <- lmorph(lmom)
  }

  SMALL <- 1e-05
  if (abs(G) < SMALL) {
    para[1:2] <- pargum(lmom)$para
    return(list(type = "gev", para = para, source = "pargev"))
  }

  GAM <- exp(lgamma(1 + G))
  para[2] <- lmom$lambdas[2] * G / (GAM * (1 - 2^(-G)))
  para[1] <- lmom$lambdas[1] - para[2] * (1 - GAM) / G

  return(list(type = "gev", para = para, source = "pargev"))
}


#' MLE with return level and delta method SE
#'
#' @description Internal function that computes GEV MLE using both
#' constrained optimization (solnp) and ismev::gev.fit, selects the
#' better fit, and returns the covariance matrix for delta method SE.
#'
#' @param data Numeric vector of data.
#' @param ntry Number of optimization attempts. Default is 5.
#' @param quant Numeric vector of probabilities for quantile estimation.
#'
#' @return A list containing:
#' \describe{
#'   \item{nllh}{Negative log-likelihood}
#'   \item{mle}{MLE estimates (mu, sigma, xi) in Hosking style}
#'   \item{qua.mle}{Quantile estimates at \code{quant} probabilities}
#'   \item{data}{Input data}
#'   \item{cov}{3x3 covariance matrix from MLE}
#'   \item{quant}{Input quantile probabilities}
#' }
#'
#' @keywords internal
gev.rl.delta <- function(data, ntry = 5, quant) {
  zg <- list()
  numq <- length(quant)

  kmle.h <- gev.max(data, ntry = ntry)

  kmle.coles <- gev.fit(data, show = FALSE)

  if (kmle.h$nllh < kmle.coles$nllh) {
    if (abs(kmle.h$mle[3]) >= 0.49) kmle.h$mle[3] <- sign(kmle.h$mle[3]) * 0.49
    better <- tryCatch(gev.fit(data, show = FALSE, muinit = kmle.h$mle[1],
                               siginit = kmle.h$mle[2], shinit = -kmle.h$mle[3]))

    better.mle <- better$mle
    better.mle[3] <- -better$mle[3]
  } else {
    better <- kmle.coles
    better.mle <- kmle.coles$mle
    better.mle[3] <- -kmle.coles$mle[3]
  }

  if (abs(better.mle[3]) >= 1) {
    better.mle[3] <- sign(better.mle[3]) * 0.98
  }

  savem <- vec2par(better.mle, 'gev')
  zg$nllh <- better$nllh
  zg$mle <- better.mle
  zg$qua.mle <- quagev(quant[1:numq], savem)
  zg$data <- data
  zg$cov <- better$cov
  zg$quant <- quant

  return(zg)
}


#' Initialize parameters for GEV with fixed xi
#'
#' @description Internal function that generates initial (mu, sigma) values
#' for optimization with fixed shape parameter, using Gumbel L-moment estimates
#' and random perturbations.
#'
#' @param data Numeric vector of data.
#' @param ntry Number of initial parameter sets to generate.
#'
#' @return A matrix with \code{ntry} rows and 2 columns (mu, sigma).
#'
#' @keywords internal
ginit.xifix <- function(data, ntry) {
  initx <- matrix(0, nrow = ntry, ncol = 2)

  lmom_init <- lmoms(data, nmom = 5)
  lmom_est <- pargum(lmom_init, checklmom = TRUE)

  initx[1, 1] <- lmom_est$para[1]
  initx[1, 2] <- lmom_est$para[2]

  maxm1 <- ntry
  maxm2 <- maxm1 - 1
  initx[2:maxm1, 1] <- initx[1, 1] + rnorm(n = maxm2, mean = 0, sd = 7)
  initx[2:maxm1, 2] <- initx[1, 2] + rnorm(n = maxm2, mean = 2, sd = 2)
  initx[2:maxm1, 2] <- pmax(0.1, initx[2:maxm1, 2])

  return(initx)
}


#' MLE for GEV with fixed shape parameter (single candidate)
#'
#' @description Internal function that computes MLE of GEV location and scale
#' parameters with a fixed shape parameter using L-BFGS-B optimization.
#' Optionally computes Prescott-Walden Hessian for variance estimation.
#'
#' @param xdat Numeric vector of data.
#' @param xifix Fixed shape parameter value. Default is -0.1.
#' @param ntry Number of optimization attempts. Default is 5.
#' @param varcom Logical. If TRUE, computes Prescott-Walden covariance matrix.
#'
#' @return A list of class "gev.xifix" containing:
#' \describe{
#'   \item{conv}{Convergence status (0 = success)}
#'   \item{nllh}{Negative log-likelihood at the optimum}
#'   \item{mle}{MLE estimates (mu, sigma, xi)}
#'   \item{cov}{2x2 covariance matrix (if \code{varcom=TRUE})}
#' }
#'
#' @keywords internal
gev.xifix.sing <- function(xdat = NULL, xifix = -0.1, ntry = 5, varcom = NULL) {
  zx <- list()
  kx <- list()
  value <- list()

  init.xi <- matrix(0, nrow = ntry, ncol = 2)
  init.xi <- ginit.xifix(xdat, ntry)
  xi <- xifix

  upsig <- diff(quantile(xdat, probs = c(0.25, 0.75))) * 5
  upmu <- abs(median(xdat)) * 3

  tryCatch(
    for (itry in 1:nrow(init.xi)) {
      value <- try(optim(init.xi[itry, ], fn = gev.xilik2,
                         lower = c(-upmu, 0), upper = c(upmu, upsig),
                         method = "L-BFGS-B",
                         xifix = xifix, xdat = xdat))

      if (is(value)[1] == "try-error") {
        kx[[itry]] <- list(value = 10^6)
      } else {
        kx[[itry]] <- value
      }
    }
  )

  optim_value <- data.frame(num = 1:ntry, value = sapply(kx, function(x) x$value[which.min(x$value)]))
  optim_table1 <- optim_value[order(optim_value$value), ]
  selc_num <- optim_table1[1, "num"]

  x <- kx[[selc_num]]

  zx$conv <- x$convergence
  zx$nllh <- x$value[which.min(x$value)]

  if (zx$conv != 0) {
    zx$mle <- NA
  } else {
    zx$mle <- x$par
    zx$mle[3] <- xifix
    if (varcom == TRUE) {
      Hess <- PrescottW(par2 = x$par, xifix = xifix, nsam = length(xdat))
      if (is.na(Hess[1, 1])) {
        zx$cov <- matrix(NA, 2, 2)
      } else if (det(Hess) <= 0) {
        zx$cov <- matrix(NA, 2, 2)
      } else {
        zx$cov <- solve(Hess)
      }
    }
  }

  class(zx) <- "gev.xifix"
  return(zx)
}


#' Prescott-Walden expected information matrix for GEV with fixed xi
#'
#' @description Internal function that computes the Prescott-Walden expected
#' information (Hessian) matrix for GEV parameters (mu, sigma) with fixed xi.
#'
#' @param par2 Numeric vector of (mu, sigma) estimates.
#' @param xifix Fixed shape parameter value.
#' @param nsam Sample size.
#'
#' @return A 2x2 expected information matrix.
#'
#' @keywords internal
PrescottW <- function(par2 = NULL, xifix = NULL, nsam = NULL) {
  sig <- par2[2]
  xi <- xifix
  Hess <- matrix(NA, 2, 2)

  if (xi < 0.5) {
    pw <- (1 - xi)^2 * gamma(1 - 2 * xi)
    Hess[1, 1] <- nsam * pw / (sig^2)
    Hess[2, 2] <- nsam * (1 - 2 * gamma(2 - xi) + pw) / (sig^2 * xi^2)
    Hess[1, 2] <- nsam * (pw - gamma(2 - xi)) / (sig^2 * xi)
    Hess[2, 1] <- Hess[1, 2]
  }

  return(Hess)
}


#' Construct covariance matrix C for model averaging SE
#'
#' @description Internal function that constructs the cross-covariance matrix
#' between submodel quantile estimators using the delta method and quantile
#' correlation approximation.
#'
#' @param mywt Weight computation result list from \code{weight.com()}.
#' @param cov22 Array of 2x2 covariance matrices for each submodel (2 x 2 x numk).
#' @param quant Numeric vector of quantile probabilities.
#'
#' @return A list containing:
#' \describe{
#'   \item{MatC}{Array (numk x numk x numq) of cross-covariance values}
#'   \item{fin.se}{SE under fixed weights for MA}
#'   \item{fin.se.bma}{SE under fixed weights for BMA}
#'   \item{numk}{Number of submodels}
#'   \item{numq}{Number of quantiles}
#'   \item{wtgd}{MA weights}
#'   \item{bmaw}{BMA weights}
#' }
#'
#' @keywords internal
cons.MatC <- function(mywt, cov22, quant) {
  numk <- length(mywt$wtgd)
  numq <- length(quant)
  MatC <- array(NA, c(numk, numk, numq))
  cov2i <- matrix(NA, 2, 2)
  cov2j <- matrix(NA, 2, 2)

  para3 <- mywt$prob.call$mle3

  for (i in 1:numk) {
    mle3i <- para3[i, ]
    cov2i <- as.matrix(cov22[, , i])

    for (j in i:numk) {
      mle3j <- para3[j, ]
      cov2j <- as.matrix(cov22[, , j])

      if (any(is.na(cov2i)) | any(is.na(cov2j))) {
        MatC[i, j, 1:numq] <- NA
      } else {
        MatC[i, j, 1:numq] <- delta.gev(gevf3 = NULL, mle3i = mle3i, cov2i = cov2i,
                                        mle3j = mle3j, cov2j = cov2j,
                                        quant = quant, d3yes = FALSE)$covij[1:numq]
        MatC[j, i, 1:numq] <- MatC[i, j, 1:numq]
      }
    }
  }

  wtgd <- mywt$wtgd
  bmaw <- mywt$bmaw
  fin.se <- rep(NA, numq)
  fin.se.bma <- rep(NA, numq)

  for (iq in 1:numq) {
    fin.se[iq] <- sqrt(t(wtgd) %*% MatC[, , iq] %*% wtgd)
    fin.se.bma[iq] <- sqrt(t(bmaw) %*% MatC[, , iq] %*% bmaw)
  }

  zz <- list()
  zz$MatC <- MatC
  zz$fin.se <- fin.se
  zz$fin.se.bma <- fin.se.bma
  zz$numk <- numk
  zz$numq <- numq
  zz$wtgd <- wtgd
  zz$bmaw <- bmaw

  return(zz)
}


#' GEV negative log-likelihood with fixed xi (wrapper)
#'
#' @description Internal wrapper function that calls \code{gev.xilik()}.
#'
#' @param a Numeric vector of (mu, sigma).
#' @param xifix Fixed shape parameter value.
#' @param xdat Numeric vector of data.
#'
#' @return Negative log-likelihood value (scalar).
#'
#' @keywords internal
gev.xilik2 <- function(a, xifix = xifix, xdat = xdat) {
  gev.xilik(a, xifix = xifix, xdat = xdat)
}


#' GEV negative log-likelihood with fixed xi
#'
#' @description Internal function that computes the negative log-likelihood
#' for the GEV distribution with a fixed shape parameter (Hosking parameterization).
#'
#' @param a Numeric vector of (mu, sigma).
#' @param xifix Fixed shape parameter value.
#' @param xdat Numeric vector of data.
#'
#' @return Negative log-likelihood value (scalar). Returns 10^6 if invalid.
#'
#' @keywords internal
gev.xilik <- function(a, xifix = NULL, xdat = NULL) {
  mu <- a[1]
  sc <- a[2]
  xi <- xifix

  if (sc <= 0) return(10^6)

  nsam <- length(xdat)
  y <- rep(0, nsam)
  y <- (xdat - mu) / sc
  y <- 1 - xi * y  # Hosking style

  for (i in 1:nsam) {
    y[i] <- max(0, y[i], na.rm = TRUE)
  }

  if (any(y <= 0)) return(10^6)

  if (abs(xi) >= 10^(-5)) {
    ooxi <- 1 / xi
  } else {
    ooxi <- sign(xi) * 10^5
  }

  b2 <- sum(log(y)) * (1 - ooxi)
  b3 <- sum(exp(ooxi * log(y)))

  nllh <- nsam * (log(sc)) + b2 + b3
  return(nllh)
}


#' L-moment estimation with bootstrap standard errors
#'
#' @description Internal function that computes L-moment estimates of GEV
#' parameters and optionally performs bootstrap resampling to obtain
#' covariance matrices and standard errors for quantile estimates.
#'
#' @param data Numeric vector of data.
#' @param B Number of bootstrap samples.
#' @param quant Numeric vector of probabilities for quantile estimation.
#' @param boot Logical. If TRUE (default), perform bootstrap.
#' @param trim Left trimming level for L-moments (integer). Default is NULL (0).
#'
#' @return A list containing:
#' \describe{
#'   \item{lme}{L-moment estimates (mu, sigma, xi)}
#'   \item{qua.lme}{Quantile estimates from LME}
#'   \item{quant}{Input quantile probabilities}
#'   \item{cov.par}{3x3 covariance of bootstrap parameter estimates (if boot=TRUE)}
#'   \item{cov.lambda}{3x3 covariance of bootstrap L-moments (if boot=TRUE)}
#'   \item{qua.lme.se}{SE of quantile estimates from bootstrap (if boot=TRUE)}
#' }
#'
#' @keywords internal
lme.boots <- function(data, B = NULL, quant, boot = TRUE, trim = NULL) {
  z <- list()
  numq <- length(quant)
  sam.bootL <- list()
  lme.boot <- matrix(NA, nrow = B, ncol = 3)
  ratios <- matrix(NA, nrow = B, ncol = 3)
  lme.seboot <- matrix(NA, nrow = B, ncol = numq)
  med <- rep(NA, B)
  lambdas <- matrix(NA, nrow = B, ncol = 3)

  nsam <- length(data)
  re <- sort(data)
  lmt <- lmoms(data, nmom = 3)
  klmet <- pargev(lmt, checkmom = TRUE)
  z$lme <- klmet$para[1:3]
  savet <- vec2par(klmet$para[1:3], 'gev')
  z$qua.lme <- quagev(quant[1:numq], savet)
  z$quant <- quant

  if (boot == TRUE) {
    for (ib in 1:B) {
      sam.bootL[[ib]] <- sample(data, nsam, replace = TRUE)
      lm.boot <- lmoms(sam.bootL[[ib]], nmom = 3)

      lme.boot[ib, 1:3] <- pargev(lm.boot, checklmom = FALSE)$para[1:3]
      save <- vec2par(lme.boot[ib, 1:3], 'gev')
      lme.seboot[ib, 1:numq] <- quagev(quant[1:numq], save)

      lambdas[ib, 1:3] <- lm.boot$lambdas[1:3]
      ratios[ib, 1] <- lm.boot$lambdas[1]
      ratios[ib, 2:3] <- lm.boot$ratios[2:3]
      med[ib] <- median(sam.bootL[[ib]])
    }

    lme.bb <- ratios
    lme.bb[1:B, 1] <- med[1:B]
    z$cov.med.rat <- cov(lme.bb[, 1:3])

    lme.bb <- lambdas
    lme.bb[1:B, 1] <- med[1:B]
    z$cov.med.lam <- cov(lme.bb[, 1:3])

    z$cov.lambda <- cov(lambdas[, 1:3])
    z$B <- B
    z$lme.boot <- lme.boot
    z$cov.rat <- cov(ratios[, 1:3])
    z$cov.par <- cov(lme.boot[, 1:3])

    qua.lme.se <- rep(NA, numq)
    for (qi in 1:numq) {
      qua.lme.se[qi] <- sqrt(var(lme.seboot[1:B, qi], na.rm = TRUE))
    }
    z$qua.lme.se <- qua.lme.se
    z$qua.lme.boot <- lme.seboot
  }

  return(z)
}


#' Revised Coles-Dixon prior function
#'
#' @description Internal function that computes a revised Coles-Dixon prior
#' probability for the shape parameter, used as a penalty in BMA.
#'
#' @param xi Numeric vector of shape parameter values.
#'
#' @return Numeric vector of prior probability values.
#'
#' @keywords internal
rcd <- function(xi) {
  minxi <- -0.2
  nxi <- length(xi)
  pfxi <- rep(NA, nxi)

  for (i in 1:nxi) {
    if (is.na(xi[i]) | is.null(xi[i])) {
      pfxi[i] <- 0
    } else {
      if (-1.0 < xi[i] & xi[i] <= minxi) {
        b1 <- -1 / (1 + minxi)
        b0 <- 1 - b1 * minxi
        pfxi[i] <- b0 + b1 * xi[i]
      }
    }
    if (xi[i] <= -1.0 | xi[i] > minxi) pfxi[i] <- 1
  }

  return(pfxi)
}


#' Empirical prior for BMA based on MLE and LME
#'
#' @description Internal function that computes an empirical prior distribution
#' for the shape parameter based on the range between MLE and LME estimates.
#'
#' @param mle3 Matrix of candidate submodel parameters (numk x 3).
#' @param mle MLE parameter estimates (mu, sigma, xi).
#' @param lme LME parameter estimates (mu, sigma, xi).
#'
#' @return Numeric vector of prior probability values for each candidate.
#'
#' @keywords internal
emp.prior <- function(mle3, mle, lme) {
  z <- list()
  xi <- mle3[, 3]
  nxi <- length(xi)
  pfxi <- rep(NA, nxi)

  mnx <- min(mle[3], lme[3])
  mxx <- max(mle[3], lme[3])
  u <- -0.6
  u0 <- -0.0

  if (mxx <= u) {
    pfxi <- 1
    return(pfxi)
  }
  if (mnx <= u) {
    mnx <- u + 0.05
    b <- 0
  } else {
    b <- 1 / (mnx - u - u0)
  }
  b2 <- 1 / (mnx - u - u0)

  for (i in 1:nxi) {
    if (is.na(xi[i]) | is.null(xi[i])) {
      pfxi[i] <- 0
    } else {
      if (u + u0 < xi[i] & xi[i] < mnx) {
        pfxi[i] <- (xi[i] - u - u0) * b
      } else if (mnx <= xi[i] & xi[i] <= mxx) {
        pfxi[i] <- 1
      } else if (mxx < xi[i] & xi[i] < mxx + mnx - u) {
        pfxi[i] <- 1 + (mxx - xi[i]) * b2
      } else if (xi[i] <= u + u0 | xi[i] >= mxx + mnx - u) {
        pfxi[i] <- 0
      }
    }
  }

  return(pfxi)
}


#' Moving average smoother for quantiles and weights
#'
#' @description Internal function that applies moving average smoothing
#' to quantile estimates and weights across candidate submodels.
#'
#' @param order Order of the moving average.
#' @param numk Number of candidate submodels.
#' @param numq Number of quantile probabilities.
#' @param zp Matrix (numq x numk) of quantile estimates.
#' @param wt Numeric vector of weights (length numk).
#'
#' @return A list containing:
#' \describe{
#'   \item{ez}{Matrix (numq x numk) of smoothed quantile estimates}
#'   \item{ew}{Numeric vector of smoothed weights}
#' }
#'
#' @keywords internal
movave <- function(order, numk, numq, zp, wt = NULL) {
  za <- list()
  q <- order
  ez <- matrix(NA, numq, numk)
  ew <- rep(NA, numk)

  if (numk >= (q + 1)) {
    for (ip in 1:numq) {
      ez[ip, ] <- rollmean(zp[ip, 1:numk], q, fill = "extend", align = "center")
    }
  } else if (numk <= q) {
    for (ip in 1:numq) {
      for (ik in 1:numk) {
        ez[ip, ik] <- mean(zp[ip, 1:numk])
      }
    }
  }

  if (numk >= (q + 1)) {
    ew <- rollmean(wt, q, fill = "extend", align = "center")
  } else if (numk <= q) {
    for (ik in 1:numk) {
      ew[ik] <- mean(wt[1:numk])
    }
  }

  za$ez <- ez
  za$ew <- ew
  return(za)
}


#' Delta method variance and cross-covariance for GEV quantiles
#'
#' @description Internal function that computes delta method variances
#' for GEV quantile estimates. Can compute either the variance from
#' a full 3-parameter MLE (d3yes=TRUE) or the cross-covariance between
#' two submodels with fixed xi (d3yes=FALSE).
#'
#' @param gevf3 Full MLE result list with \code{cov} and \code{mle} (used when d3yes=TRUE).
#' @param mle3i Parameter vector for submodel i (used when d3yes=FALSE).
#' @param cov2i 2x2 covariance matrix for submodel i.
#' @param mle3j Parameter vector for submodel j.
#' @param cov2j 2x2 covariance matrix for submodel j.
#' @param quant Numeric vector of quantile probabilities.
#' @param d3yes Logical. If TRUE, compute 3-parameter delta method variance.
#'
#' @return A list containing:
#' \describe{
#'   \item{v3}{Variance for each quantile (if d3yes=TRUE)}
#'   \item{covij}{Cross-covariance for each quantile (if d3yes=FALSE)}
#' }
#'
#' @keywords internal
delta.gev <- function(gevf3 = NULL, mle3i = NULL, cov2i = NULL,
                      mle3j = NULL, cov2j = NULL,
                      quant = NULL, d3yes = FALSE) {
  z <- list()
  numq <- length(quant)
  p <- 1 - quant
  yp <- -log(1 - p)

  if (d3yes == TRUE) {
    c3 <- gevf3$cov
    mle <- gevf3$mle
    xi <- mle[3]
    sig <- mle[2]
    d12 <- cbind(rep(1, numq), (1 - yp^(xi)) / xi)
    d3 <- sig * (1 - yp^(xi)) / (xi^2) + sig * (yp^(xi)) * log(yp) / xi
    delta3 <- cbind(d12, d3)

    for (iq in 1:numq) {
      z$v3[iq] <- delta3[iq, ] %*% c3 %*% delta3[iq, ]
    }
    return(z)
  }

  xi2i <- mle3i[3]
  xi2j <- mle3j[3]

  qqc <- c(seq(0.1, 0.9, 0.1), 0.95, 0.98, 0.99, 0.995, 0.998, 0.999)
  zp.cori <- quagev(qqc, vec2par(mle3i, 'gev'))
  zp.corj <- quagev(qqc, vec2par(mle3j, 'gev'))
  corr.rt <- cor(zp.cori, zp.corj)

  covij <- rep(NA, length(quant))
  vi <- rep(NA, numq)
  vj <- rep(NA, numq)
  deltai <- matrix(NA, 2, numq)
  deltaj <- matrix(NA, 2, numq)

  for (id in 1:length(quant)) {
    deltai[1:2, id] <- c(1, (1 - yp[id]^(xi2i)) / xi2i)
    deltaj[1:2, id] <- c(1, (1 - yp[id]^(xi2j)) / xi2j)

    vi[id] <- t(deltai[, id]) %*% cov2i %*% deltai[, id]
    vj[id] <- t(deltaj[, id]) %*% cov2j %*% deltaj[, id]

    covij[id] <- corr.rt * sqrt(vi[id] * vj[id])
  }

  z$covij <- covij
  return(z)
}


#' Find surrogate GEV parameters for model-averaged quantiles
#'
#' @description Internal function that fits a single GEV distribution to
#' the model-averaged quantile curve, producing surrogate GEV parameters
#' that approximate the model-averaged quantile function.
#'
#' @param zpf.surr Numeric vector of model-averaged quantiles at \code{xqa}.
#' @param xqa Numeric vector of probabilities at which quantiles are evaluated.
#' @param init.surr Initial GEV parameter estimates for optimization.
#'
#' @return A list (from \code{optim()}) with additional components:
#' \describe{
#'   \item{par}{Surrogate GEV parameters (mu, sigma, xi)}
#'   \item{zp.surrmodel}{Quantiles from the surrogate model at \code{xqa}}
#'   \item{zp.MA}{The input model-averaged quantiles}
#' }
#'
#' @keywords internal
surrogate <- function(zpf.surr = NULL, xqa, init.surr) {
  press.fun <- function(par) {
    theta <- par
    if (theta[2] <= 0) return(10^10)
    if (theta[3] <= -1.0 | theta[3] >= 1) return(10^10)
    mzp <- quagev(xqa, vec2par(theta, 'gev'))
    fun <- sum((zpf.surr - mzp)^2)
    return(fun)
  }

  surrx <- list()
  surrx <- optim(par = init.surr, fn = press.fun)

  surrx$zp.surrmodel <- quagev(xqa, vec2par(surrx$par, 'gev'))
  surrx$zp.MA <- zpf.surr
  return(surrx)
}


#' Interpolate missing covariance matrices across submodels
#'
#' @description Internal function that fills in missing 2x2 covariance matrices
#' for submodels using natural spline interpolation across the shape parameter.
#'
#' @param numk Number of candidate submodels.
#' @param para3 Matrix (numk x 3) of submodel parameters.
#' @param cov2 List of 2x2 covariance matrices (one per submodel).
#'
#' @return Array (2 x 2 x numk) of covariance matrices with missing values
#'   filled by interpolation.
#'
#' @keywords internal
cov.interp <- function(numk, para3, cov2 = NULL) {
  cov22 <- array(NA, c(2, 2, numk))

  for (ik in 1:numk) {
    cov22[, , ik] <- cov2[[ik]]
  }

  id <- which(!is.na(cov22[1, 1, ]))
  id.na <- which(is.na(cov22[1, 1, ]))
  if (length(id.na) == numk) stop("all cov22 are NA")

  if (length(id) < numk) {
    xsp <- para3[id, 3]
    y1 <- cov22[1, 1, id]
    y2 <- cov22[2, 2, id]
    y12 <- cov22[1, 2, id]

    covint <- matrix(NA, 3, numk)
    covint[1, id.na] <- spline(x = xsp, y1, method = "natural", xout = para3[id.na, 3])$y
    covint[2, id.na] <- spline(x = xsp, y2, method = "natural", xout = para3[id.na, 3])$y
    covint[3, id.na] <- spline(x = xsp, y12, method = "natural", xout = para3[id.na, 3])$y

    cov22[1, 1, id.na] <- covint[1, id.na]
    cov22[2, 2, id.na] <- covint[2, id.na]
    cov22[1, 2, id.na] <- covint[3, id.na]
    cov22[2, 1, id.na] <- covint[3, id.na]
  }

  return(cov22)
}


#' Asymptotic variance of model-averaged quantile estimates
#'
#' @description Internal function that computes asymptotic standard errors
#' for model-averaged quantile estimates under both fixed-weight and
#' random-weight assumptions.
#'
#' @param mywt Weight computation result list from \code{weight.com()}.
#' @param covint Array (2 x 2 x numk) of interpolated covariance matrices.
#' @param qqq Numeric vector of quantile probabilities.
#' @param order Moving average order for smoothing. Default is 2.
#'
#' @return A list containing:
#' \describe{
#'   \item{fin.se.MA.qua}{SE under fixed weights for MA}
#'   \item{fin.se.bma.qua}{SE under fixed weights for BMA}
#'   \item{adj.se.MA.qua}{SE under random weights for MA}
#'   \item{adj.se.bma.qua}{SE under random weights for BMA}
#'   \item{MatC}{Cross-covariance array (numk x numk x numq)}
#' }
#'
#' @keywords internal
asymp.var <- function(mywt, covint, qqq = NULL, order = 2) {
  zx <- list()
  consC <- list()
  wtgd <- mywt$wtgd
  bmaw <- mywt$bmaw
  numq <- length(qqq)
  numk <- length(wtgd)
  zp <- mywt$zp

  consC <- cons.MatC(mywt, covint, quant = qqq)

  zx$fin.se.MA.qua <- consC$fin.se
  zx$fin.se.bma.qua <- consC$fin.se.bma

  ezvar <- rep(NA, numq)
  ewvar <- rep(NA, numq)
  ewvar.bma <- rep(NA, numq)
  trCD <- rep(0, numq)
  trCD.bma <- rep(0, numq)

  D <- cov.dir(wtgd)
  Dbma <- cov.dir(bmaw)

  move <- movave(order, numk, numq, zp, wt = wtgd)
  ez <- move$ez
  ew <- move$ew

  moveb <- movave(order, numk, numq, zp, wt = bmaw)
  ewbma <- moveb$ew

  for (ip in 1:numq) {
    ezvar[ip] <- t(ez)[, ip] %*% D %*% ez[ip, ]
    ewvar[ip] <- t(ew) %*% consC$MatC[, , ip] %*% ew
    ewvar.bma[ip] <- t(ewbma) %*% consC$MatC[, , ip] %*% ewbma

    trCD[ip] <- sum(diag(consC$MatC[, , ip] %*% D)) + ezvar[ip]
    trCD.bma[ip] <- sum(diag(consC$MatC[, , ip] %*% Dbma)) + ezvar[ip]
  }

  zx$adj.se.MA.qua <- sqrt(ewvar + trCD)
  zx$adj.se.bma.qua <- sqrt(ewvar.bma + trCD.bma)
  zx$MatC <- consC$MatC

  return(zx)
}


#' Dirichlet covariance matrix for weights
#'
#' @description Internal function that computes the covariance matrix of
#' weights assuming a Dirichlet-type distribution.
#'
#' @param wtgd Numeric vector of model weights.
#'
#' @return A (numk x numk) covariance matrix.
#'
#' @keywords internal
cov.dir <- function(wtgd) {
  numk <- length(wtgd)
  D <- matrix(NA, numk, numk)

  for (i in 1:numk) D[i, i] <- wtgd[i] * (1 - wtgd[i]) / 2

  for (i in 1:(numk - 1)) {
    for (j in (i + 1):numk) {
      D[i, j] <- -wtgd[i] * wtgd[j] / 2
      D[j, i] <- D[i, j]
    }
  }

  return(D)
}


# ============================================================
# Coles-Dixon penalized MLE and Restricted MLE functions
# ============================================================

#' Coles-Dixon Penalized MLE for GEV
#'
#' Computes maximum penalized likelihood estimates for GEV parameters
#' using the Coles and Dixon (1999) prior penalty on the shape parameter.
#'
#' @param xdat Numeric vector of data.
#' @param ntry Number of random starting points for optimization. Default is 10.
#'
#' @return A list containing:
#'   \item{mle}{MLE estimates (mu, sigma, xi) in Hosking style}
#'   \item{nllh}{Negative log-likelihood at the optimum}
#'   \item{conv}{Convergence status (0 = success)}
#'   \item{nsample}{Sample size}
#'
#' @references
#' Coles, S., & Dixon, M. (1999). Likelihood-based inference for extreme
#' value models. Extremes, 2(1), 5-23.
#'
#' @keywords internal
mle.gev.CD <- function(xdat, ntry = 10) {
  z <- list()
  k <- list()
  n <- ntry

  nsample <- length(xdat)
  z$nsample <- nsample

  init <- matrix(0, nrow = ntry, ncol = 3)
  init <- init.gevmax(xdat, ntry)

  # Likelihood with Coles-Dixon penalty
  gev1.lik.CD.h <- function(a) {
    mu <- a[1]
    sc <- a[2]
    xi <- a[3]

    y <- (xdat - mu) / sc
    y <- 1 - xi * y  # Hosking style

    for (i in 1:nsample) {
      y[i] <- max(0, y[i], na.rm = TRUE)
    }

    if (any(y <= 0) || any(sc <= 0))
      return(10^6)

    if (abs(xi) >= 10^(-5)) {
      ooxi <- 1 / xi
    } else {
      ooxi <- sign(xi) * 10^5
    }

    zz <- nsample * (log(sc)) + sum(exp(ooxi * log(y))) + sum(log(y) * (1 - (ooxi)))

    # Coles-Dixon penalty
    zz <- zz - log(cd.hos(xi))

    return(zz)
  }

  mup <- abs(mean(xdat)) * 3
  sigup <- sqrt(var(xdat)) * 5

  tryCatch(
    for (i in 1:nrow(init)) {
      value <- try(solnp(init[i, ], fun = gev1.lik.CD.h,
                         LB = c(-mup, 0, -0.99), UB = c(mup, sigup, 0.499),
                         control = list(trace = 0, outer.iter = 20,
                                        delta = 1.e-7, inner.iter = 200, tol = 1.e-5)))

      if (is(value)[1] == "try-error") {
        k[[i]] <- list(value = 10^6)
      } else {
        k[[i]] <- value
      }
    }
  )

  optim_value <- data.frame(num = 1:n, value = sapply(k, function(x) x$value[which.min(x$value)]))
  optim_table1 <- optim_value[order(optim_value$value), ]
  selc_num <- optim_table1[1, "num"]

  x <- k[[selc_num]]

  z$conv <- x$convergence
  z$nllh <- x$value[which.min(x$value)]
  z$mle <- x$par  # Hosking style parameter

  return(z)
}


#' Coles-Dixon Penalty Function
#'
#' Computes the Coles-Dixon prior probability for the shape parameter.
#'
#' @param sxi Shape parameter value (Hosking style, negative for heavy tails).
#'
#' @return Prior probability value.
#'
#' @keywords internal
cd.hos <- function(sxi) {
  if (sxi >= 0) {
    pf <- 1.0
  } else if (sxi <= -1) {
    pf <- 1.e-20
  } else {
    pf <- exp(-((1 / (1 + sxi)) - 1))  # Hosking style xi
  }
  return(pf)
}


#' Restricted MLE for GEV (Mixed Estimation)
#'
#' Computes restricted maximum likelihood estimates for GEV parameters
#' with constraints on the mean or median matching the sample statistics.
#'
#' @param xdat Numeric vector of data.
#' @param ntry Number of random starting points. Default is 5.
#' @param rest Restriction type: 'mean' (default) or 'median'.
#' @param quant Probabilities for quantile estimation. Default is c(0.99, 0.995).
#' @param trim Left trimming level. Default is 0.
#' @param CD.mle Coles-Dixon MLE (optional). If NULL, computed internally.
#' @param mle Standard MLE (optional). If NULL, computed internally.
#' @param second Logical. If TRUE, compute second-stage REMLE. Default is TRUE.
#' @param w.mpse Logical. If TRUE, compute MPSE. Default is FALSE.
#'
#' @return A list containing:
#'   \item{remle1}{First-stage REMLE estimates}
#'   \item{qua.remle1}{Quantiles from first-stage REMLE}
#'   \item{remle2}{Second-stage REMLE estimates (if second=TRUE)}
#'   \item{qua.remle2}{Quantiles from second-stage REMLE}
#'   \item{rest.method}{Restriction method used}
#'
#' @keywords internal
remle.gev <- function(xdat, ntry = 5, rest = 'mean', quant = c(0.99, 0.995),
                      trim = 0, CD.mle = NULL, mle = NULL, second = TRUE,
                      w.mpse = FALSE) {
  z <- list()
  k <- list()
  data <- xdat
  if (rest == 'med') rest <- 'median'

  nsample <- length(xdat)
  z$nsample <- nsample

  if (is.null(CD.mle)) {
    CD.mle <- rep(NA, 3)
    CD.mle <- mle.gev.CD(xdat = data, ntry = 5)$mle  # Hosking style xi
  }
  if (is.null(mle)) {
    mle <- rep(NA, 3)
    mle <- gev.max(xdat = data, ntry = 5)$mle  # Hosking style xi
  }

  if (ntry < 4) ntry <- 4
  init <- matrix(NA, ntry, 3)
  init <- init.gevmax(data = xdat, ntry = ntry - 2)
  init <- rbind(init, matrix(NA, 2, 3))
  init[ntry - 1, 1:3] <- mle[1:3]
  init[ntry, 1:3] <- CD.mle[1:3]

  init <- na.omit(init)
  ntry <- nrow(init)

  upsig <- sqrt(var(xdat)) * 3
  upmu <- abs(mean(xdat)) * 3

  ntrim <- trim
  med.dat <- median(data)
  mean.dat <- mean(data)
  lmx <- lmoms(data, nmom = 3)

  nar <- c('rest.Ex', 'rest.med')
  if (rest == 'mean') {
    sel <- 1
  } else if (rest == 'median') {
    sel <- 2
  }

  # Define constraint functions in parent environment
  rest.Ex <- function(a, xdat, lmx, med.dat, mean.dat) {
    lam1 <- lmomgev(para = vec2par(a, 'gev'))$lambdas[1]
    return(lam1 - mean.dat)
  }

  rest.med <- function(a, xdat, lmx, med.dat, mean.dat) {
    med <- quagev(0.5, vec2par(a, 'gev'))
    return(med - med.dat)
  }

  rest.Ex2 <- function(a, xdat, lmx, med.dat, mean.dat) {
    lamp <- lmomgev(para = vec2par(a, 'gev'))$lambdas[1:2]
    return(c(lamp[1] - lmx$lambdas[1], lamp[2] - lmx$lambdas[2]))
  }

  rest.med2 <- function(a, xdat, lmx, med.dat, mean.dat) {
    medx <- quagev(0.5, vec2par(a, 'gev'))
    lam2 <- lmomgev(para = vec2par(a, 'gev'))$lambdas[2]
    return(c(medx - med.dat, lam2 - lmx$lambdas[2]))
  }

  gev.lik.remax <- function(a, xdat, lmx, med.dat, mean.dat) {
    mu <- a[1]
    sc <- a[2]
    xi <- a[3]

    y <- (xdat - mu) / sc
    y <- 1 - xi * y  # Hosking style

    nsample <- length(xdat)
    for (i in 1:nsample) {
      y[i] <- max(0, y[i], na.rm = TRUE)
    }

    if (any(y <= 0) || any(sc <= 0))
      return(10^6)

    if (abs(xi) >= 10^(-5)) {
      ooxi <- 1 / xi
    } else {
      ooxi <- sign(xi) * 10^5
    }

    zz <- nsample * (log(sc)) + sum(exp(ooxi * log(y))) + sum(log(y) * (1 - (ooxi)))
    return(zz)
  }

  # Select constraint function
  if (sel == 1) {
    eq_fun <- rest.Ex
  } else {
    eq_fun <- rest.med
  }

  tryCatch(
    for (i in 1:ntry) {
      work <- list()
      work <- try(solnp(init[i, ], fun = gev.lik.remax,
                        eqfun = eq_fun, eqB = 0,
                        LB = c(-upmu, 0, -0.99), UB = c(upmu, upsig, 0.99),
                        control = list(trace = 0, outer.iter = 20,
                                        delta = 1.e-6, inner.iter = 200, tol = 1.e-5),
                        xdat = xdat, lmx = lmx, med.dat = med.dat,
                        mean.dat = mean.dat))

      if (is(work)[1] == "try-error") {
        k[[i]] <- list(values = 10^6)
      } else if (work$convergence != 0) {
        k[[i]] <- list(values = 10^6)
      } else {
        k[[i]] <- work
      }
    }
  )

  optim_value <- data.frame(num = 1:ntry, values = sapply(k, function(x) x$values[which.min(x$values)]))
  optim_table1 <- optim_value[order(optim_value$values), ]
  selc_num <- optim_table1[1, "num"]
  x <- list()

  x <- k[[selc_num]]

  z$remle1.value <- x$values[which.min(x$values)]
  z$para.remle1 <- x$pars

  if (is.null(x$pars) | any(is.na(x$pars)) | z$remle1.value == 10^6) {
    x$pars <- NA
    z$para.remle1 <- NA
    z$qua.remle1 <- NA
  } else if (x$pars[2] <= 0 | abs(x$pars[3]) >= 1.0) {
    x$pars <- NA
    z$para.remle1 <- NA
    z$qua.remle1 <- NA
  } else {
    z$qua.remle1 <- quagev(quant, vec2par(x$pars, 'gev'))
  }

  z$rest.method <- rest

  if (second == TRUE) {
    k2 <- list()
    if (sel == 1) {
      eq_fun2 <- rest.Ex2
    } else {
      eq_fun2 <- rest.med2
    }

    init2 <- rbind(init, rep(NA, 3))
    init2[nrow(init2), 1:3] <- z$para.remle1[1:3]
    init2 <- na.omit(init2)
    xtry <- nrow(init2)

    tryCatch(
      for (i in 1:xtry) {
        value2 <- list()
        value2 <- try(solnp(init2[i, ], fun = gev.lik.remax,
                            eqfun = eq_fun2, eqB = c(0, 0),
                            LB = c(-upmu, 0, -0.99), UB = c(upmu, upsig, 0.99),
                            control = list(trace = 0, outer.iter = 20,
                                            delta = 1.e-6, inner.iter = 200, tol = 1.e-5),
                            xdat = xdat, lmx = lmx, med.dat = med.dat,
                            mean.dat = mean.dat))

        if (is(value2)[1] == "try-error") {
          k2[[i]] <- list(values = 10^6)
        } else if (value2$convergence != 0) {
          k2[[i]] <- list(values = 10^6)
        } else {
          k2[[i]] <- value2
        }
      }
    )

    optim_value <- data.frame(num = 1:xtry, values = sapply(k2, function(x) x$values[which.min(x$values)]))
    optim_table1 <- optim_value[order(optim_value$values), ]
    selc_num <- optim_table1[1, "num"]
    xw <- list()

    xw <- k2[[selc_num]]

    z$remle2.value <- xw$values[which.min(xw$values)]
    z$para.remle2 <- xw$pars

    if (is.null(xw$pars) | any(is.na(xw$pars)) | z$remle2.value == 10^6) {
      xw$pars <- NA
      z$para.remle2 <- NA
      z$qua.remle2 <- NA
    } else if (xw$pars[2] <= 0 | abs(xw$pars[3]) >= 1.0) {
      xw$pars <- NA
      z$para.remle2 <- NA
      z$qua.remle2 <- NA
    } else {
      z$qua.remle2 <- quagev(quant, vec2par(xw$pars, 'gev'))
    }
  }

  z$mpse <- NA
  z$qua.mpse <- NA

  return(z)
}


# ============================================================
# Weight computation functions (from weight.com.BMA_15Dec25.R)
# ============================================================

#' v1.x beta preference function (frozen copy for MAGEV)
#'
#' @description Frozen copy of the GLmom v1.x `pk.beta.stnary()`
#' implementation, kept so that the MAGEV BMA prior (`set.prior()`) is
#' unaffected by the v2.0.0 redesign of `pk.beta()` (different support,
#' out-of-support value, and q-adaptation trigger).
#'
#' @param para A vector of GEV parameters.
#' @param lme.center L-moment estimates as center.
#' @param p Shape parameter.
#' @param q Shape parameter q (optional, if provided uses fixed limits).
#' @param c0 Limit parameter (default 0.3).
#' @param c1 Scaling parameter (default 10).
#' @param c2 Upper limit parameter (default 5).
#' @return A list containing pk.one, p, and q.
#' @noRd
pk.beta.stnary.v1 = function(para=NULL, lme.center=NULL, p=NULL,
                             q=NULL, c0=0.3, c1=10, c2=5){

  pk=list()

  if(is.null(q)){

    ulim= c0
    aa= max(-1, lme.center[3]-ulim)
    bb= min(0.5, lme.center[3]+ulim)
    al=min(aa,bb)
    bl=max(aa,bb)

    if(lme.center[3] <= 0) {
      qlim= min(abs(lme.center[3])*c1, c2)
    }else{ qlim =0.0 }

    q=p+qlim

  }else if(!is.null(q)){
    lme.center[3]=0
    al= -0.5; bl=0.5
  }

  Be  <- integrate(Befun, lower=al, upper=bl,
                   al=al, bl=bl, p=p, q=q)[1]$value

  pk$pk.one=1
  if(lme.center[3] < 0.5){
    if( (para[3] > al) & (para[3] < bl) ) {
      pk$pk.one <- ((-al+para[3])^(p-1))*((bl-para[3])^(q-1))/ Be
    }}
  if(is.na(pk$pk.one)) pk$pk.one=1
  pk$p=p
  pk$q=q
  return(pk)
}


#' Beta prior for model averaging
#'
#' @description Internal function that computes a Beta distribution prior
#' probability for a candidate shape parameter in model averaging.
#'
#' @param x Shape parameter value to evaluate.
#' @param xi_lme L-moment estimate of xi (unused in this version).
#' @param p Beta shape parameter p. Default is 2.
#' @param q Beta shape parameter q. Default is 5.
#' @param al Lower bound of support.
#' @param bl Upper bound of support.
#'
#' @return Prior probability value (scalar).
#'
#' @keywords internal
prior.beta.stnary.ma <- function(x = NULL, xi_lme = NULL,
                                  p = 2, q = 5, al, bl) {
  Bef <- function(w) { ((-al + w)^(p - 1)) * ((bl - w)^(q - 1)) }
  Be <- integrate(Bef, lower = al, upper = bl)[1]$value

  if ((-al + x) <= 0) {
    left.w <- 0
  } else {
    left.w <- (-al + x)
  }
  if ((bl - x) <= 0) {
    right.w <- 0
  } else {
    right.w <- (bl - x)
  }

  prior <- (left.w^(p - 1)) * (right.w^(q - 1)) / Be
  return(prior)
}


#' Set BMA prior distribution for candidate shape parameters
#'
#' @description Internal function that sets up the prior distribution for
#' Bayesian Model Averaging (BMA) weights. Supports both normal and beta
#' priors, with hyperparameters adapted based on the weighting method and
#' L-moment estimate of xi.
#'
#' @param pen Prior type: "norm" (normal) or "beta".
#' @param numk Number of candidate submodels.
#' @param xi_lme L-moment estimate of the shape parameter.
#' @param kpar Numeric vector of candidate xi values.
#' @param weight Weighting method name ("like", "gLd", or "med").
#'
#' @return A list containing:
#' \describe{
#'   \item{prior}{Numeric vector of prior probabilities (length numk)}
#'   \item{prior_mu_std}{(if pen="norm") Mean and std of the normal prior}
#'   \item{p_q_beta}{(if pen="beta") p and q parameters of the beta prior}
#' }
#'
#' @keywords internal
set.prior <- function(pen = NULL, numk = NULL, xi_lme = NULL, kpar = NULL,
                      weight = NULL) {
  zp <- list()
  prior <- rep(NA, numk)
  lme <- xi_lme

  if (weight == 'like') {
    if (pen == "beta") {
      p <- 2
      c0 <- 0.35
      c1 <- 40
      c2 <- 12
    } else if (pen == "norm") {
      multa <- 2.2
      lme.cut <- -0.5
      std.cut <- -0.45
    }
  } else if (weight == 'gLd' | weight == 'med') {
    if (pen == "beta") {
      p <- 2
      c0 <- 0.35
      c1 <- 20
      c2 <- 7
    } else if (pen == "norm") {
      multa <- 1.5
      lme.cut <- -0.45
      std.cut <- -0.4
    }
  }

  if (pen == "norm") {
    if (lme >= 0) {
      prior <- rep(1 / numk, numk)
      mu.prior <- 0
      std.prior <- 1
    }

    if (lme < 0) {
      if (lme >= lme.cut) {
        mu.prior <- multa * lme
      } else if (lme < lme.cut) {
        mu.prior <- multa * lme.cut
      }

      if (weight == "like") {
        if (lme >= std.cut) {
          std.prior <- -(std.cut - lme) / 5 + 0.11
        } else {
          std.prior <- 0.11
        }
      }
      if (weight == "gLd" | weight == "med") {
        if (lme >= std.cut) {
          std.prior <- -(std.cut - lme) / 4 + 0.14
        } else {
          std.prior <- 0.14
        }
      }

      for (i in 1:numk) {
        prior[i] <- dnorm(x = kpar[i], mean = mu.prior, sd = std.prior)
      }
    }
  } else if (pen == "beta") {
    for (i in 1:numk) {
      pklist <- pk.beta.stnary.v1(para = c(1, 0, kpar[i]),
                                  lme.center = c(1, 0, xi_lme),
                                  p = p, c0 = c0, c1 = c1, c2 = c2)
      prior[i] <- pklist$pk.one
    }
  }

  zp$prior <- prior
  if (pen == "norm") zp$prior_mu_std <- c(mu.prior, std.prior)
  if (pen == "beta") zp$p_q_beta <- round(c(p, pklist$q), 2)
  return(zp)
}


#' Compute model averaging weights
#'
#' @description Internal function that computes weights for model averaging
#' across K candidate GEV submodels. Supports multiple weighting schemes
#' including likelihood-based, generalized L-moment distance, and median-based
#' methods. Optionally includes BMA prior integration.
#'
#' @param data Numeric vector of data.
#' @param numk Number of candidate submodels.
#' @param hosking List containing LME results, MLE, and bootstrap information.
#' @param kpar Numeric vector of candidate xi values.
#' @param numom Number of L-moments to use (default 3).
#' @param xqa Probability vector for surrogate model fitting.
#' @param varcom Logical. Whether to compute variance components (default TRUE).
#' @param boot.lme Logical. Whether bootstrap LME was performed (default TRUE).
#' @param cov.lme Pre-computed LME covariance (default NULL).
#' @param surr Logical. Whether to compute surrogate quantiles (default FALSE).
#' @param type Type of computation: "full" (default).
#' @param trim Left trimming level for L-moments.
#' @param cov.type Covariance type: "ratio" or "lambda" (default "ratio").
#' @param bma Logical. Whether to compute BMA weights (default TRUE).
#' @param pen BMA prior type: "norm" or "beta" (default "norm").
#'
#' @return A list containing:
#' \describe{
#'   \item{weight}{Weighting method name}
#'   \item{numk}{Number of submodels}
#'   \item{wtgd}{MA weight vector (length numk)}
#'   \item{bmaw}{BMA weight vector (length numk)}
#'   \item{zp}{Quantile matrix (numq x numk)}
#'   \item{kpar}{Candidate xi values}
#'   \item{prob.call}{Submodel fitting results}
#'   \item{xzp}{Surrogate quantile matrix (if surr=TRUE)}
#' }
#'
#' @keywords internal
weight.com <- function(data = NULL, numk = NULL, hosking = NULL,
                           kpar = NULL, numom = 3, xqa = NULL, varcom = TRUE,
                           boot.lme = TRUE, cov.lme = NULL, surr = FALSE,
                           type = 'full', trim = NULL, cov.type = 'ratio',
                           bma = TRUE, pen = "norm") {
  z <- list()
  quant <- hosking$quant
  weight <- hosking$weight
  B <- hosking$B
  start <- hosking$start

  if (bma == TRUE) {
    prior <- set.prior(pen = pen, numk = numk, xi_lme = hosking$lme[3],
                       kpar = kpar, weight = weight)$prior
  } else if (bma != TRUE) {
    prior <- rep(1 / numk, numk)
  }

  prob.call <- list()
  prob.call <- dist.noboot(data = data, numk = numk, hosking = hosking,
                           boot.lme = boot.lme,
                           kpar = kpar, numom = numom, ntry = 5, varcom = varcom,
                           cov.lme = cov.lme, trim = trim, cov.type = cov.type)

  notid <- which(!is.na(prob.call$mle3[, 1]))
  prob.call$mle3 <- prob.call$mle3[notid, ]
  mle3 <- prob.call$mle3
  numk <- length(notid)

  kpar <- kpar[notid]

  if (surr == TRUE) {
    nxq <- length(xqa)
    xzp <- matrix(NA, nrow = nxq, ncol = numk)
  }
  numq <- length(quant)

  zp <- matrix(NA, nrow = numq, ncol = numk)
  wtgd <- rep(NA, numk)
  bmaw <- rep(NA, numk)

  lme <- hosking$lme
  mle <- hosking$mle

  if (weight == 'gLd' | weight == 'med' | weight == 'cvt' | weight == 'fcv') {
    if (varcom == TRUE | weight == 'fcv') {
      cov2 <- list()
      for (i in 1:numk) cov2[[i]] <- prob.call$cov2[[notid[i]]]
    }

    if (weight == 'cvt') {
      naic <- prob.call$aic[notid]
      id <- which(is.na(naic))
      amin <- naic - min(naic, na.rm = TRUE)
      wtgd <- exp(-amin) / sum(exp(-amin), na.rm = TRUE)
      wtgd[id] <- 0
      bmaw <- prior * exp(-amin) / sum(prior * exp(-amin), na.rm = TRUE)
      bmaw[id] <- 0
    }

    if (weight == 'gLd' | weight == 'med' | weight == 'fcv') {
      if (weight == 'gLd') {
        kcol <- 1
      } else if (weight == 'med') {
        kcol <- 2
      }
      if (weight == 'fcv') {
        kcol <- 3
      }

      prob.mtx <- prob.call$prob.mtx[notid, ]
      wtgd[1:numk] <- prob.mtx[1:numk, kcol] / sum(prob.mtx[, kcol], na.rm = TRUE)
      bmaw[1:numk] <- prior * prob.mtx[1:numk, kcol] / sum(prior * prob.mtx[, kcol], na.rm = TRUE)
    }
  }

  if (weight == 'lcv' | weight == 'like' | weight == 'opt') {
    pertr <- hosking$pertr

    if (weight == 'lcv' | weight == 'like') {
      prob.call <- wlik.xifix(data, numk = numk, kpar = kpar, weight = weight,
                              pertr = pertr, varcom = varcom, type = type,
                              prior = prior, trim = trim)

      if (varcom == TRUE) {
        cov2 <- list()
        for (i in 1:numk) cov2[[i]] <- prob.call$cov2[[notid[i]]]
      }
    }

    wtgd <- prob.call$wt
    bmaw <- prob.call$bmaw
    prob.call$mle3 <- t(prob.call$kfix)

    for (ip in 1:numk) {
      save <- vec2par(prob.call$kfix[1:3, ip], 'gev')
      zp[1:numq, ip] <- quagev(quant[1:numq], save)
      if (surr == TRUE) xzp[1:nxq, ip] <- quagev(xqa[1:nxq], save)
    }
  } else {
    for (ip in 1:numk) {
      if (any(is.na(prob.call$mle3[ip, 1:3]))) {
        zp[1:numq, ip] <- NA
        if (surr == TRUE) xzp[1:nxq, ip] <- NA
      } else {
        save <- vec2par(prob.call$mle3[ip, 1:3], 'gev')
        zp[1:numq, ip] <- quagev(quant[1:numq], save)
        if (surr == TRUE) xzp[1:nxq, ip] <- quagev(xqa[1:nxq], save)
      }
    }
  }

  z$weight <- weight
  z$numk <- numk
  z$wtgd <- wtgd
  z$bmaw <- bmaw
  z$zp <- zp
  z$kpar <- kpar
  z$prob.call <- prob.call

  if (varcom == TRUE) {
    z$prob.call$cov2 <- cov2
  } else {
    z$prob.call$cov2 <- NA
  }
  if (surr == TRUE) z$xzp <- xzp

  return(z)
}


#' Likelihood-based weights with fixed xi
#'
#' @description Internal function that computes AIC-based model averaging
#' weights using the profile likelihood with L-moment submodel estimates
#' for each candidate xi.
#'
#' @param data Numeric vector of data.
#' @param numk Number of candidate submodels.
#' @param kpar Numeric vector of candidate xi values.
#' @param weight Weighting method name.
#' @param pertr Perturbation parameter (default 0.85).
#' @param varcom Logical. Whether to compute variance (default FALSE).
#' @param type Type of computation.
#' @param prior Numeric vector of BMA prior values.
#' @param trim Left trimming level.
#'
#' @return A list containing:
#' \describe{
#'   \item{bmaw}{BMA weight vector}
#'   \item{wt}{MA weight vector}
#'   \item{aic}{AIC values for each submodel}
#'   \item{kfix}{Matrix (3 x numk) of submodel parameters}
#'   \item{cov2}{List of 2x2 covariance matrices (if varcom=TRUE)}
#' }
#'
#' @keywords internal
wlik.xifix <- function(data = NULL, numk = NULL, kpar = NULL, weight = NULL,
                       pertr = 0.85, varcom = FALSE, type = NULL,
                       prior = NULL, trim = NULL) {
  z <- list()
  eprior <- prior
  kfix <- matrix(NA, nrow = 3, ncol = numk)
  nbic <- rep(NA, numk)
  bmin <- rep(NA, numk)
  wt <- rep(NA, numk)
  bmaw <- rep(NA, numk)
  para <- rep(NA, 3)
  lm3 <- list()
  re <- sort(data)
  nsam <- length(data)

  ntrim <- trim
  lmtrim <- lmoms(re[(ntrim + 1):nsam], nmom = 3)
  if (is.null(eprior)) eprior <- 1

  cov2.cv <- list()

  for (ip in 1:numk) {
    kfix[1:3, ip] <- pargev.xifix(lmtrim, xifix = kpar[ip])$para[1:3]

    if (varcom == TRUE) {
      Hess <- PrescottW(par2 = as.vector(kfix[1:2, ip]), xifix = kpar[ip],
                        nsam = length(re[(ntrim + 1):nsam]))

      if (is.na(Hess[1, 1])) {
        cov2.cv[[ip]] <- matrix(NA, 2, 2)
      } else if (det(Hess) <= 0) {
        cov2.cv[[ip]] <- matrix(NA, 2, 2)
      } else {
        cov2.cv[[ip]] <- solve(Hess)
      }
    }
  }

  if (weight == 'like') {
    for (ip in 1:numk) {
      ntrim <- trim
      dtrim <- re[(ntrim + 1):nsam]
      nbic[ip] <- gev.xilik(a = kfix[1:2, ip], xifix = kpar[ip], xdat = dtrim)
      if (nbic[ip] >= 10^6) nbic[ip] <- NA
    }

    numpar <- 2
    nbic <- 2 * nbic + 2 * numpar
    bmin <- nbic - min(nbic, na.rm = TRUE)
    wt <- exp(-bmin) / sum(exp(-bmin), na.rm = TRUE)
    bmaw <- prior * exp(-bmin) / sum(prior * exp(-bmin), na.rm = TRUE)
  }

  bmaw[which(is.na(bmaw))] <- 0
  wt[which(is.na(wt))] <- 0

  z$bmaw <- bmaw
  z$wt <- wt
  z$aic <- nbic
  z$kfix <- kfix
  if (varcom == TRUE) {
    z$cov2 <- cov2.cv
  }

  return(z)
}


#' Fit submodels and compute distance-based probabilities
#'
#' @description Internal function that fits GEV submodels with fixed xi for
#' each candidate and computes generalized L-moment distance or median-based
#' probabilities for weight construction.
#'
#' @param data Numeric vector of data.
#' @param numk Number of candidate submodels.
#' @param hosking List containing LME results and bootstrap information.
#' @param boot.lme Logical. Whether bootstrap LME was performed (default TRUE).
#' @param kpar Numeric vector of candidate xi values.
#' @param numom Number of L-moments to use.
#' @param ntry Number of optimization attempts. Default is 5.
#' @param varcom Logical. Whether to compute variance.
#' @param cov.lme Pre-computed LME covariance (default NULL).
#' @param trim Left trimming level.
#' @param cov.type Covariance type: "ratio" or "lambda" (default "lambda").
#'
#' @return A list containing:
#' \describe{
#'   \item{aic}{AIC values for each submodel}
#'   \item{mle3}{Matrix (numk x 3) of submodel parameter estimates}
#'   \item{kfix}{List of submodel fitting results}
#'   \item{cov2}{List of 2x2 covariance matrices (if varcom=TRUE)}
#'   \item{prob.mtx}{Matrix (numk x 3) of distance-based probabilities}
#'   \item{gdd}{Matrix (numk x 2) of generalized distances}
#' }
#'
#' @keywords internal
dist.noboot <- function(data = NULL, numk = NULL, hosking = NULL, boot.lme = TRUE,
                        kpar = NULL, numom = NULL, ntry = 5, varcom = NULL,
                        cov.lme = NULL, trim = NULL, cov.type = 'lambda') {
  cov2 <- list()
  kfix <- list()
  zw <- list()
  weight <- hosking$weight
  B <- hosking$B
  if (boot.lme == TRUE) boot.lme <- TRUE

  mle3 <- matrix(NA, nrow = numk, ncol = 3)
  lmB.med <- rep(NA, B)
  aic <- rep(NA, numk)

  nsample <- length(data)
  lm <- lmoms(data, nmom = numom)
  medata <- median(data)

  for (ip in 1:numk) {
    if (is.na(kpar[ip])) {
      aic[ip] <- NA
      mle3[ip, 1:3] <- NA
    } else {
      kfix[[ip]] <- gev.xifix.sing(xdat = data, xifix = kpar[ip], ntry = ntry,
                                   varcom = varcom)

      if (kfix[[ip]]$conv != 0) {
        aic[ip] <- NA
        mle3[ip, 1:3] <- NA
      } else {
        aic[ip] <- 2 * kfix[[ip]]$nllh + 2 * 2
        mle3[ip, 1:3] <- kfix[[ip]]$mle[1:3]
      }
      if (varcom == TRUE) cov2[[ip]] <- kfix[[ip]]$cov
    }
  }

  if (all(is.na(mle3[1:numk, 1]))) {
    lm3 <- lmoms(data, nmom = 3)
    for (ip in 1:numk) {
      mle3[ip, 1:3] <- pargev.xifix(lm3, xifix = kpar[ip])$para[1:3]
    }
  }

  if (weight == 'gLd') {
    if (cov.type == 'ratio') {
      if (boot.lme == TRUE) {
        cov.lm <- hosking$cov.rat
      } else {
        cov.lm <- cov.lme$cov.rat
      }
    } else if (cov.type == 'lambda') {
      if (boot.lme == TRUE) {
        cov.lm <- hosking$cov.lambda
      } else {
        cov.lm <- cov.lme$cov.lambda
      }
    }
    Vinv <- solve(cov.lm)
    detV <- det(cov.lm)
    zw$cov.lm <- list(Vinv = Vinv, detV = detV)
  }

  if (weight == 'med') {
    if (cov.type == 'ratio') {
      if (boot.lme == TRUE) {
        cov.med <- hosking$cov.med.rat
      } else {
        cov.med <- cov.lme$cov.med.rat
      }
    } else if (cov.type == 'lambda') {
      if (boot.lme == TRUE) {
        cov.med <- hosking$cov.med.lam
      } else {
        cov.med <- cov.lme$cov.med.lam
      }
    }

    Vinv.med <- solve(cov.med)
    detV.med <- det(cov.med)
    Vinv <- Vinv.med
    detV <- detV.med
    zw$cov.lm <- list(Vinv = Vinv.med, detV = detV.med)
  }

  if (weight == 'gLd' | weight == 'med') {
    normal <- list()
    normal$prob.mtx <- matrix(NA, nrow = numk, ncol = 2)

    normal <- com.prdist(data = data, numk = numk, kfix = kfix,
                         Vinv = Vinv, detV = detV, numom = numom,
                         hosking = hosking, trim = trim,
                         cov.type = cov.type)

    for (ip in 1:numk) {
      if (any(is.na(kfix[[ip]]$mle[1:3]))) {
        mle3[ip, 1:3] <- NA
        normal$prob.mtx[ip, 1:2] <- 0.0
      } else {
        mle3[ip, 1:3] <- kfix[[ip]]$mle[1:3]
      }
    }
  }

  zw$aic <- aic
  zw$mle3 <- mle3
  zw$kfix <- kfix
  if (varcom == TRUE) {
    zw$cov2 <- cov2
  } else {
    zw$cov2 <- NA
  }
  zw$prob.mtx <- matrix(NA, numk, 3)

  if (weight == 'gLd' | weight == 'med') {
    zw$prob.mtx <- normal$prob.mtx
    zw$gdd <- normal$gdd
  }

  return(zw)
}


#' Compute generalized L-moment distance probabilities
#'
#' @description Internal function that computes generalized L-moment distance
#' and median-based distance probabilities for each candidate submodel.
#'
#' @param data Numeric vector of data.
#' @param numk Number of candidate submodels.
#' @param kfix List of submodel fitting results.
#' @param Vinv Inverse of the L-moment covariance matrix.
#' @param detV Determinant of the L-moment covariance matrix.
#' @param numom Number of L-moments.
#' @param hosking List containing LME results.
#' @param trim Left trimming level.
#' @param cov.type Covariance type: "ratio" or "lambda" (default "lambda").
#'
#' @return A list containing:
#' \describe{
#'   \item{prob.mtx}{Matrix (numk x 2) of probabilities (col 1 = gLd, col 2 = med)}
#'   \item{gdd}{Matrix (numk x 2) of generalized distances}
#' }
#'
#' @keywords internal
com.prdist <- function(data = NULL, numk = NULL, kfix = NULL,
                       Vinv = NULL, detV = NULL, numom = NULL,
                       hosking = NULL, trim = NULL, cov.type = "lambda") {
  weight <- hosking$weight

  ww1 <- list()
  kfix.lm <- list()
  dd <- matrix(NA, nrow = numom, ncol = 2)
  gdd <- matrix(NA, nrow = numk, ncol = 2)
  simple <- rep(NA, numk)
  prob.mtx <- matrix(NA, nrow = numk, ncol = 2)

  re <- sort(data)
  nsam <- length(data)
  ntrim <- trim

  lm <- lmoms(data, nmom = numom)

  for (ip in 1:numk) {
    if (kfix[[ip]]$conv != 0) {
      prob.mtx[ip, 1:2] <- 0
    } else {
      if (kfix[[ip]]$mle[3] <= -1.0) kfix[[ip]]$mle[3] <- -0.999
      simple[ip] <- kfix[[ip]]$mle[3]
      savek <- vec2par(kfix[[ip]]$mle, 'gev')
      kfix.lm[[ip]] <- lmomgev(savek)
    }
  }

  if (weight == 'gLd') {
    for (ip in 1:numk) {
      if (kfix[[ip]]$conv == 0) {
        lmtrim.cvt <- lmoms(re[(ntrim + 1):nsam], nmom = 3)
        dd[1, 1] <- lmtrim.cvt$lambdas[1] - kfix.lm[[ip]]$lambdas[1]

        if (cov.type == 'ratio') {
          dd[2:numom, 1] <- lmtrim.cvt$ratios[2:numom] - kfix.lm[[ip]]$ratios[2:numom]
        } else if (cov.type == 'lambda') {
          dd[2, 1] <- lmtrim.cvt$lambdas[2] - kfix.lm[[ip]]$lambdas[2]
          dd[3, 1] <- lmtrim.cvt$lambdas[3] - kfix.lm[[ip]]$lambdas[3]
        }

        gdd[ip, 1] <- t(dd[1:numom, 1]) %*% Vinv %*% dd[1:numom, 1]
        prob.mtx[ip, 1] <- exp(-gdd[ip, 1] / 2) / ((2 * pi)^(numom / 2) * sqrt(detV))
        if (is.na(prob.mtx[ip, 1])) prob.mtx[ip, 1] <- 0
      }
    }
  }

  if (weight == 'med') {
    for (ip in 1:numk) {
      if (kfix[[ip]]$conv == 0) {
        medata <- median(re[(ntrim + 1):nsam])
        lmtrim.cvt <- lmoms(re[(ntrim + 1):nsam], nmom = 3)

        savek <- vec2par(kfix[[ip]]$mle, 'gev')
        dd[1, 2] <- medata - quagev(0.5, savek)

        if (cov.type == 'ratio') {
          dd[2:numom, 2] <- lmtrim.cvt$ratios[2:numom] - kfix.lm[[ip]]$ratios[2:numom]
        } else if (cov.type == 'lambda') {
          dd[2, 2] <- lmtrim.cvt$lambdas[2] - kfix.lm[[ip]]$lambdas[2]
          dd[3, 2] <- lmtrim.cvt$lambdas[3] - kfix.lm[[ip]]$lambdas[3]
        }

        gdd[ip, 2] <- t(dd[1:numom, 2]) %*% Vinv %*% dd[1:numom, 2]
        prob.mtx[ip, 2] <- exp(-gdd[ip, 2] / 2) / ((2 * pi)^(numom / 2) * sqrt(detV))
        if (is.na(prob.mtx[ip, 2])) prob.mtx[ip, 2] <- 0
      }
    }
  }

  ww1$prob.mtx <- prob.mtx
  ww1$gdd <- gdd
  return(ww1)
}


#' Adaptively expand or prune candidate xi set
#'
#' @description Internal function that adaptively adjusts the candidate xi
#' set by removing low-weight candidates and adding new candidates near
#' boundary or dominant regions to improve weight coverage.
#'
#' @param wtgd Numeric vector of current weights.
#' @param numk Number of candidate submodels.
#' @param kpar Numeric vector of candidate xi values.
#' @param remove Threshold below which candidates are removed. Default is 0.004.
#' @param dist Spacing for new candidates. Default is 0.02.
#' @param tre1 Lower threshold for boundary detection. Default is 0.01.
#' @param tre8 Upper threshold for dominant candidate. Default is 0.5.
#'
#' @return A list containing:
#' \describe{
#'   \item{kpar2}{Updated candidate xi values}
#'   \item{aw}{Adjustment indicator (0 = no change needed)}
#'   \item{numk}{Updated number of candidates}
#' }
#'
#' @keywords internal
new.kpar2 <- function(wtgd = NULL, numk = NULL, kpar = NULL,
                      remove = 0.004, dist = 0.02, tre1 = 0.01, tre8 = 0.5) {
  wtgd[which(is.na(wtgd))] <- 0
  zp <- list()
  aw <- 0

  nid <- which(wtgd < remove)
  id <- which(wtgd >= remove)
  kpar <- kpar[id]
  wtgd <- wtgd[id]
  numk2 <- length(kpar)
  if (numk2 < numk) aw <- 10
  numk <- numk2
  kpar2 <- kpar

  if (numk == 0) {
    kpar2 <- -0.1
    numk <- 1
  }

  if (numk == 1) {
    ad <- 1.5
    kpar2 <- c(NA, NA, kpar2, NA, NA)
    kpar2[2] <- max(kpar2[3] - dist / ad, -0.99)
    kpar2[1] <- max(kpar2[3] - 2 * dist / ad, -0.99)
    kpar2[4] <- min(kpar2[3] + dist / ad, 0.49)
    kpar2[5] <- min(kpar2[3] + 2 * dist / ad, 0.49)
    aw <- 11
  } else {
    work1 <- work2 <- work3 <- 0

    if (wtgd[1] >= tre1 & wtgd[1] <= tre8) {
      kpar2 <- c(rep(NA, 2), kpar)
      kpar2[2] <- max(kpar[1] - dist, -0.99)
      kpar2[1] <- max(kpar2[2] - dist, -0.99)
      work1 <- 1
    } else if (wtgd[1] > tre8) {
      kpar2 <- c(rep(NA, 4), kpar)
      kpar2[4] <- max(kpar[1] - dist / 2, -0.99)
      kpar2[3] <- max(kpar2[4] - dist / 2, -0.99)
      kpar2[2] <- max(kpar2[3] - dist / 2, -0.99)
      kpar2[1] <- max(kpar2[2] - dist / 2, -0.99)
      work1 <- 20
    }

    if (work1 == 0 & wtgd[numk] >= tre1 & wtgd[numk] <= tre8) {
      kpar2 <- c(kpar, rep(NA, 2))
      kpar2[numk + 1] <- min(kpar2[numk] + dist, 0.49)
      kpar2[numk + 2] <- min(kpar2[numk + 1] + dist, 0.49)
      work2 <- 1
    }
    if (work1 == 0 & wtgd[numk] > tre8) {
      kpar2 <- c(kpar, rep(NA, 4))
      kpar2[numk + 1] <- min(kpar2[numk] + dist / 2, 0.49)
      kpar2[numk + 2] <- min(kpar2[numk + 1] + dist / 2, 0.49)
      kpar2[numk + 3] <- min(kpar2[numk + 2] + dist / 2, 0.49)
      kpar2[numk + 4] <- min(kpar2[numk + 3] + dist / 2, 0.49)
      work2 <- 20
    }

    if (work1 == 1 & wtgd[numk] >= tre1) {
      kpar2 <- c(kpar2, rep(NA, 2))
      kpar2[numk + 3] <- min(kpar2[numk + 2] + dist, 0.49)
      kpar2[numk + 4] <- min(kpar2[numk + 3] + dist, 0.49)
      work2 <- 2
    }

    lek <- length(kpar2)
    mxid <- which.max(wtgd)
    if (wtgd[mxid] > tre8 & mxid != 1 & mxid != numk) {
      kpar2 <- c(kpar2, rep(NA, 4))
      kpar2[lek + 1] <- max(kpar[mxid] - dist / 3, -0.99)
      kpar2[lek + 2] <- max(kpar[mxid] - 2 * dist / 3, -0.99)
      kpar2[lek + 3] <- min(kpar[mxid] + dist / 3, 0.49)
      kpar2[lek + 4] <- min(kpar[mxid] + 2 * dist / 3, 0.49)
      work3 <- 4
    }
    aw <- aw + work1 + work2 + work3
  }

  numk <- length(kpar2)

  zp$kpar2 <- sort(kpar2)
  zp$aw <- aw
  zp$numk <- numk
  return(zp)
}


# ============================================================
# Candidate xi selection functions (from cand.xi.paper_12Dec25.R)
# ============================================================

#' Select candidate shape parameter values for model averaging
#'
#' @description Internal function that selects K candidate shape parameter
#' (xi) values from the profile likelihood confidence interval. Falls back
#' to bootstrap LME quantiles if the profile likelihood fails.
#'
#' @param data Numeric vector of data.
#' @param hosking List containing LME results and bootstrap information.
#' @param mle MLE parameter estimates (mu, sigma, xi) in Hosking style.
#' @param pick0 Confidence level for the profile CI. Default is 0.95.
#' @param nint Number of points for profile likelihood evaluation. Default is 256.
#' @param start Starting method: "mle" (default), "lme", or "mix".
#' @param numk Number of candidate submodels.
#' @param figure Logical. Whether to produce a profile likelihood plot (default TRUE).
#' @param cov.lme Pre-computed LME covariance (default NULL).
#' @param bma Logical. Whether BMA is being used (default FALSE).
#' @param pen BMA prior type (default "beta").
#'
#' @return A list containing:
#' \describe{
#'   \item{kpar}{Numeric vector of K candidate xi values}
#'   \item{start}{Starting method actually used}
#'   \item{get.ci}{Profile CI result (if start="mle")}
#'   \item{ymin}{Minimum y value for plotting (if start="mle")}
#' }
#'
#' @keywords internal
cand.xi <- function(data, hosking = NULL, mle = NULL, pick0 = 0.95,
                              nint = 256, start = 'mle', numk = NULL,
                              figure = TRUE, cov.lme = NULL, bma = FALSE,
                              pen = "beta") {
  wx <- list()
  SMALL <- 1e-05
  kpar <- rep(NA, numk)
  move <- FALSE

  while (start == 'mle' | start == 'mix') {
    newse <- max(0.3, abs(mle[3])) * 0.2

    mle.coles <- mle
    mle.coles[3] <- -mle[3]

    xlow <- mle.coles[3] - 25 * newse
    xup <- mle.coles[3] + 12 * newse
    xlow <- max(-0.99, min(0.5, xlow), na.rm = TRUE)
    xup <- min(0.99, max(-0.5, xup), na.rm = TRUE)
    xlow2 <- min(xlow, xup)
    xup2 <- max(xlow, xup)

    pilo <- (1 - pick0) / 2
    piup <- 1 - pilo
    eqpr <- seq(pilo, piup, by = pick0 / (numk - 1))
    numpi <- floor(numk / 2)
    pipick <- rep(NA, numpi)

    pipick[1] <- pick0
    for (i in 2:numpi) {
      pipick[i] <- 1 - 2 * eqpr[i]
    }

    get.ci <- gev.profxi.mdfy(data, mle = mle.coles,
                                    xlow = xlow2, xup = xup2,
                                    pick.v = pipick, nint = nint, figure = figure)

    if (get.ci$fail == TRUE) {
      start <- 'lme'
      move <- TRUE
      break
    }

    wx$get.ci <- get.ci

    lo.lim <- min(-get.ci$w1$ci1[1], -get.ci$w1$ci2[1])
    up.lim <- max(-get.ci$w1$ci1[1], -get.ci$w1$ci2[1])

    kpar[1] <- max(lo.lim, -0.99)
    kpar[numk] <- min(up.lim, 0.99)

    if (is.odd.me(numk)) kpar[floor(numk / 2) + 1] <- -get.ci$w1$xmax

    for (i in 2:numpi) {
      lo.lim <- min(-get.ci$w1$ci1[i], -get.ci$w1$ci2[i])
      up.lim <- max(-get.ci$w1$ci1[i], -get.ci$w1$ci2[i])
      kpar[i] <- max(lo.lim, -0.99)
      kpar[numk - i + 1] <- min(up.lim, 0.99)
    }

    if (kpar[numk] < -0.3) {
      kpar2 <- rep(NA, numk + 2)
      kpar2[numk + 1] <- max(-0.3, kpar[numk] + 0.05)
      kpar2[numk + 2] <- max(-0.3, kpar2[numk + 1] + 0.05)
      kpar[1:numk] <- c(kpar[3:numk], kpar2[numk + 1], kpar2[numk + 2])
    }

    wx$kpar.mle <- kpar
    break
  }

  if (start == 'lme' | start == 'mix') {
    pilo <- (1 - pick0) / 2
    piup <- 1 - pilo
    eqpr <- seq(pilo, piup, by = pick0 / (numk - 1))

    if (!is.null(cov.lme)) {
      hosking$lme.boot <- cov.lme$lme.boot
    } else {
      if (hosking$boot.lme == FALSE & move == TRUE) {
        Bnew <- hosking$B
        qua <- hosking$quant
        hosking <- lme.boots(data = data, B = Bnew, quant = qua, boot = TRUE)
      }
    }

    kpar <- quantile(hosking$lme.boot[, 3], prob = eqpr, na.rm = TRUE)

    if (kpar[numk] < -0.5) kpar[numk] <- max(-0.5, kpar[numk] + 0.05)
    wx$kpar.lme <- kpar
    if (move == TRUE) wx$kpar.mle <- kpar
    hosking$start <- start
  }

  if (start == 'mix') {
    wx$kpar.mix <- (wx$kpar.mle + wx$kpar.lme) / 2
  }

  if (start == 'mle') {
    kpar <- wx$kpar.mle
  } else if (start == 'lme') {
    kpar <- wx$kpar.lme
  } else if (start == 'mix') {
    kpar <- wx$kpar.mix
  }

  if (any(abs(kpar) <= SMALL)) {
    kpar[which(abs(kpar) <= SMALL)] <- SMALL * 1.2
  }

  if (kpar[numk] >= 0.5) kpar[numk] <- 0.49
  wx$kpar <- kpar
  wx$start <- start
  if (start == 'mle') wx$ymin <- get.ci$ymin

  return(wx)
}


#' Check if a number is odd
#'
#' @param x Integer value.
#'
#' @return Logical. TRUE if x is odd, FALSE otherwise.
#'
#' @keywords internal
is.odd.me <- function(x) {
  return(ifelse(x %% 2 == 1, TRUE, FALSE))
}


#' Modified profile likelihood for GEV shape parameter
#'
#' @description Internal function that computes the profile log-likelihood
#' for the GEV shape parameter with linear extrapolation beyond the
#' observed range. Used for constructing confidence intervals on xi.
#'
#' @param data Numeric vector of data.
#' @param mle MLE parameter estimates (mu, sigma, xi) in Coles parameterization.
#' @param xlow Lower bound for xi search.
#' @param xup Upper bound for xi search.
#' @param pick.v Numeric vector of confidence levels for CI computation.
#' @param nint Number of grid points. Default is 256.
#' @param figure Logical. Whether to plot the profile likelihood. Default is FALSE.
#'
#' @return A list containing:
#' \describe{
#'   \item{fail}{Logical. TRUE if profile likelihood is degenerate}
#'   \item{start}{Starting method recommendation}
#'   \item{w1}{Profile CI results from \code{comp.prof.ci()}}
#'   \item{ymin}{Minimum y value for plotting}
#'   \item{ymax}{Maximum y value for plotting}
#' }
#'
#' @keywords internal
gev.profxi.mdfy <- function(data = NULL, mle = NULL, xlow, xup,
                                  pick.v = NULL, nint = 256,
                                  figure = FALSE) {
  pick <- pick.v[1]
  w <- list()
  v <- numeric(nint)

  w$fail <- FALSE
  w$start <- 'mle'

  conf <- pick.v
  xlow.t <- min(xlow, xup)
  xup.t <- max(xlow, xup)
  xlow <- max(-0.9, xlow.t, na.rm = TRUE)
  xup <- min(0.9, xup.t, na.rm = TRUE)
  x <- seq(xlow, xup, length.out = nint)
  sol <- c(mle[1], mle[2])

  gev.plikxi <- function(a, data = data, xi = xi) {
    if (a[2] <= 0) {
      l <- 10^6
      return(l)
    }
    if (abs(xi) < 10^(-6)) {
      y <- (data - a[1]) / a[2]
      l <- length(y) * log(a[2]) + sum(exp(-y)) + sum(y)
    } else {
      y <- (data - a[1]) / a[2]
      y <- 1 + xi * y
      if (any(y <= 0)) {
        l <- 10^6
      } else {
        l <- length(y) * log(a[2]) + sum(y^(-1 / xi)) + sum(log(y)) * (1 / xi + 1)
      }
      l
    }
  }

  for (i in 1:nint) {
    xi <- x[i]
    opt <- optim(sol, gev.plikxi, data = data, xi = xi)
    sol <- opt$par
    v[i] <- opt$value
  }

  for (i in 1:nint) {
    if (v[i] >= 10^6) v[i] <- NA
  }

  d <- data.frame(x = x, v = -v)
  d <- na.omit(d)

  irt <- 0
  idmax <- which.max(d$v)
  if (idmax == 1) {
    irt <- 1
  } else if (idmax == length(d$x)) {
    irt <- 1
  }

  if (irt == 1) {
    w$start <- 'lme'
    w$fail <- TRUE
    return(w)
  }

  port <- 1.3
  x0 <- min(d$x)
  x9 <- max(d$x)
  bd0 <- max(0.2, abs(x0)) * sign(x0)
  bd9 <- max(0.2, abs(x9)) * sign(x9)

  xmin <- min(-0.7, x0 - sign(x0) * port * bd0)
  xmax <- max(0.7, x9 + sign(x9) * port * bd9)

  inter <- 1
  n.add <- 32
  yleft <- rep(NA, n.add)
  yright <- rep(NA, n.add)

  if (xmax > x9) {
    idmost <- which.max(d$x)
    b.over <- (d$v[idmost] - d$v[idmost - 1]) / (d$x[idmost] - d$x[idmost - 1])
    extra.over <- seq(d$x[idmost], xmax, by = (xmax - d$x[idmost]) / n.add)
    b.over <- b.over * 1.2
    for (i in 1:length(extra.over)) {
      yright[i] <- d$v[idmost] - b.over * (d$x[idmost] - extra.over[i])
    }
  }

  if (x0 > xmin) {
    idlst <- which.min(d$x)
    b.bef <- (d$v[idlst + 1] - d$v[idlst]) / (d$x[idlst + 1] - d$x[idlst])
    extra.bef <- seq(xmin, d$x[idlst], by = (d$x[idlst] - xmin) / n.add)
    b.bef <- b.bef * 1.2
    for (i in 1:(length(extra.bef))) {
      yleft[i] <- d$v[idlst] - b.bef * (d$x[idlst] - extra.bef[i])
    }
  }

  msp1 <- data.frame(x = c(extra.bef, d$x, extra.over), y = c(yleft, d$v, yright))
  dv1 <- msp1$y

  w1 <- comp.prof.ci(d = msp1, v = dv1, conf = conf)

  w$w1 <- w1
  inter <- 1
  d <- msp1
  d$v <- msp1$y

  if (figure == TRUE & pick >= .90) {
    halfchi <- 0.5 * qchisq(conf[1], 1)
    w$ymin <- w$w1$vmax - (halfchi * 2.7) + 0.2
    w$ymax <- w$w1$vmax + 0.8 * halfchi - 0.2

    xlim0 <- -(w$w1$ci2 + (w$w1$ci_length) / 4)
    xlim1 <- -(w$w1$ci1 - (w$w1$ci_length) / 4)

    if (inter == 1) {
      co <- "green"
    } else if (inter == 2) {
      co <- "blue"
    }

    plot(-d$x, d$v,
         xlim = c(xlim0[1] - 0.02, xlim1[1] + 0.02),
         ylim = c(w$w1$vmax - (halfchi * 2.7), w$w1$vmax + 0.8 * halfchi),
         type = "l", xlab = "xi_Hosking in GEV",
         col = co, lwd = 2,
         ylab = "Profile Log-likelihood", main = c("Profile CI for xi"))
    ma <- w$w1$vmax
    abline(h = ma, col = 4)
    abline(h = ma - 0.5 * qchisq(conf[1], 1), col = 4)
    abline(v = c(-w$w1$xmax), col = 2, lty = 2, lwd = 2)
    text(x = c(-mle[3]), y = w$ymax, labels = "mle")
    abline(v = c(-w$w1$ci1[1], -w$w1$ci2[1]), col = 2, lty = 2)
    text(x = -w$w1$ci1[1], y = w$ymax,
         labels = paste("", round(-w$w1$ci1[1], 2), sep = ""), col = 2)
    text(x = -w$w1$ci2[1], y = w$ymax,
         labels = paste("", round(-w$w1$ci2[1], 2), sep = ""), col = 2)

    lm <- lmoms(data)
    lme <- pargev(lm)$para[3]
    abline(v = lme, lty = 5, lwd = 2, col = "purple")
    text(x = lme, y = w$ymin, labels = "lme")
  }

  return(w)
}


#' Compute profile likelihood confidence intervals
#'
#' @description Internal function that extracts confidence intervals from
#' a profile log-likelihood curve at specified confidence levels.
#'
#' @param d Data frame with columns \code{x} (xi values) and \code{v} (log-likelihood).
#' @param v Numeric vector of log-likelihood values.
#' @param conf Numeric vector of confidence levels (e.g., 0.95).
#'
#' @return A list containing:
#' \describe{
#'   \item{vmax}{Maximum log-likelihood value}
#'   \item{nllh}{Same as vmax}
#'   \item{xmax}{xi value at maximum likelihood}
#'   \item{ci1}{Lower CI bounds for each confidence level}
#'   \item{ci2}{Upper CI bounds for each confidence level}
#'   \item{ci_length}{CI lengths for each confidence level}
#' }
#'
#' @keywords internal
comp.prof.ci <- function(d, v, conf = NULL) {
  w <- list()
  numcf <- length(conf)
  d$v <- v
  w$vmax <- max(d$v)
  nmsp <- length(d$v)

  w$nllh <- w$vmax
  idmax <- which.max(d$v)
  w$xmax <- d$x[idmax]

  ci1 <- rep(NA, numcf)
  ci2 <- rep(NA, numcf)
  ci_length <- rep(NA, numcf)

  for (i in 1:numcf) {
    chisq <- w$vmax - 0.5 * qchisq(conf[i], 1)
    diff <- d$v - chisq

    ci1[i] <- d$x[which.min(abs(diff[1:idmax]))]
    ci2[i] <- d$x[(idmax + 1):nmsp][which.min(abs(diff[(idmax + 1):nmsp]))]

    ci_min <- min(ci1[i], ci2[i])
    ci_max <- max(ci1[i], ci2[i])
    ci1[i] <- ci_min
    ci2[i] <- ci_max

    ci_length[i] <- ci2[i] - ci1[i]
  }

  w$ci1 <- ci1
  w$ci2 <- ci2
  w$ci_length <- ci_length

  return(w)
}

#' Phliu Agrometeorological Station Data
#'
#' Annual maximum daily precipitation and station metadata from the Phliu
#' Agrometeorological Station, Chanthaburi province, Thailand (1984-2023).
#' A Mann-Kendall test indicates a significant increasing trend in the
#' annual maxima (tau = 0.235, p = 0.033), so this dataset is used for
#' demonstrating non-stationary (GEV11) estimation; it is the heavy
#' rainfall application of Shin et al. (2025).
#'
#' @format A data frame with 40 rows and 9 columns:
#' \describe{
#'   \item{Station.ID}{Station identifier (character)}
#'   \item{year}{Year of observation (numeric, 1984-2023)}
#'   \item{prec}{Annual maximum daily precipitation in mm (numeric)}
#'   \item{Name}{Station name (character)}
#'   \item{zone}{Climate zone code (character)}
#'   \item{latitude}{Station latitude in degrees (numeric)}
#'   \item{longitude}{Station longitude in degrees (numeric)}
#'   \item{Starting.year}{Record start year (integer)}
#'   \item{Ending.year}{Record end year (numeric)}
#' }
#'
#' @source Phliu Agrometeorological Station, Thailand.
#' 
#' @references 
#' Shin, Y., Shin, Y., Park, J., & Park, J. S. (2025). Generalized method of L-moment estimation
#' for stationary and nonstationary extreme value models. arXiv preprint arXiv:2512.20385.
#' \doi{10.48550/arXiv.2512.20385}
#'
#' @examples
#' data(PhliuAgromet)
#' head(PhliuAgromet)
#'
#' \donttest{
#' # Fit non-stationary GEV11 model by GLME
#' result <- glme.gev11(PhliuAgromet$prec, ntry = 5)
#' print(result$para.glme)
#' }
#'
"PhliuAgromet"

#' Bangkok Maximum Rainfall Data
#'
#' Annual maximum daily rainfall data from Bangkok, Thailand.
#' This dataset is used for demonstrating model averaging methods
#' for high quantile estimation in extreme value analysis.
#'
#' @format A data frame with 58 rows and 5 columns:
#' \describe{
#'   \item{X1}{Annual maximum daily rainfall in mm (numeric)}
#'   \item{X2}{2nd largest annual daily rainfall in mm (numeric)}
#'   \item{X3}{3rd largest annual daily rainfall in mm (numeric)}
#'   \item{X4}{4th largest annual daily rainfall in mm (numeric)}
#'   \item{X5}{5th largest annual daily rainfall in mm (numeric)}
#' }
#'
#' @source Thai Meteorological Department (TMD; \url{https://www.tmd.go.th})
#'
#' @references
#' Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
#' for estimating high quantiles of extreme values: Application to heavy rainfall.
#' \emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
#' \doi{10.1007/s00477-025-03167-x}
#'
#' @examples
#' data(bangkok)
#' head(bangkok)
#'
#' # Estimate high quantiles using model averaging
#' result <- ma.gev(bangkok$X1, quant = c(0.99, 0.995))
#' print(result$qua.ma)
#'
"bangkok"

#' Haenam Maximum Rainfall Data
#'
#' Annual maximum daily rainfall data from Haenam, South Korea.
#' This dataset is used for demonstrating model averaging methods
#' for high quantile estimation in extreme value analysis.
#'
#' @format A data frame with 52 rows and 2 columns:
#' \describe{
#'   \item{year}{Year of observation (integer, 1971-2022)}
#'   \item{X1}{Annual maximum daily rainfall in mm (numeric)}
#' }
#'
#' @source Korea Meteorological Administration (KMA; \url{https://www.weather.go.kr})
#'
#' @references
#' Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
#' for estimating high quantiles of extreme values: Application to heavy rainfall.
#' \emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
#' \doi{10.1007/s00477-025-03167-x}
#'
#' @examples
#' data(haenam)
#' head(haenam)
#'
#' # Estimate high quantiles using model averaging
#' result <- ma.gev(haenam$X1, quant = c(0.98, 0.99, 0.995))
#' print(result$qua.ma)
#'
"haenam"

# Shin et al. (2025b) compatibility functions
# Non-stationary GEV estimation using L-moments

#' Non-stationary GEV Parameter Estimation
#'
#' @description
#' Estimates parameters of a non-stationary Generalized Extreme Value (GEV)
#' distribution using the L-moment-based algorithm from Shin et al. (2025,
#' J. Korean Stat. Soc.).
#'
#' This is a convenience wrapper around \code{\link{lme.gev11}}, providing
#' compatibility with the original nsgev package interface (and with
#' GLmom v1.x).
#'
#' @param xdat A numeric vector of data to be fitted (e.g., annual maximum values).
#' @param ntry Number of attempts for optimization (default 20).
#' @param ftol Function tolerance for optimization (default 1e-6).
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{para.prop} - The proposed L-moment based estimates
#'         (mu0, mu1, sigma0, sigma1, xi).
#'   \item \code{precis} - Precision of the optimization.
#' }
#'
#' @references
#' Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
#' model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
#' \doi{10.1007/s42952-025-00325-3}
#'
#' @seealso \code{\link{lme.gev11}} for the recommended interface,
#'   \code{\link{glme.gev11}} for the full GLME method with penalty functions,
#'   \code{\link{gado.prop_11}} for detailed estimation results.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' data(PhliuAgromet)
#' result <- nsgev(PhliuAgromet$prec, ntry = 5)
#' print(result$para.prop)
#'
#' @export
nsgev <- function(xdat, ntry = 20, ftol = 1e-6) {
  z <- list()

  result <- lme.gev11(xdat, ntry = ntry, ftol = ftol)

  # Return in nsgev format
  z$para.prop <- result$lme.gev11
  z$precis <- result$precis

  return(z)
}

#' Comprehensive Non-stationary GEV Estimation (deprecated)
#'
#' @description
#' Estimates parameters of a non-stationary GEV distribution using multiple
#' methods: Weighted Least Squares (WLS), GN16 method, and the proposed
#' L-moment method from Shin et al. (2025, J. Korean Stat. Soc.).
#'
#' As of GLmom v2.0.0 this function is deprecated: it was renamed to
#' \code{\link{lme.gev11}} (for the proposed method), and the auxiliary
#' methods are available as \code{\link{strup.gev11}} and
#' \code{\link{GN16.gev11}}. This wrapper reassembles the v1.x output
#' format from those functions and will be removed in a future release.
#'
#' @param xdat A numeric vector of data to be fitted.
#' @param ntry Number of attempts for optimization (default 20).
#' @param ftol Function tolerance for optimization (default 1e-6).
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{para.prop} - L-moment based estimates (proposed method).
#'   \item \code{para.gado} - GN16 method estimates.
#'   \item \code{para.wls} - Weighted least squares estimates.
#'   \item \code{strup.org} - Original non-stationary WLSE by Strup method.
#'   \item \code{lme.sta} - Stationary L-moment estimates.
#' }
#'
#' @references
#' Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
#' model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
#' \doi{10.1007/s42952-025-00325-3}
#'
#' @seealso \code{\link{lme.gev11}} for the recommended interface,
#'   \code{\link{strup.gev11}}, \code{\link{GN16.gev11}},
#'   \code{\link{glme.gev11}} for the full GLME method,
#'   \code{\link{nsgev}} for the simple interface.
#'
#' @author Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
#'
#' @examples
#' \donttest{
#' data(PhliuAgromet)
#' # deprecated wrapper; suppressWarnings() silences the deprecation notice
#' result <- suppressWarnings(gado.prop_11(PhliuAgromet$prec, ntry = 5))
#' print(result$para.prop)
#' print(result$lme.sta)
#' }
#'
#' @export
gado.prop_11 <- function(xdat, ntry = 20, ftol = 1e-6) {
  .Deprecated("lme.gev11")
  z <- list()

  result <- lme.gev11(xdat, ntry = ntry, ftol = ftol)
  z$para.prop <- result$lme.gev11

  strup <- strup.gev11(xdat, init.rob = TRUE)
  z$para.wls <- strup$strup.mdfy
  z$strup.org <- strup$strup.para

  gado <- GN16.gev11(xdat)
  z$para.gado <- gado$para.gado.org

  z$lme.sta <- pargev(lmoms(xdat, nmom = 3))$para
  names(z$lme.sta) <- c("mu_sta", "sigma_sta", "xi_sta")

  return(z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{surrogate}
\alias{surrogate}
\title{Find surrogate GEV parameters for model-averaged quantiles}
\usage{
surrogate(zpf.surr = NULL, xqa, init.surr)
}
\arguments{
\item{zpf.surr}{Numeric vector of model-averaged quantiles at \code{xqa}.}

\item{xqa}{Numeric vector of probabilities at which quantiles are evaluated.}

\item{init.surr}{Initial GEV parameter estimates for optimization.}
}
\value{
A list (from \code{optim()}) with additional components:
\describe{
  \item{par}{Surrogate GEV parameters (mu, sigma, xi)}
  \item{zp.surrmodel}{Quantiles from the surrogate model at \code{xqa}}
  \item{zp.MA}{The input model-averaged quantiles}
}
}
\description{
Internal function that fits a single GEV distribution to
the model-averaged quantile curve, producing surrogate GEV parameters
that approximate the model-averaged quantile function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ran.gev.R
\name{qns.gev_all}
\alias{qns.gev_all}
\title{Quantile function for non-stationary GEV (return period input)}
\usage{
qns.gev_all(Tp = NULL, para = NULL, year = NULL, model = NULL)
}
\arguments{
\item{Tp}{Return period (scalar).}

\item{para}{Parameter vector for the non-stationary model.}

\item{year}{Numeric vector of time points.}

\item{model}{Model type: "gev11", "gev10", "gev20", or "gev01".}
}
\value{
Numeric vector of quantile values (one per time point).
}
\description{
Internal function that computes time-varying quantiles for
non-stationary GEV models given a return period.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalty.R
\name{check.penalty}
\alias{check.penalty}
\title{Validate and resolve penalty settings}
\usage{
check.penalty(
  pen = NULL,
  pen.choice = NULL,
  p = NULL,
  c1 = NULL,
  c2 = NULL,
  mu = NULL,
  std = NULL
)
}
\arguments{
\item{pen}{Penalty name (character).}

\item{pen.choice}{Preset number or NULL.}

\item{p, c1, c2}{Beta penalty hyperparameters (used when pen.choice is NULL).}

\item{mu, std}{Normal penalty hyperparameters (used when pen.choice is NULL).}
}
\value{
A list with elements pen, p, c1, c2, mu, std.
}
\description{
Normalizes the penalty name (e.g., "Beta" to "beta") and, when
\code{pen.choice} is given, resolves the preset hyperparameters:
for \code{pen="beta"}, choices 1-6 give (p, c1, c2) =
(6,3,1), (6,5,2), (6,7,3), (2,3,0.5), (2,5,1), (2,7,1.5);
for \code{pen="norm"}, choices 1-4 give (mu, std) =
(-0.5,0.25), (-0.5,0.15), (-0.6,0.25), (-0.6,0.15).
These presets follow Table 1 and Eq. (15) of Shin et al. (2025).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{com.prdist}
\alias{com.prdist}
\title{Compute generalized L-moment distance probabilities}
\usage{
com.prdist(
  data = NULL,
  numk = NULL,
  kfix = NULL,
  Vinv = NULL,
  detV = NULL,
  numom = NULL,
  hosking = NULL,
  trim = NULL,
  cov.type = "lambda"
)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{numk}{Number of candidate submodels.}

\item{kfix}{List of submodel fitting results.}

\item{Vinv}{Inverse of the L-moment covariance matrix.}

\item{detV}{Determinant of the L-moment covariance matrix.}

\item{numom}{Number of L-moments.}

\item{hosking}{List containing LME results.}

\item{trim}{Left trimming level.}

\item{cov.type}{Covariance type: "ratio" or "lambda" (default "lambda").}
}
\value{
A list containing:
\describe{
  \item{prob.mtx}{Matrix (numk x 2) of probabilities (col 1 = gLd, col 2 = med)}
  \item{gdd}{Matrix (numk x 2) of generalized distances}
}
}
\description{
Internal function that computes generalized L-moment distance
and median-based distance probabilities for each candidate submodel.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GN16.gev11.R
\name{time.m.gev11}
\alias{time.m.gev11}
\title{Time-varying moment step of the GN16 method}
\usage{
\method{time}{m.gev11}(qmax = NULL, orig.para = NULL, rob = FALSE)
}
\arguments{
\item{qmax}{Pseudo-maxima series from \code{\link{make.qmax.gev11}}.}

\item{orig.para}{Parameter vector (mu0, mu1, sigma0, sigma1, xi).}

\item{rob}{If TRUE, uses robust regression for the location trend.}
}
\value{
A list with para.gado.org and para.gado.mdfy.
}
\description{
Internal step of \code{\link{GN16.gev11}}: fits a
stationary GEV to the pseudo-maxima and converts the result to the
GEV11 parameterization (both original and modified specification).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{is.odd.me}
\alias{is.odd.me}
\title{Check if a number is odd}
\usage{
is.odd.me(x)
}
\arguments{
\item{x}{Integer value.}
}
\value{
Logical. TRUE if x is odd, FALSE otherwise.
}
\description{
Check if a number is odd
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{cand.xi}
\alias{cand.xi}
\title{Select candidate shape parameter values for model averaging}
\usage{
cand.xi(
  data,
  hosking = NULL,
  mle = NULL,
  pick0 = 0.95,
  nint = 256,
  start = "mle",
  numk = NULL,
  figure = TRUE,
  cov.lme = NULL,
  bma = FALSE,
  pen = "beta"
)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{hosking}{List containing LME results and bootstrap information.}

\item{mle}{MLE parameter estimates (mu, sigma, xi) in Hosking style.}

\item{pick0}{Confidence level for the profile CI. Default is 0.95.}

\item{nint}{Number of points for profile likelihood evaluation. Default is 256.}

\item{start}{Starting method: "mle" (default), "lme", or "mix".}

\item{numk}{Number of candidate submodels.}

\item{figure}{Logical. Whether to produce a profile likelihood plot (default TRUE).}

\item{cov.lme}{Pre-computed LME covariance (default NULL).}

\item{bma}{Logical. Whether BMA is being used (default FALSE).}

\item{pen}{BMA prior type (default "beta").}
}
\value{
A list containing:
\describe{
  \item{kpar}{Numeric vector of K candidate xi values}
  \item{start}{Starting method actually used}
  \item{get.ci}{Profile CI result (if start="mle")}
  \item{ymin}{Minimum y value for plotting (if start="mle")}
}
}
\description{
Internal function that selects K candidate shape parameter
(xi) values from the profile likelihood confidence interval. Falls back
to bootstrap LME quantiles if the profile likelihood fails.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalty.R
\name{Befun}
\alias{Befun}
\title{Beta function integrand for penalty calculation}
\usage{
Befun(w, al = al, bl = bl, p = p, q = q)
}
\arguments{
\item{w}{Integration variable.}

\item{al}{Lower bound.}

\item{bl}{Upper bound.}

\item{p}{Shape parameter p.}

\item{q}{Shape parameter q.}
}
\value{
Integrand value.
}
\description{
Beta function integrand for penalty calculation
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{sel.para_all}
\alias{sel.para_all}
\title{Select parameters using goodness-of-fit criterion}
\usage{
sel.para_all(xdat, para.sel = NULL, model = NULL, obj.fun = NULL)
}
\arguments{
\item{xdat}{Numeric vector of data.}

\item{para.sel}{Matrix of candidate parameter vectors (one per row).}

\item{model}{Model name string (e.g., "gev11").}

\item{obj.fun}{Numeric vector of objective values used for tie-breaking.}
}
\value{
A list containing:
\describe{
  \item{para}{Selected parameter vector}
  \item{min.itry}{Index of selected try}
  \item{gof}{Goodness-of-fit values}
  \item{obj.fun}{Objective function values}
}
}
\description{
Internal function that selects the best parameter set among
candidate solutions using a goodness-of-fit measure based on the expected
vs. observed number of exceedances (see Shin et al., 2025b, JKSS).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{rcd}
\alias{rcd}
\title{Revised Coles-Dixon prior function}
\usage{
rcd(xi)
}
\arguments{
\item{xi}{Numeric vector of shape parameter values.}
}
\value{
Numeric vector of prior probability values.
}
\description{
Internal function that computes a revised Coles-Dixon prior
probability for the shape parameter, used as a penalty in BMA.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{pargev.xifix}
\alias{pargev.xifix}
\title{GEV parameter estimation with fixed shape parameter (MAGEV internal)}
\usage{
pargev.xifix(lmom, xifix = 0.1, checklmom = TRUE, ...)
}
\arguments{
\item{lmom}{L-moments object from \code{lmomco::lmoms()}.}

\item{xifix}{Fixed shape parameter value. Default is 0.1.}

\item{checklmom}{Logical. Whether to check L-moment validity. Default is TRUE.}

\item{...}{Additional arguments (unused).}
}
\value{
A list with components:
\describe{
  \item{type}{Character "gev"}
  \item{para}{Numeric vector of GEV parameters (mu, sigma, xi)}
  \item{source}{Character "pargev"}
}
}
\description{
Internal function that estimates GEV location and scale
parameters given a fixed shape parameter, using L-moment equations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{movave}
\alias{movave}
\title{Moving average smoother for quantiles and weights}
\usage{
movave(order, numk, numq, zp, wt = NULL)
}
\arguments{
\item{order}{Order of the moving average.}

\item{numk}{Number of candidate submodels.}

\item{numq}{Number of quantile probabilities.}

\item{zp}{Matrix (numq x numk) of quantile estimates.}

\item{wt}{Numeric vector of weights (length numk).}
}
\value{
A list containing:
\describe{
  \item{ez}{Matrix (numq x numk) of smoothed quantile estimates}
  \item{ew}{Numeric vector of smoothed weights}
}
}
\description{
Internal function that applies moving average smoothing
to quantile estimates and weights across candidate submodels.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalty.R
\name{penalty.fun}
\alias{penalty.fun}
\title{Penalty dispatcher (negative log prior)}
\usage{
penalty.fun(
  par,
  mu = NULL,
  std = NULL,
  lme = NULL,
  pen = "beta",
  p = NULL,
  c0 = 0.35,
  c1 = NULL,
  c2 = NULL,
  q = NULL
)
}
\arguments{
\item{par}{Parameter vector; only \code{par[3]} (shape) is penalized.}

\item{mu, std}{Hyperparameters for the normal penalty.}

\item{lme}{Center (L-moment pre-estimate) for the adaptive beta penalty.}

\item{pen}{Penalty type.}

\item{p, c0, c1, c2, q}{Hyperparameters for the beta penalty.}
}
\value{
Negative log prior value (scalar).
}
\description{
Returns the negative log of the preference (prior) function for the
selected penalty type. Dispatches to \code{\link{pk.norm.stnary}}
("norm"), the adaptive \code{\link{pk.beta}} ("beta"), the fixed
literature beta penalties ("ms": Beta(6,9); "park": Beta(2.5,2.5);
"cannon": Beta(2,3.3)), the Coles-Dixon exponential penalty ("cd"),
or zero ("no").
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{find_max_beta.pk}
\alias{find_max_beta.pk}
\title{Find the shape parameter maximizing the beta penalty}
\usage{
find_max_beta.pk(lme.center, p = p, c0 = c0, c1 = c1, c2 = c2)
}
\arguments{
\item{lme.center}{Center vector (mu0, sigma0, xi) of the penalty.}

\item{p, c0, c1, c2}{Beta penalty hyperparameters.}
}
\value{
The xi value maximizing the penalty.
}
\description{
Finds the value of xi (over a grid within the adaptive
beta penalty support) which maximizes the beta penalty function.
Used as one of the initial points for optimization.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{gev.xilik}
\alias{gev.xilik}
\title{GEV negative log-likelihood with fixed xi}
\usage{
gev.xilik(a, xifix = NULL, xdat = NULL)
}
\arguments{
\item{a}{Numeric vector of (mu, sigma).}

\item{xifix}{Fixed shape parameter value.}

\item{xdat}{Numeric vector of data.}
}
\value{
Negative log-likelihood value (scalar). Returns 10^6 if invalid.
}
\description{
Internal function that computes the negative log-likelihood
for the GEV distribution with a fixed shape parameter (Hosking parameterization).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{gev.xifix.sing}
\alias{gev.xifix.sing}
\title{MLE for GEV with fixed shape parameter (single candidate)}
\usage{
gev.xifix.sing(xdat = NULL, xifix = -0.1, ntry = 5, varcom = NULL)
}
\arguments{
\item{xdat}{Numeric vector of data.}

\item{xifix}{Fixed shape parameter value. Default is -0.1.}

\item{ntry}{Number of optimization attempts. Default is 5.}

\item{varcom}{Logical. If TRUE, computes Prescott-Walden covariance matrix.}
}
\value{
A list of class "gev.xifix" containing:
\describe{
  \item{conv}{Convergence status (0 = success)}
  \item{nllh}{Negative log-likelihood at the optimum}
  \item{mle}{MLE estimates (mu, sigma, xi)}
  \item{cov}{2x2 covariance matrix (if \code{varcom=TRUE})}
}
}
\description{
Internal function that computes MLE of GEV location and scale
parameters with a fixed shape parameter using L-BFGS-B optimization.
Optionally computes Prescott-Walden Hessian for variance estimation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev.R
\name{glme.gev}
\alias{glme.gev}
\title{Generalized L-moments estimation for generalized extreme value distribution}
\usage{
glme.gev(
  xdat,
  ntry = 5,
  pen = "beta",
  pen.choice = 1,
  mu = -0.5,
  std = 0.2,
  p = 6,
  c1 = 3,
  c2 = 1,
  c0 = 0.35,
  q = NULL,
  show = FALSE,
  method = "BFGS",
  maxit = 70,
  abstol = 1e-05
)
}
\arguments{
\item{xdat}{A numeric vector of data to be fitted.}

\item{ntry}{Number of attempts for parameter estimation (default 5). Higher
values increase the chance of finding the global optimum by trying
different initial conditions.}

\item{pen}{Type of penalty function: Choose among "norm", "beta" (default),
"ms", "park", "cannon", "cd", and "no" (without penalty function).}

\item{pen.choice}{Choice number of penalty function specifying hyperparameters
(default 1). For "beta": 1-6 correspond to (p, c1, c2) =
(6,3,1), (6,5,2), (6,7,3), (2,3,0.5), (2,5,1), (2,7,1.5).
For "norm": 1-4 correspond to (mu, std) =
(-0.5,0.25), (-0.5,0.15), (-0.6,0.25), (-0.6,0.15).
Set \code{pen.choice=NULL} to use the hyperparameters given by
\code{p, c1, c2} (beta) or \code{mu, std} (norm) directly.}

\item{mu}{Mean hyperparameter for "norm" penalty function (default -0.5).}

\item{std}{Standard deviation hyperparameter for "norm" penalty function (default 0.2).}

\item{p}{Shape hyperparameter for "beta" penalty function (default 6).}

\item{c1}{Scaling hyperparameter for "beta" penalty function (default 3).}

\item{c2}{Upper limit hyperparameter for "beta" penalty function (default 1).}

\item{c0}{Half-width of the adaptive beta penalty support (default 0.35).}

\item{q}{Optional fixed second shape parameter for the beta penalty. If
given, the beta penalty support is fixed instead of data-adaptive.}

\item{show}{If TRUE, prints the objective value and parameters of each try.}

\item{method}{Optimization method passed to \code{\link[stats]{optim}}
(default "BFGS").}

\item{maxit}{Maximum number of iterations for \code{optim} (default 70).}

\item{abstol}{Absolute and relative convergence tolerance for \code{optim}
(default 1e-5).}
}
\value{
The glme.gev function returns a list containing the following elements:
\itemize{
 \item para.glme - The estimated parameters of the Generalized Extreme Value distribution.
 \item para.lme - The L-moment estimates of the parameters.
 \item nllh.glme - The penalized negative log-likelihood of the GLME solution.
 \item convergence - 0 if the optimization converged, 5 otherwise.
 \item pen, pen_pen.choice - The penalization method (and choice) used.
 \item p_q - (for beta penalty) The p and q values used.
 \item c1_c2, c0_c1_c2 - (for beta penalty) The hyperparameters used.
 \item mu_std - (for norm penalty) The mu and std values used.
 \item covinv.lmom - The inverse of the covariance matrix of the L-moments.
 \item lcovdet - The log determinant of the covariance matrix.
 \item data - The input data (used by the plot method).
}
The object has class \code{"glme"}; see \code{\link{GLmom-methods}}.
}
\description{
This function estimates the parameters of the Generalized Extreme Value
distribution by the generalized method of L-moment estimation (GLME),
which minimizes the generalized L-moment distance plus a penalty
(preference) function on the shape parameter.
}
\details{
The equations for the L-moments for LME of the GEVD are
\deqn{ \underline{\bf \lambda} - \underline{\bf l} = \underline{\bf 0},}
where \eqn{ \underline{\bf \lambda} =(\lambda_1,\; \lambda_2,\; \lambda_3)^t } and \eqn{\underline{\bf l} =(l_1,\; l_2,\; l_3)^t}.
Next, we define the generalized L-moments distance (GLD) as;
\deqn{(\underline{\bf \lambda} -\underline{\bf l})^t V^{-1} (\underline{\bf \lambda} -\underline{\bf l}),}
where \eqn{V} is the variance-covariance matrix of the sample L-moments up to the third order.
}
\examples{
# Load example heavy-rainfall data
data(haenam)
x <- haenam$X1

# Estimate GEV parameters using beta penalty (default)
result <- glme.gev(x, ntry = 5)
print(result$para.glme)

# Using Martins-Stedinger penalty
result_ms <- glme.gev(x, ntry = 5, pen = "ms")
print(result_ms$para.glme)

}
\references{
Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
L-moment estimation for stationary and nonstationary extreme value models.
arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
}
\seealso{
\code{\link{glme.gev11}} for non-stationary GEV estimation,
  \code{\link{ma.gev}} for model averaging estimation,
  \code{\link{glme.like}} for the objective function,
  \code{\link{quagev.NS}} for quantile computation.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{haenam}
\alias{haenam}
\title{Haenam Maximum Rainfall Data}
\format{
A data frame with 52 rows and 2 columns:
\describe{
  \item{year}{Year of observation (integer, 1971-2022)}
  \item{X1}{Annual maximum daily rainfall in mm (numeric)}
}
}
\source{
Korea Meteorological Administration (KMA; \url{https://www.weather.go.kr})
}
\usage{
haenam
}
\description{
Annual maximum daily rainfall data from Haenam, South Korea.
This dataset is used for demonstrating model averaging methods
for high quantile estimation in extreme value analysis.
}
\examples{
data(haenam)
head(haenam)

# Estimate high quantiles using model averaging
result <- ma.gev(haenam$X1, quant = c(0.98, 0.99, 0.995))
print(result$qua.ma)

}
\references{
Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
for estimating high quantiles of extreme values: Application to heavy rainfall.
\emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
\doi{10.1007/s00477-025-03167-x}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_plots.R
\name{magev.rlplot}
\alias{magev.rlplot}
\title{Return Level Plot for MAGEV}
\usage{
magev.rlplot(par = NULL, se.vec = NULL, data = NULL)
}
\arguments{
\item{par}{A numeric vector of GEV parameters (mu, sigma, xi) in Hosking style.
Typically from \code{zx$surr$par} where \code{zx} is the output of
\code{\link{ma.gev}}.}

\item{se.vec}{A numeric vector of standard errors for the quantile estimates
corresponding to the plotting positions. Typically from \code{zx$ranw.se.ma}.}

\item{data}{A numeric vector of observed data (annual maxima).}
}
\value{
NULL. The function produces a plot as a side effect.
}
\description{
Displays fitted return levels with 95% confidence intervals against
return period on a log scale.
}
\details{
The plot shows:
\itemize{
  \item Black line: Fitted return level curve
  \item Blue lines: 95% confidence interval (mean +/- 1.96 * SE)
  \item Black points: Observed data at empirical return periods
}

The x-axis (return period) is on a log scale, ranging from 0.1 to 900 years.
}
\examples{
data(haenam)
ff <- c(seq(0.01, 0.09, by = 0.01), 0.1, 0.2, 0.3, 0.4, 0.5,
        0.6, 0.7, 0.8, 0.9, 0.93, 0.95, 0.98, 0.99,
        0.993, 0.995, 0.998, 0.999)
zx <- ma.gev(haenam$X1, quant = ff, weight = 'like1',
             numk = 9, varcom = TRUE)
magev.rlplot(par = zx$surr$par, se.vec = zx$ranw.se.ma, data = haenam$X1)

}
\references{
Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
for estimating high quantiles of extreme values: Application to heavy rainfall.
\emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
\doi{10.1007/s00477-025-03167-x}
}
\seealso{
\code{\link{ma.gev}} for the main model averaging function,
  \code{\link{magev.qqplot}} for Q-Q diagnostic plots.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev.R
\name{init.glme}
\alias{init.glme}
\title{Initialize starting values for GLME optimization}
\usage{
init.glme(
  data = NULL,
  ntry = NULL,
  model = "gev00",
  pretheta = NULL,
  xdat = NULL
)
}
\arguments{
\item{data}{A numeric vector of data to be fitted.}

\item{ntry}{Number of initial parameter sets to generate (minimum 2).}

\item{model}{Either "gev00" (stationary, default) or "gev11".}

\item{pretheta}{Optional pre-estimate (mu0, mu1, sigma0, sigma1, xi)
used for the first candidate when \code{model="gev11"}.}

\item{xdat}{Deprecated alias of \code{data} (v1.x compatibility).}
}
\value{
A matrix with \code{ntry} rows and 3 columns,
  where each row is a candidate starting point for optimization.
}
\description{
Generates multiple starting parameter sets for multi-start optimization
in GLME estimation, for both the stationary GEV (\code{model="gev00"})
and the non-stationary GEV11 (\code{model="gev11"}). Uses L-moment
estimates as a base and adds random perturbations.

For \code{model="gev11"} the second column is returned on the log scale
(matching sigma0 of the GEV11 parameterization), and when \code{pretheta}
is supplied the first candidate is a perturbation of the pre-estimate.
}
\examples{
data(haenam)
inits <- init.glme(haenam$X1, ntry = 5)
print(inits)

}
\seealso{
\code{\link{glme.gev}}, \code{\link{glme.gev11}} which use this
  function internally.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ran.gev.R
\name{qns.gev11}
\alias{qns.gev11}
\title{Quantile function for GEV11 model}
\usage{
qns.gev11(Tp = NULL, para = NULL, year = NULL)
}
\arguments{
\item{Tp}{Return period.}

\item{para}{Parameter vector (mu0, mu1, sigma0, sigma1, xi).}

\item{year}{Time vector.}
}
\value{
Numeric vector of quantile values (one per time point).
}
\description{
Internal wrapper that computes time-varying quantiles for the
GEV11 model given a return period.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsgev.R
\name{nsgev}
\alias{nsgev}
\title{Non-stationary GEV Parameter Estimation}
\usage{
nsgev(xdat, ntry = 20, ftol = 1e-06)
}
\arguments{
\item{xdat}{A numeric vector of data to be fitted (e.g., annual maximum values).}

\item{ntry}{Number of attempts for optimization (default 20).}

\item{ftol}{Function tolerance for optimization (default 1e-6).}
}
\value{
A list containing:
\itemize{
  \item \code{para.prop} - The proposed L-moment based estimates
        (mu0, mu1, sigma0, sigma1, xi).
  \item \code{precis} - Precision of the optimization.
}
}
\description{
Estimates parameters of a non-stationary Generalized Extreme Value (GEV)
distribution using the L-moment-based algorithm from Shin et al. (2025,
J. Korean Stat. Soc.).

This is a convenience wrapper around \code{\link{lme.gev11}}, providing
compatibility with the original nsgev package interface (and with
GLmom v1.x).
}
\examples{
data(PhliuAgromet)
result <- nsgev(PhliuAgromet$prec, ntry = 5)
print(result$para.prop)

}
\references{
Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
\doi{10.1007/s42952-025-00325-3}
}
\seealso{
\code{\link{lme.gev11}} for the recommended interface,
  \code{\link{glme.gev11}} for the full GLME method with penalty functions,
  \code{\link{gado.prop_11}} for detailed estimation results.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsgev.R
\name{gado.prop_11}
\alias{gado.prop_11}
\title{Comprehensive Non-stationary GEV Estimation (deprecated)}
\usage{
gado.prop_11(xdat, ntry = 20, ftol = 1e-06)
}
\arguments{
\item{xdat}{A numeric vector of data to be fitted.}

\item{ntry}{Number of attempts for optimization (default 20).}

\item{ftol}{Function tolerance for optimization (default 1e-6).}
}
\value{
A list containing:
\itemize{
  \item \code{para.prop} - L-moment based estimates (proposed method).
  \item \code{para.gado} - GN16 method estimates.
  \item \code{para.wls} - Weighted least squares estimates.
  \item \code{strup.org} - Original non-stationary WLSE by Strup method.
  \item \code{lme.sta} - Stationary L-moment estimates.
}
}
\description{
Estimates parameters of a non-stationary GEV distribution using multiple
methods: Weighted Least Squares (WLS), GN16 method, and the proposed
L-moment method from Shin et al. (2025, J. Korean Stat. Soc.).

As of GLmom v2.0.0 this function is deprecated: it was renamed to
\code{\link{lme.gev11}} (for the proposed method), and the auxiliary
methods are available as \code{\link{strup.gev11}} and
\code{\link{GN16.gev11}}. This wrapper reassembles the v1.x output
format from those functions and will be removed in a future release.
}
\examples{
\donttest{
data(PhliuAgromet)
# deprecated wrapper; suppressWarnings() silences the deprecation notice
result <- suppressWarnings(gado.prop_11(PhliuAgromet$prec, ntry = 5))
print(result$para.prop)
print(result$lme.sta)
}

}
\references{
Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
\doi{10.1007/s42952-025-00325-3}
}
\seealso{
\code{\link{lme.gev11}} for the recommended interface,
  \code{\link{strup.gev11}}, \code{\link{GN16.gev11}},
  \code{\link{glme.gev11}} for the full GLME method,
  \code{\link{nsgev}} for the simple interface.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev.R
\name{boot.cov}
\alias{boot.cov}
\title{Bootstrap covariance of sample L-moments}
\usage{
boot.cov(xdat, BB)
}
\arguments{
\item{xdat}{Numeric data vector.}

\item{BB}{Number of bootstrap replicates.}
}
\value{
A 3x3 covariance matrix.
}
\description{
Computes the covariance matrix of the first three sample
L-moments by nonparametric bootstrap. Used as a fallback when the
direct covariance estimate is singular.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{gof.ene_all}
\alias{gof.ene_all}
\title{Goodness-of-fit by number of exceedances}
\usage{
gof.ene_all(xdat, vecT = c(5, 10, 20, 40, 80), para = NULL, model = NULL)
}
\arguments{
\item{xdat}{Numeric vector of data.}

\item{vecT}{Vector of return periods.}

\item{para}{Parameter vector.}

\item{model}{Model name string.}
}
\value{
Sum of the relative absolute differences.
}
\description{
Internal function computing a goodness-of-fit measure
comparing the expected and observed numbers of exceedances of the
T-year levels for a set of return periods.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{comp.prof.ci}
\alias{comp.prof.ci}
\title{Compute profile likelihood confidence intervals}
\usage{
comp.prof.ci(d, v, conf = NULL)
}
\arguments{
\item{d}{Data frame with columns \code{x} (xi values) and \code{v} (log-likelihood).}

\item{v}{Numeric vector of log-likelihood values.}

\item{conf}{Numeric vector of confidence levels (e.g., 0.95).}
}
\value{
A list containing:
\describe{
  \item{vmax}{Maximum log-likelihood value}
  \item{nllh}{Same as vmax}
  \item{xmax}{xi value at maximum likelihood}
  \item{ci1}{Lower CI bounds for each confidence level}
  \item{ci2}{Upper CI bounds for each confidence level}
  \item{ci_length}{CI lengths for each confidence level}
}
}
\description{
Internal function that extracts confidence intervals from
a profile log-likelihood curve at specified confidence levels.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strup.gev11.R
\name{strup.gev11}
\alias{strup.gev11}
\title{Weighted least squares estimation for the non-stationary GEV11 model}
\usage{
strup.gev11(xdat, init.rob = TRUE, wls.rob = FALSE, strup.fin.rob = TRUE)
}
\arguments{
\item{xdat}{A numeric vector of data to be fitted.}

\item{init.rob}{If TRUE (default), the initial location trend is fitted
by robust regression (\code{\link[robustbase]{lmrob}}); otherwise by
ordinary least squares.}

\item{wls.rob}{If TRUE, the log-scale regression uses robust regression
(default FALSE).}

\item{strup.fin.rob}{If TRUE (default), the final location trend
re-specification uses robust regression.}
}
\value{
A list containing:
\itemize{
  \item strup.para - Original WLS estimates (mu0, mu1, sigma0, sigma1, xi).
  \item strup.mdfy - Modified (re-specified) WLS estimates.
}
}
\description{
Estimates the parameters of the non-stationary GEV11 model by the
weighted least squares (WLS) method of Strupczewski and Kaczmarek (2001),
with the final-specification modification described in Shin et al.
(2025b, JKSS). The result is used as the pre-estimate (starting value)
by \code{\link{glme.gev11}} and \code{\link{lme.gev11}}.
}
\examples{
data(PhliuAgromet)
result <- strup.gev11(PhliuAgromet$prec)
print(result$strup.mdfy)

}
\references{
Strupczewski, W. G. & Kaczmarek, Z. (2001). Non-stationary approach to
at-site flood frequency modelling II. Weighted least squares estimation.
Journal of Hydrology, 248(1-4), 143-151.

Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
\doi{10.1007/s42952-025-00325-3}
}
\seealso{
\code{\link{glme.gev11}}, \code{\link{lme.gev11}},
  \code{\link{GN16.gev11}}.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{gev.rl.delta}
\alias{gev.rl.delta}
\title{MLE with return level and delta method SE}
\usage{
gev.rl.delta(data, ntry = 5, quant)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{ntry}{Number of optimization attempts. Default is 5.}

\item{quant}{Numeric vector of probabilities for quantile estimation.}
}
\value{
A list containing:
\describe{
  \item{nllh}{Negative log-likelihood}
  \item{mle}{MLE estimates (mu, sigma, xi) in Hosking style}
  \item{qua.mle}{Quantile estimates at \code{quant} probabilities}
  \item{data}{Input data}
  \item{cov}{3x3 covariance matrix from MLE}
  \item{quant}{Input quantile probabilities}
}
}
\description{
Internal function that computes GEV MLE using both
constrained optimization (solnp) and ismev::gev.fit, selects the
better fit, and returns the covariance matrix for delta method SE.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev.R
\name{ma.gev}
\alias{ma.gev}
\title{Model Averaging for GEV High Quantile Estimation}
\usage{
ma.gev(
  data = NULL,
  quant = c(0.98, 0.99, 0.995),
  weight = "like1",
  numk = 12,
  B = 200,
  varcom = TRUE,
  trim = 0,
  fig = FALSE,
  bma = FALSE,
  pen = "norm",
  CD = FALSE,
  remle = FALSE
)
}
\arguments{
\item{data}{A numeric vector of data to be fitted (e.g., annual maxima).}

\item{quant}{The probabilities corresponding to high quantiles to be estimated.
Default is c(0.98, 0.99, 0.995).}

\item{weight}{The weighting method name. Options are:
\itemize{
  \item 'like', 'like0', 'like1' (default): Likelihood-based weights (AIC)
  \item 'gLd', 'gLd0', 'gLd1', 'gLd2': Generalized L-moment distance weights
  \item 'med', 'med1', 'med2': Median-based weights
  \item 'cvt': Conventional AIC weights
}
Variants with numbers indicate left trimming level (0, 1, or 2).}

\item{numk}{The number of candidate submodels K. Default is 12.}

\item{B}{The number of bootstrap samples. Default is 200.}

\item{varcom}{Logical. Whether to compute variance of quantile estimates.
Default is TRUE.}

\item{trim}{The number of left trimming for L-moments. Usually 0 (default), 1, or 2.}

\item{fig}{Logical. Whether to produce diagnostic plots. Default is FALSE.}

\item{bma}{Logical. Whether to use Bayesian Model Averaging. Default is FALSE.}

\item{pen}{Penalty type for BMA prior: 'norm' (normal, default) or 'beta'.}

\item{CD}{Logical. Whether to compute Coles-Dixon penalized MLE. Default is FALSE.}

\item{remle}{Logical. Whether to compute restricted MLE. Default is FALSE.}
}
\value{
A list containing:
\itemize{
  \item mle.hosking - MLE estimates in Hosking style (mu, sigma, xi)
  \item qua.mle - Quantile estimates from MLE
  \item mle.cov3 - Covariance matrix of MLE (3x3)
  \item qua.se.mle.delta - Standard errors of MLE quantiles (delta method)
  \item lme - L-moment estimates (mu, sigma, xi)
  \item lme.cov3 - Covariance matrix of LME (bootstrap)
  \item qua.lme - Quantile estimates from LME
  \item qua.se.lme.boots - Standard errors of LME quantiles (bootstrap)
  \item qua.ma - Model-averaged quantile estimates
  \item w.ma - Weights used for model averaging
  \item fixw.se.ma - Asymptotic SE under fixed weights
  \item ranw.se.ma - Asymptotic SE under random weights
  \item surr - Surrogate model parameters (mu, sigma, xi)
  \item pick_xi - Selected xi values for K submodels
  \item qua.bma - (if bma=TRUE) BMA quantile estimates
  \item w.bma - (if bma=TRUE) BMA weights
  \item mle.CD - (if CD=TRUE) Coles-Dixon penalized MLE
  \item qua.CD - (if CD=TRUE) Quantile estimates from CD-MLE
  \item remle1 - (if remle=TRUE) Restricted MLE (first constraint)
  \item qua.remle1 - (if remle=TRUE) Quantile estimates from remle1
  \item remle2 - (if remle=TRUE) Restricted MLE (second constraint)
  \item qua.remle2 - (if remle=TRUE) Quantile estimates from remle2
  \item quant - The quantile probabilities used
}
}
\description{
This function estimates high quantiles of the Generalized Extreme Value (GEV)
distribution using model averaging with mixed criteria. It combines Maximum
Likelihood Estimation (MLE) and L-moment Estimation (LME) to construct
candidate submodels and assign weights effectively.
}
\details{
The model averaging approach works as follows:
\enumerate{
  \item MLE and LME of GEV parameters are computed
  \item K candidate shape parameters (xi) are selected from profile likelihood CI
  \item For each candidate xi, MLE with fixed xi is computed
  \item Weights are assigned based on the selected method
  \item Final quantile estimates are weighted averages across submodels
}

The weighting schemes include:
\itemize{
  \item 'like': AIC-based weights using likelihood with fixed xi
  \item 'gLd': Weights based on generalized L-moment distance
  \item 'med': Weights based on median and L-moment distance
  \item 'cvt': Conventional AIC weights
}

When bma=TRUE, Bayesian model averaging is applied with prior specified by pen.
}
\examples{
# Load example data
data(haenam)
x <- haenam$X1

# Basic usage with likelihood weights
result <- ma.gev(x, quant = c(0.95, 0.99), weight = 'like1', B = 100)
print(result$qua.ma)     # Model-averaged quantiles
print(result$qua.mle)    # MLE quantiles for comparison
print(result$qua.lme)    # LME quantiles for comparison

# Using generalized L-moment distance weights
result2 <- ma.gev(x, quant = c(0.95, 0.99), weight = 'gLd', B = 100)
print(result2$w.ma)      # Model weights

}
\references{
Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
for estimating high quantiles of extreme values: Application to heavy rainfall.
\emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
\doi{10.1007/s00477-025-03167-x}
}
\seealso{
\code{\link{glme.gev}} for stationary GLME estimation,
  \code{\link{magev.ksensplot}} for K sensitivity analysis,
  \code{\link{magev.qqplot}} for Q-Q diagnostic plots,
  \code{\link{magev.rlplot}} for return level plots.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{delta.gev}
\alias{delta.gev}
\title{Delta method variance and cross-covariance for GEV quantiles}
\usage{
delta.gev(
  gevf3 = NULL,
  mle3i = NULL,
  cov2i = NULL,
  mle3j = NULL,
  cov2j = NULL,
  quant = NULL,
  d3yes = FALSE
)
}
\arguments{
\item{gevf3}{Full MLE result list with \code{cov} and \code{mle} (used when d3yes=TRUE).}

\item{mle3i}{Parameter vector for submodel i (used when d3yes=FALSE).}

\item{cov2i}{2x2 covariance matrix for submodel i.}

\item{mle3j}{Parameter vector for submodel j.}

\item{cov2j}{2x2 covariance matrix for submodel j.}

\item{quant}{Numeric vector of quantile probabilities.}

\item{d3yes}{Logical. If TRUE, compute 3-parameter delta method variance.}
}
\value{
A list containing:
\describe{
  \item{v3}{Variance for each quantile (if d3yes=TRUE)}
  \item{covij}{Cross-covariance for each quantile (if d3yes=FALSE)}
}
}
\description{
Internal function that computes delta method variances
for GEV quantile estimates. Can compute either the variance from
a full 3-parameter MLE (d3yes=TRUE) or the cross-covariance between
two submodels with fixed xi (d3yes=FALSE).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{GLmom-methods}
\alias{GLmom-methods}
\alias{print.glme}
\alias{summary.glme}
\alias{plot.glme}
\alias{print.glme11}
\alias{summary.glme11}
\alias{plot.glme11}
\alias{print.lme11}
\alias{summary.lme11}
\alias{plot.lme11}
\alias{print.magev}
\alias{summary.magev}
\alias{plot.magev}
\title{Methods for fitted GLmom objects}
\usage{
\method{print}{glme}(x, ...)

\method{summary}{glme}(object, ...)

\method{plot}{glme}(x, ...)

\method{print}{glme11}(x, ...)

\method{summary}{glme11}(object, ...)

\method{plot}{glme11}(x, ...)

\method{print}{lme11}(x, ...)

\method{summary}{lme11}(object, ...)

\method{plot}{lme11}(x, ...)

\method{print}{magev}(x, ...)

\method{summary}{magev}(object, ...)

\method{plot}{magev}(x, ...)
}
\arguments{
\item{x, object}{A fitted object.}

\item{...}{Further arguments passed to plotting functions; ignored by
\code{print} and \code{summary}.}
}
\value{
\code{x} (or \code{object}), invisibly.
}
\description{
The estimation functions return classed objects: \code{\link{glme.gev}}
returns an object of class \code{"glme"}, \code{\link{glme.gev11}} of
class \code{"glme11"}, \code{\link{lme.gev11}} of class \code{"lme11"},
and \code{\link{ma.gev}} of class \code{"magev"}. The objects remain
ordinary lists, so all documented fields (e.g., \code{$para.glme}) stay
accessible as before.

\code{print} shows the parameter estimates and the penalty settings;
\code{summary} additionally reports the auxiliary estimates stored in
the object; \code{plot} draws a quantile-quantile diagnostic: against
the fitted GEV distribution for \code{"glme"}, on the standard Gumbel
scale (after the transformation of Eq. 8 in the manuscript) for
\code{"glme11"} and \code{"lme11"}, and via \code{\link{magev.qqplot}}
for \code{"magev"}.
}
\examples{
data(haenam)
fit <- glme.gev(haenam$X1)
fit
summary(fit)
plot(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ran.gev.R
\name{ran.gev_all}
\alias{ran.gev_all}
\title{Random number generation for non-stationary GEV models}
\usage{
ran.gev_all(nsample, para, model = NULL)
}
\arguments{
\item{nsample}{Sample size (number of time points).}

\item{para}{Parameter vector; its length and meaning depend on
\code{model} (see \code{\link{quagev.NS}}). For "gev11":
(mu0, mu1, sigma0, sigma1, xi).}

\item{model}{Model type: "gev", "gev00", "gev01", "gev10", "gev11",
or "gev20".}
}
\value{
A numeric vector of length \code{nsample}.
}
\description{
Generates a random sample from a stationary or non-stationary GEV model.
For non-stationary models the location and/or scale vary with time
t = 1, ..., nsample:
mu(t) = mu0 + mu1*t + mu2*t^2 and sigma(t) = exp(sigma0 + sigma1*t).
}
\examples{
set.seed(1)
# Non-stationary GEV11 sample of size 50
x <- ran.gev_all(50, para = c(100, 0.5, 3, 0.005, -0.2), model = "gev11")
plot(x, type = "l")

}
\references{
Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
\doi{10.1007/s42952-025-00325-3}
}
\seealso{
\code{\link{quagev.NS}} for the corresponding quantile function,
  \code{\link{glme.gev11}}, \code{\link{lme.gev11}}.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{new.kpar2}
\alias{new.kpar2}
\title{Adaptively expand or prune candidate xi set}
\usage{
new.kpar2(
  wtgd = NULL,
  numk = NULL,
  kpar = NULL,
  remove = 0.004,
  dist = 0.02,
  tre1 = 0.01,
  tre8 = 0.5
)
}
\arguments{
\item{wtgd}{Numeric vector of current weights.}

\item{numk}{Number of candidate submodels.}

\item{kpar}{Numeric vector of candidate xi values.}

\item{remove}{Threshold below which candidates are removed. Default is 0.004.}

\item{dist}{Spacing for new candidates. Default is 0.02.}

\item{tre1}{Lower threshold for boundary detection. Default is 0.01.}

\item{tre8}{Upper threshold for dominant candidate. Default is 0.5.}
}
\value{
A list containing:
\describe{
  \item{kpar2}{Updated candidate xi values}
  \item{aw}{Adjustment indicator (0 = no change needed)}
  \item{numk}{Updated number of candidates}
}
}
\description{
Internal function that adaptively adjusts the candidate xi
set by removing low-weight candidates and adding new candidates near
boundary or dominant regions to improve weight coverage.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{gev.xilik2}
\alias{gev.xilik2}
\title{GEV negative log-likelihood with fixed xi (wrapper)}
\usage{
gev.xilik2(a, xifix = xifix, xdat = xdat)
}
\arguments{
\item{a}{Numeric vector of (mu, sigma).}

\item{xifix}{Fixed shape parameter value.}

\item{xdat}{Numeric vector of data.}
}
\value{
Negative log-likelihood value (scalar).
}
\description{
Internal wrapper function that calls \code{gev.xilik()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalty.R
\name{MS_pk}
\alias{MS_pk}
\title{Martins-Stedinger prior function}
\usage{
MS_pk(para = para, p = 6, q = 9)
}
\arguments{
\item{para}{A vector of GEV parameters (location, scale, shape).}

\item{p}{Shape parameter for beta distribution (default 6).}

\item{q}{Shape parameter for beta distribution (default 9).}
}
\value{
Prior probability value (scalar).
}
\description{
Computes the Martins-Stedinger Beta(6,9) prior probability for the GEV
shape parameter on the interval \eqn{[-0.5, 0.5]}.
}
\examples{
# Evaluate MS prior at xi = -0.2
MS_pk(para = c(100, 20, -0.2))

}
\references{
Martins, E. S. & Stedinger, J. R. (2000). Generalized maximum-likelihood
generalized extreme-value quantile estimators for hydrologic data.
Water Resources Research, 36(3), 737-744.
\doi{10.1029/1999WR900330}
}
\seealso{
\code{\link{pk.beta}} for the adaptive beta penalty,
  \code{\link{glme.gev}} which uses these penalty functions.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{asymp.var}
\alias{asymp.var}
\title{Asymptotic variance of model-averaged quantile estimates}
\usage{
asymp.var(mywt, covint, qqq = NULL, order = 2)
}
\arguments{
\item{mywt}{Weight computation result list from \code{weight.com()}.}

\item{covint}{Array (2 x 2 x numk) of interpolated covariance matrices.}

\item{qqq}{Numeric vector of quantile probabilities.}

\item{order}{Moving average order for smoothing. Default is 2.}
}
\value{
A list containing:
\describe{
  \item{fin.se.MA.qua}{SE under fixed weights for MA}
  \item{fin.se.bma.qua}{SE under fixed weights for BMA}
  \item{adj.se.MA.qua}{SE under random weights for MA}
  \item{adj.se.bma.qua}{SE under random weights for BMA}
  \item{MatC}{Cross-covariance array (numk x numk x numq)}
}
}
\description{
Internal function that computes asymptotic standard errors
for model-averaged quantile estimates under both fixed-weight and
random-weight assumptions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strup.gev11.R
\name{wls.gev11}
\alias{wls.gev11}
\title{Log-scale regression step of the Strupczewski WLS method}
\usage{
wls.gev11(xdat, res = NULL, rob = NULL)
}
\arguments{
\item{xdat}{Numeric data vector.}

\item{res}{Residuals from the location trend fit.}

\item{rob}{If TRUE, uses robust regression (setting "KS2014").}
}
\value{
A list with element \code{sig} (the two coefficients).
}
\description{
Internal step of \code{\link{strup.gev11}}: regresses the
log absolute residuals on time to estimate the log-scale coefficients
(sigma0, sigma1).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev.R
\name{init.gevmax}
\alias{init.gevmax}
\title{Initialize parameters for GEV MLE estimation}
\usage{
init.gevmax(data = NULL, ntry = NULL)
}
\arguments{
\item{data}{A numeric vector of data to be fitted.}

\item{ntry}{Number of initial parameter sets to generate.}
}
\value{
A matrix with `ntry` rows and 3 columns, where each row represents
a set of initial parameters (location, scale, shape) for the GEV distribution.
}
\description{
This function initializes parameters for GEV maximum likelihood estimation.
}
\details{
The function generates `ntry` sets of initial parameters for the GEV distribution.
It uses L-moment estimates as a starting point and then generates additional
sets of parameters using random perturbations.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{PrescottW}
\alias{PrescottW}
\title{Prescott-Walden expected information matrix for GEV with fixed xi}
\usage{
PrescottW(par2 = NULL, xifix = NULL, nsam = NULL)
}
\arguments{
\item{par2}{Numeric vector of (mu, sigma) estimates.}

\item{xifix}{Fixed shape parameter value.}

\item{nsam}{Sample size.}
}
\value{
A 2x2 expected information matrix.
}
\description{
Internal function that computes the Prescott-Walden expected
information (Hessian) matrix for GEV parameters (mu, sigma) with fixed xi.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{ginit.xifix}
\alias{ginit.xifix}
\title{Initialize parameters for GEV with fixed xi}
\usage{
ginit.xifix(data, ntry)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{ntry}{Number of initial parameter sets to generate.}
}
\value{
A matrix with \code{ntry} rows and 2 columns (mu, sigma).
}
\description{
Internal function that generates initial (mu, sigma) values
for optimization with fixed shape parameter, using Gumbel L-moment estimates
and random perturbations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{weight.com}
\alias{weight.com}
\title{Compute model averaging weights}
\usage{
weight.com(
  data = NULL,
  numk = NULL,
  hosking = NULL,
  kpar = NULL,
  numom = 3,
  xqa = NULL,
  varcom = TRUE,
  boot.lme = TRUE,
  cov.lme = NULL,
  surr = FALSE,
  type = "full",
  trim = NULL,
  cov.type = "ratio",
  bma = TRUE,
  pen = "norm"
)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{numk}{Number of candidate submodels.}

\item{hosking}{List containing LME results, MLE, and bootstrap information.}

\item{kpar}{Numeric vector of candidate xi values.}

\item{numom}{Number of L-moments to use (default 3).}

\item{xqa}{Probability vector for surrogate model fitting.}

\item{varcom}{Logical. Whether to compute variance components (default TRUE).}

\item{boot.lme}{Logical. Whether bootstrap LME was performed (default TRUE).}

\item{cov.lme}{Pre-computed LME covariance (default NULL).}

\item{surr}{Logical. Whether to compute surrogate quantiles (default FALSE).}

\item{type}{Type of computation: "full" (default).}

\item{trim}{Left trimming level for L-moments.}

\item{cov.type}{Covariance type: "ratio" or "lambda" (default "ratio").}

\item{bma}{Logical. Whether to compute BMA weights (default TRUE).}

\item{pen}{BMA prior type: "norm" or "beta" (default "norm").}
}
\value{
A list containing:
\describe{
  \item{weight}{Weighting method name}
  \item{numk}{Number of submodels}
  \item{wtgd}{MA weight vector (length numk)}
  \item{bmaw}{BMA weight vector (length numk)}
  \item{zp}{Quantile matrix (numq x numk)}
  \item{kpar}{Candidate xi values}
  \item{prob.call}{Submodel fitting results}
  \item{xzp}{Surrogate quantile matrix (if surr=TRUE)}
}
}
\description{
Internal function that computes weights for model averaging
across K candidate GEV submodels. Supports multiple weighting schemes
including likelihood-based, generalized L-moment distance, and median-based
methods. Optionally includes BMA prior integration.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{trans.gum01}
\alias{trans.gum01}
\title{Transform non-stationary GEV data to standard Gumbel}
\usage{
trans.gum01(xdat, para = NULL, na.ok = FALSE)
}
\arguments{
\item{xdat}{Numeric vector of data.}

\item{para}{GEV11 parameter vector (mu0, mu1, sigma0, sigma1, xi).}

\item{na.ok}{If TRUE, invalid (out-of-support) values are returned as NA;
if FALSE, they are clipped.}
}
\value{
Numeric vector on the standard Gumbel scale.
}
\description{
Transforms observations from a non-stationary GEV11 model
with parameters \code{para} = (mu0, mu1, sigma0, sigma1, xi) to the
standard Gumbel scale.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bangkok}
\alias{bangkok}
\title{Bangkok Maximum Rainfall Data}
\format{
A data frame with 58 rows and 5 columns:
\describe{
  \item{X1}{Annual maximum daily rainfall in mm (numeric)}
  \item{X2}{2nd largest annual daily rainfall in mm (numeric)}
  \item{X3}{3rd largest annual daily rainfall in mm (numeric)}
  \item{X4}{4th largest annual daily rainfall in mm (numeric)}
  \item{X5}{5th largest annual daily rainfall in mm (numeric)}
}
}
\source{
Thai Meteorological Department (TMD; \url{https://www.tmd.go.th})
}
\usage{
bangkok
}
\description{
Annual maximum daily rainfall data from Bangkok, Thailand.
This dataset is used for demonstrating model averaging methods
for high quantile estimation in extreme value analysis.
}
\examples{
data(bangkok)
head(bangkok)

# Estimate high quantiles using model averaging
result <- ma.gev(bangkok$X1, quant = c(0.99, 0.995))
print(result$qua.ma)

}
\references{
Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
for estimating high quantiles of extreme values: Application to heavy rainfall.
\emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
\doi{10.1007/s00477-025-03167-x}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{glme.gev11}
\alias{glme.gev11}
\title{Generalized L-moments estimation for non-stationary GEV11 model}
\usage{
glme.gev11(
  xdat,
  ntry = 5,
  ftol = 1e-06,
  opt.choose = "nllh",
  pen = "beta",
  pen.choice = 1,
  mu = -0.55,
  std = 0.2,
  p = 6,
  c1 = 5,
  c2 = 2,
  c0 = 0.35,
  q = NULL,
  show = FALSE,
  init.rob = TRUE,
  glme.pre = NULL
)
}
\arguments{
\item{xdat}{A numeric vector of data to be fitted.}

\item{ntry}{Number of attempts for parameter estimation (default 5).}

\item{ftol}{Tolerance for convergence (default 1e-6).}

\item{opt.choose}{Selection criterion among the multi-start solutions:
"nllh" (default, penalized negative log-likelihood) or "gof"
(goodness-of-fit by the number-of-exceedances measure).}

\item{pen}{Type of penalty function: "norm", "beta" (default), "ms",
"park", "cannon", "cd", or "no".}

\item{pen.choice}{Choice number for penalty hyperparameters (default 1).
See \code{\link{glme.gev}} for the preset values. Set to NULL to use
\code{p, c1, c2} (beta) or \code{mu, std} (norm) directly.}

\item{mu}{Mean for normal penalty (default -0.55).}

\item{std}{Std for normal penalty (default 0.2).}

\item{p}{Shape for beta penalty (default 6).}

\item{c1}{Scaling for beta penalty (default 5).}

\item{c2}{Limit for beta penalty (default 2).}

\item{c0}{Half-width of the adaptive beta penalty support (default 0.35).}

\item{q}{Optional fixed second shape parameter for the beta penalty.}

\item{show}{If TRUE, prints the objective value of each try.}

\item{init.rob}{Use robust regression for the WLS initialization
(default TRUE).}

\item{glme.pre}{Deprecated (ignored). The WLS pre-estimate is always used.}
}
\value{
A list containing:
\itemize{
  \item para.glme - Proposed GLME estimates (mu0, mu1, sigma0, sigma1, xi).
  \item nllh.glme - Penalized negative log-likelihood of the solution.
  \item convergence - 0 if converged, 5 otherwise.
  \item pen, pen_pen.choice - Penalty method (and choice) used.
  \item p_q, c1_c2, c0_c1_c2 - (for beta) hyperparameters used.
  \item mu_std - (for norm) hyperparameters used.
  \item strup.org - WLSE by the Strupczewski method (\code{strup.para}).
  \item para.wls - Modified WLS estimates (\code{strup.mdfy}).
  \item para.gado - GN16 method estimates.
  \item lme.sta - Stationary L-moment estimates.
  \item para.lme - Pure L-moment estimates; returned only when
    \code{pen="no"} (then identical to para.glme). For other penalties
    use \code{\link{lme.gev11}} directly.
  \item precis - (only when \code{pen="no"}) precision of the L-moment
    equation solution.
  \item data - The input data (used by the plot method).
}
The object has class \code{"glme11"}; see \code{\link{GLmom-methods}}.
}
\description{
This function estimates parameters of the non-stationary GEV11 model
where mu(t) = mu0 + mu1*t and sigma(t) = exp(sigma0 + sigma1*t),
by the generalized method of L-moment estimation (GLME).

Starting from the WLS pre-estimate of \code{\link{strup.gev11}},
the penalized objective (generalized L-moment distance based on the
asymptotic covariance of the sample L-moments, plus penalty) is
minimized over (mu0, sigma0, xi) with mu1 and sigma1 held at their
pre-estimated values. With \code{pen="no"} the result equals the pure
L-moment method \code{\link{lme.gev11}} of Shin et al. (2025b).
}
\examples{
data(PhliuAgromet)
x <- PhliuAgromet$prec

\donttest{
# Estimate non-stationary GEV11 parameters
result <- glme.gev11(x, ntry = 5)
print(result$para.glme)  # Proposed GLME estimates
}

}
\references{
Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
L-moment estimation for stationary and nonstationary extreme value models.
arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}

Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
\doi{10.1007/s42952-025-00325-3}
}
\seealso{
\code{\link{glme.gev}} for stationary GEV estimation,
  \code{\link{lme.gev11}} for the pure L-moment method (no penalty),
  \code{\link{strup.gev11}} for the WLS pre-estimation,
  \code{\link{GN16.gev11}} for the GN16 method,
  \code{\link{quagev.NS}} for non-stationary quantile computation.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{prior.beta.stnary.ma}
\alias{prior.beta.stnary.ma}
\title{Beta prior for model averaging}
\usage{
prior.beta.stnary.ma(x = NULL, xi_lme = NULL, p = 2, q = 5, al, bl)
}
\arguments{
\item{x}{Shape parameter value to evaluate.}

\item{xi_lme}{L-moment estimate of xi (unused in this version).}

\item{p}{Beta shape parameter p. Default is 2.}

\item{q}{Beta shape parameter q. Default is 5.}

\item{al}{Lower bound of support.}

\item{bl}{Upper bound of support.}
}
\value{
Prior probability value (scalar).
}
\description{
Internal function that computes a Beta distribution prior
probability for a candidate shape parameter in model averaging.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{lme.boots}
\alias{lme.boots}
\title{L-moment estimation with bootstrap standard errors}
\usage{
lme.boots(data, B = NULL, quant, boot = TRUE, trim = NULL)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{B}{Number of bootstrap samples.}

\item{quant}{Numeric vector of probabilities for quantile estimation.}

\item{boot}{Logical. If TRUE (default), perform bootstrap.}

\item{trim}{Left trimming level for L-moments (integer). Default is NULL (0).}
}
\value{
A list containing:
\describe{
  \item{lme}{L-moment estimates (mu, sigma, xi)}
  \item{qua.lme}{Quantile estimates from LME}
  \item{quant}{Input quantile probabilities}
  \item{cov.par}{3x3 covariance of bootstrap parameter estimates (if boot=TRUE)}
  \item{cov.lambda}{3x3 covariance of bootstrap L-moments (if boot=TRUE)}
  \item{qua.lme.se}{SE of quantile estimates from bootstrap (if boot=TRUE)}
}
}
\description{
Internal function that computes L-moment estimates of GEV
parameters and optionally performs bootstrap resampling to obtain
covariance matrices and standard errors for quantile estimates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{cd.hos}
\alias{cd.hos}
\title{Coles-Dixon Penalty Function}
\usage{
cd.hos(sxi)
}
\arguments{
\item{sxi}{Shape parameter value (Hosking style, negative for heavy tails).}
}
\value{
Prior probability value.
}
\description{
Computes the Coles-Dixon prior probability for the shape parameter.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev.R
\name{glme.like}
\alias{glme.like}
\title{Calculate the likelihood for Generalized L-moments estimation of GEV distribution}
\usage{
glme.like(
  par,
  xdat = xdat,
  slmgev = slmgev,
  covinv = covinv,
  lcovdet = lcovdet,
  mu = mu,
  std = std,
  lme = lme,
  pen = pen,
  p = p,
  c1 = c1,
  c2 = c2,
  c0 = 0.35,
  q = NULL
)
}
\arguments{
\item{par}{A vector of GEV parameters (location, scale, shape).}

\item{xdat}{A numeric vector of data.}

\item{slmgev}{Sample L-moments of the data.}

\item{covinv}{Inverse of the covariance matrix of the sample L-moments.}

\item{lcovdet}{Log determinant of the covariance matrix.}

\item{mu}{Mean for the normal penalization (used when pen='norm').}

\item{std}{Standard deviation for the normal penalization (used when pen='norm').}

\item{lme}{L-moment estimates of the parameters.}

\item{pen}{Penalization method: 'norm', 'beta', 'ms', 'park', 'cannon', 'cd', or 'no'.}

\item{p}{Shape parameter for beta penalty.}

\item{c1}{Scaling parameter for beta penalty.}

\item{c2}{Upper limit parameter for beta penalty.}

\item{c0}{Half-width of the adaptive beta penalty support (default 0.35).}

\item{q}{Optional fixed second shape parameter for the beta penalty.}
}
\value{
A numeric value representing the penalized negative log-likelihood.
A lower value indicates a better fit.
}
\description{
This function calculates the likelihood (or more precisely, a penalized negative log-likelihood)
for the Generalized L-moments estimation of the Generalized Extreme Value (GEV) distribution.
}
\details{
The function performs the following steps:
1. Checks if the parameters are within valid ranges.
2. Calculates the expected L-moments based on the current parameters.
3. Computes the difference between expected and sample L-moments.
4. Calculates the generalized L-moments distance.
5. Applies a penalization term based on the specified method.
6. Returns the sum of the L-moments distance and the penalization term.
}
\examples{
data(haenam)
x <- haenam$X1
slm <- lmomco::lmoms(x, nmom = 3)
cov_mat <- lmomco::lmoms.cov(x, nmom = 3)
lme_par <- lmomco::pargev(slm)$para
glme.like(par = lme_par, xdat = x, slmgev = slm,
          covinv = solve(cov_mat), lcovdet = log(det(cov_mat)),
          mu = -0.5, std = 0.2, lme = lme_par, pen = "beta",
          p = 6, c1 = 3, c2 = 1)

}
\references{
Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
L-moment estimation for stationary and nonstationary extreme value models.
arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
}
\seealso{
\code{\link{glme.gev}} which calls this function for optimization.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{fun.lme.gev11}
\alias{fun.lme.gev11}
\title{L-moment distance function for GEV11 model}
\usage{
fun.lme.gev11(a, xdat = xdat, pretheta = pretheta)
}
\arguments{
\item{a}{Numeric vector (mu0, sigma0, xi) to optimize.}

\item{xdat}{Numeric vector of data.}

\item{pretheta}{Full parameter vector (mu0, mu1, sigma0, sigma1, xi)
from pre-estimation; mu1 and sigma1 are held fixed.}
}
\value{
Numeric vector of length 3 of L-moment equation residuals.
}
\description{
Computes the vector of differences between the population
L-moments of the standard Gumbel distribution and the sample L-moments
of the data transformed by \code{\link{trans.gum01}} under the current
GEV11 parameters. Solving this system equal to zero gives the L-moment
estimates (Shin et al., 2025b).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{lmoms.md.park}
\alias{lmoms.md.park}
\title{Modified L-moments computation with failure handling}
\usage{
lmoms.md.park(x, nmom = 5, mtrim = FALSE, no.stop = FALSE)
}
\arguments{
\item{x}{Numeric vector.}

\item{nmom}{Number of L-moments.}

\item{mtrim}{If TRUE, uses left-trimmed TL-moments (leftrim=5).}

\item{no.stop}{If TRUE, returns \code{ifail=1} instead of stopping on error.}
}
\value{
A list of L-moments with an additional \code{ifail} element.
}
\description{
A slightly modified version of \code{lmomco::lmoms} (by
J. Park) which returns an \code{ifail} flag instead of stopping when
the L-moments cannot be computed.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{cov.interp}
\alias{cov.interp}
\title{Interpolate missing covariance matrices across submodels}
\usage{
cov.interp(numk, para3, cov2 = NULL)
}
\arguments{
\item{numk}{Number of candidate submodels.}

\item{para3}{Matrix (numk x 3) of submodel parameters.}

\item{cov2}{List of 2x2 covariance matrices (one per submodel).}
}
\value{
Array (2 x 2 x numk) of covariance matrices with missing values
  filled by interpolation.
}
\description{
Internal function that fills in missing 2x2 covariance matrices
for submodels using natural spline interpolation across the shape parameter.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{gev.max}
\alias{gev.max}
\title{MLE for GEV distribution using constrained optimization}
\usage{
gev.max(xdat, ntry = 5)
}
\arguments{
\item{xdat}{Numeric vector of data.}

\item{ntry}{Number of random starting points for optimization. Default is 5.}
}
\value{
A list containing:
\describe{
  \item{nsample}{Sample size}
  \item{conv}{Convergence status (0 = success)}
  \item{nllh}{Negative log-likelihood at the optimum}
  \item{mle}{MLE estimates (mu, sigma, xi) in Hosking style}
}
}
\description{
Internal function that computes maximum likelihood estimates
of GEV parameters using the Rsolnp constrained optimizer with multiple
random starting points.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{cov.dir}
\alias{cov.dir}
\title{Dirichlet covariance matrix for weights}
\usage{
cov.dir(wtgd)
}
\arguments{
\item{wtgd}{Numeric vector of model weights.}
}
\value{
A (numk x numk) covariance matrix.
}
\description{
Internal function that computes the covariance matrix of
weights assuming a Dirichlet-type distribution.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GN16.gev11.R
\name{GN16.gev11}
\alias{GN16.gev11}
\title{GN16 estimation for the non-stationary GEV11 model}
\usage{
GN16.gev11(xdat, rob = FALSE)
}
\arguments{
\item{xdat}{A numeric vector of data to be fitted.}

\item{rob}{If TRUE, uses robust regression in the internal steps
(default FALSE).}
}
\value{
A list containing:
\itemize{
  \item para.gado.org - Original GN16 estimates
    (mu0, mu1, sigma0, sigma1, xi).
  \item para.gado.mdfy - Modified GN16 estimates.
}
}
\description{
Estimates the parameters of the non-stationary GEV11 model by the
quantile-based GN16 method (Gilleland and Katz, 2016; as adapted in
Shin et al., 2025b), with both the original and the modified
specification returned.
}
\examples{
data(PhliuAgromet)
result <- GN16.gev11(PhliuAgromet$prec)
print(result$para.gado.org)

}
\references{
Gilleland, E. & Katz, R. W. (2016). extRemes 2.0: An extreme value
analysis package in R. Journal of Statistical Software, 72(8), 1-39.

Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
\doi{10.1007/s42952-025-00325-3}
}
\seealso{
\code{\link{glme.gev11}}, \code{\link{lme.gev11}},
  \code{\link{strup.gev11}}.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{emp.prior}
\alias{emp.prior}
\title{Empirical prior for BMA based on MLE and LME}
\usage{
emp.prior(mle3, mle, lme)
}
\arguments{
\item{mle3}{Matrix of candidate submodel parameters (numk x 3).}

\item{mle}{MLE parameter estimates (mu, sigma, xi).}

\item{lme}{LME parameter estimates (mu, sigma, xi).}
}
\value{
Numeric vector of prior probability values for each candidate.
}
\description{
Internal function that computes an empirical prior distribution
for the shape parameter based on the range between MLE and LME estimates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{set.prior}
\alias{set.prior}
\title{Set BMA prior distribution for candidate shape parameters}
\usage{
set.prior(pen = NULL, numk = NULL, xi_lme = NULL, kpar = NULL, weight = NULL)
}
\arguments{
\item{pen}{Prior type: "norm" (normal) or "beta".}

\item{numk}{Number of candidate submodels.}

\item{xi_lme}{L-moment estimate of the shape parameter.}

\item{kpar}{Numeric vector of candidate xi values.}

\item{weight}{Weighting method name ("like", "gLd", or "med").}
}
\value{
A list containing:
\describe{
  \item{prior}{Numeric vector of prior probabilities (length numk)}
  \item{prior_mu_std}{(if pen="norm") Mean and std of the normal prior}
  \item{p_q_beta}{(if pen="beta") p and q parameters of the beta prior}
}
}
\description{
Internal function that sets up the prior distribution for
Bayesian Model Averaging (BMA) weights. Supports both normal and beta
priors, with hyperparameters adapted based on the weighting method and
L-moment estimate of xi.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{nllh.glme.gev11}
\alias{nllh.glme.gev11}
\title{GLME objective function for GEV11 model (mu0, sigma0, xi optimization)}
\usage{
nllh.glme.gev11(
  a,
  xdat = xdat,
  newtheta = newtheta,
  covinv = covinv,
  lcovdet = lcovdet,
  pen = pen,
  mu = mu,
  std = std,
  p = p,
  c0 = c0,
  c1 = c1,
  c2 = c2,
  q = q
)
}
\arguments{
\item{a}{Parameter vector (mu0, sigma0, xi).}

\item{xdat}{Data vector.}

\item{newtheta}{Full parameter vector (mu0, mu1, sigma0, sigma1, xi).}

\item{covinv}{Inverse covariance matrix of the sample L-moments.}

\item{lcovdet}{Log determinant of the covariance matrix.}

\item{pen}{Penalty type.}

\item{mu}{Normal penalty mean.}

\item{std}{Normal penalty std.}

\item{p}{Beta penalty shape.}

\item{c0}{Beta penalty support half-width.}

\item{c1}{Beta penalty scaling.}

\item{c2}{Beta penalty limit.}

\item{q}{Optional fixed second beta shape parameter.}
}
\value{
Penalized negative log-likelihood value.
}
\description{
GLME objective function for GEV11 model (mu0, sigma0, xi optimization)
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ran.gev.R
\name{set.para.model}
\alias{set.para.model}
\title{Set parameters based on non-stationary model type}
\usage{
set.para.model(para, model = NULL)
}
\arguments{
\item{para}{Parameter vector whose length and meaning depend on \code{model}.}

\item{model}{Model type: "gev11", "gev10", "gev20", "gev01", or "gev00"/"gev".}
}
\value{
A list containing:
\describe{
  \item{mu}{Numeric vector c(mu0, mu1, mu2) for location}
  \item{sig}{Numeric vector c(sigma0, sigma1) for log-scale}
  \item{xi}{Shape parameter (scalar)}
}
}
\description{
Internal function that maps a flat parameter vector to named
location (mu), scale (sig), and shape (xi) components depending on the
model specification (GEV00, GEV10, GEV11, GEV20, GEV01).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{gev.profxi.mdfy}
\alias{gev.profxi.mdfy}
\title{Modified profile likelihood for GEV shape parameter}
\usage{
gev.profxi.mdfy(
  data = NULL,
  mle = NULL,
  xlow,
  xup,
  pick.v = NULL,
  nint = 256,
  figure = FALSE
)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{mle}{MLE parameter estimates (mu, sigma, xi) in Coles parameterization.}

\item{xlow}{Lower bound for xi search.}

\item{xup}{Upper bound for xi search.}

\item{pick.v}{Numeric vector of confidence levels for CI computation.}

\item{nint}{Number of grid points. Default is 256.}

\item{figure}{Logical. Whether to plot the profile likelihood. Default is FALSE.}
}
\value{
A list containing:
\describe{
  \item{fail}{Logical. TRUE if profile likelihood is degenerate}
  \item{start}{Starting method recommendation}
  \item{w1}{Profile CI results from \code{comp.prof.ci()}}
  \item{ymin}{Minimum y value for plotting}
  \item{ymax}{Maximum y value for plotting}
}
}
\description{
Internal function that computes the profile log-likelihood
for the GEV shape parameter with linear extrapolation beyond the
observed range. Used for constructing confidence intervals on xi.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev11.R
\name{opt.glme.gev11}
\alias{opt.glme.gev11}
\title{Multi-start penalized optimization for the GEV11 model}
\usage{
opt.glme.gev11(
  xdat,
  ntry = 5,
  ftol = 1e-05,
  show = FALSE,
  pretheta = NULL,
  model = "gev11",
  pen = "beta",
  mu = NULL,
  std = NULL,
  p = 2,
  q = NULL,
  c0 = 0.35,
  c1 = 3,
  c2 = 2,
  opt.choose = "nllh"
)
}
\arguments{
\item{xdat}{Numeric data vector.}

\item{ntry}{Number of starting points.}

\item{ftol}{Convergence tolerance.}

\item{show}{If TRUE, prints progress.}

\item{pretheta}{Pre-estimated parameter vector.}

\item{model}{Model name ("gev11").}

\item{pen, mu, std, p, q, c0, c1, c2}{Penalty settings.}

\item{opt.choose}{"nllh" or "gof".}
}
\value{
A list with para.glme, nllh.glme, convergence
  (and precis when \code{pen="no"}).
}
\description{
Internal driver of \code{\link{glme.gev11}}. For
\code{pen="no"} it delegates to \code{\link{lme.gev11}}. Otherwise it
minimizes \code{\link{nllh.glme.gev11}} from several starting points,
using the fixed asymptotic covariance matrix of the first three sample
L-moments (computed for n=300 and rescaled by 300/n; see Eq. (26) of
Shin et al., 2025).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_plots.R
\name{magev.qqplot}
\alias{magev.qqplot}
\title{Q-Q Diagnostic Plot for MAGEV}
\usage{
magev.qqplot(data = NULL, zx = NULL)
}
\arguments{
\item{data}{A numeric vector of observed data.}

\item{zx}{A list object returned by \code{\link{ma.gev}} with \code{remle = TRUE}.}
}
\value{
NULL. The function produces a plot as a side effect.
}
\description{
Creates a 2x2 panel of Q-Q plots comparing observed vs. fitted quantiles
for different estimation methods: MLE, LME, surrogate (MA), and REMLE.
}
\details{
The function creates four Q-Q plots:
\itemize{
  \item Upper left: MLE (Maximum Likelihood Estimation)
  \item Upper right: LME (L-moment Estimation)
  \item Lower left: Surrogate MA (Model Averaging surrogate)
  \item Lower right: REMLE (Restricted MLE, if available)
}

Points close to the 45-degree diagonal line indicate good model fit.
}
\examples{
data(haenam)
qq <- c(seq(0.01, 0.99, by = 0.01), 0.995, 0.999)
zx <- ma.gev(haenam$X1, quant = qq, weight = 'like1',
             numk = 9, varcom = FALSE, remle = TRUE)
magev.qqplot(data = haenam$X1, zx = zx)

}
\references{
Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
for estimating high quantiles of extreme values: Application to heavy rainfall.
\emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
\doi{10.1007/s00477-025-03167-x}
}
\seealso{
\code{\link{ma.gev}} for the main model averaging function,
  \code{\link{magev.rlplot}} for return level plots.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{mle.gev.CD}
\alias{mle.gev.CD}
\title{Coles-Dixon Penalized MLE for GEV}
\usage{
mle.gev.CD(xdat, ntry = 10)
}
\arguments{
\item{xdat}{Numeric vector of data.}

\item{ntry}{Number of random starting points for optimization. Default is 10.}
}
\value{
A list containing:
  \item{mle}{MLE estimates (mu, sigma, xi) in Hosking style}
  \item{nllh}{Negative log-likelihood at the optimum}
  \item{conv}{Convergence status (0 = success)}
  \item{nsample}{Sample size}
}
\description{
Computes maximum penalized likelihood estimates for GEV parameters
using the Coles and Dixon (1999) prior penalty on the shape parameter.
}
\references{
Coles, S., & Dixon, M. (1999). Likelihood-based inference for extreme
value models. Extremes, 2(1), 5-23.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalty.R
\name{pk.norm.stnary}
\alias{pk.norm.stnary}
\alias{new_pf_norm}
\title{Normal preference function for shape parameter}
\usage{
pk.norm.stnary(para = NULL, mu = NULL, std = NULL)

new_pf_norm(para = NULL, mu = NULL, std = NULL)
}
\arguments{
\item{para}{A vector of GEV parameters.}

\item{mu}{Mean for normal distribution.}

\item{std}{Standard deviation for normal distribution.}
}
\value{
Preference function value (scalar).
}
\description{
Computes a normal distribution-based preference (penalty) function value
for the GEV shape parameter. The alias \code{new_pf_norm} is provided
for backward compatibility.
}
\examples{
# Normal preference with mean=-0.5, sd=0.2 at xi=-0.2
pk.norm.stnary(para = c(100, 20, -0.2), mu = -0.5, std = 0.2)

}
\references{
Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
L-moment estimation for stationary and nonstationary extreme value models.
arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
}
\seealso{
\code{\link{pk.beta}} for the beta penalty,
  \code{\link{MS_pk}} for the Martins-Stedinger penalty,
  \code{\link{glme.gev}} which uses these penalty functions.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{PhliuAgromet}
\alias{PhliuAgromet}
\title{Phliu Agrometeorological Station Data}
\format{
A data frame with 40 rows and 9 columns:
\describe{
  \item{Station.ID}{Station identifier (character)}
  \item{year}{Year of observation (numeric, 1984-2023)}
  \item{prec}{Annual maximum daily precipitation in mm (numeric)}
  \item{Name}{Station name (character)}
  \item{zone}{Climate zone code (character)}
  \item{latitude}{Station latitude in degrees (numeric)}
  \item{longitude}{Station longitude in degrees (numeric)}
  \item{Starting.year}{Record start year (integer)}
  \item{Ending.year}{Record end year (numeric)}
}
}
\source{
Phliu Agrometeorological Station, Thailand.
}
\usage{
PhliuAgromet
}
\description{
Annual maximum daily precipitation and station metadata from the Phliu
Agrometeorological Station, Chanthaburi province, Thailand (1984-2023).
A Mann-Kendall test indicates a significant increasing trend in the
annual maxima (tau = 0.235, p = 0.033), so this dataset is used for
demonstrating non-stationary (GEV11) estimation; it is the heavy
rainfall application of Shin et al. (2025).
}
\examples{
data(PhliuAgromet)
head(PhliuAgromet)

\donttest{
# Fit non-stationary GEV11 model by GLME
result <- glme.gev11(PhliuAgromet$prec, ntry = 5)
print(result$para.glme)
}

}
\references{
Shin, Y., Shin, Y., Park, J., & Park, J. S. (2025). Generalized method of L-moment estimation
for stationary and nonstationary extreme value models. arXiv preprint arXiv:2512.20385.
\doi{10.48550/arXiv.2512.20385}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{cons.MatC}
\alias{cons.MatC}
\title{Construct covariance matrix C for model averaging SE}
\usage{
cons.MatC(mywt, cov22, quant)
}
\arguments{
\item{mywt}{Weight computation result list from \code{weight.com()}.}

\item{cov22}{Array of 2x2 covariance matrices for each submodel (2 x 2 x numk).}

\item{quant}{Numeric vector of quantile probabilities.}
}
\value{
A list containing:
\describe{
  \item{MatC}{Array (numk x numk x numq) of cross-covariance values}
  \item{fin.se}{SE under fixed weights for MA}
  \item{fin.se.bma}{SE under fixed weights for BMA}
  \item{numk}{Number of submodels}
  \item{numq}{Number of quantiles}
  \item{wtgd}{MA weights}
  \item{bmaw}{BMA weights}
}
}
\description{
Internal function that constructs the cross-covariance matrix
between submodel quantile estimators using the delta method and quantile
correlation approximation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GN16.gev11.R
\name{make.qmax.gev11}
\alias{make.qmax.gev11}
\title{Construct pseudo-maxima for the GN16 method}
\usage{
make.qmax.gev11(xdat = NULL, orig.para = NULL, rob = FALSE)
}
\arguments{
\item{xdat}{Numeric data vector.}

\item{orig.para}{Initial parameter vector (mu0, mu1, sigma0, sigma1, xi).}

\item{rob}{If TRUE, uses robust regression.}
}
\value{
A list with sig0, sig1, and qmax.
}
\description{
Internal step of \code{\link{GN16.gev11}}: builds the
adjusted residual series (qmax) and estimates the log-scale trend
coefficients.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalty.R
\name{pk.beta}
\alias{pk.beta}
\alias{pk.beta.stnary}
\title{Beta preference function for the GEV shape parameter}
\usage{
pk.beta(
  para = NULL,
  lme.center = NULL,
  p = NULL,
  q = NULL,
  min.xi = -1,
  max.xi = 0.5,
  c0 = 0.35,
  c1 = 10,
  c2 = 2
)

pk.beta.stnary(
  para = NULL,
  lme.center = NULL,
  p = NULL,
  q = NULL,
  min.xi = -1,
  max.xi = 0.5,
  c0 = 0.35,
  c1 = 10,
  c2 = 2
)
}
\arguments{
\item{para}{A vector of GEV parameters; only \code{para[3]} (shape) is used.}

\item{lme.center}{L-moment estimates used as the center (adaptive mode).}

\item{p}{First shape parameter of the beta distribution.}

\item{q}{Second shape parameter (optional; if provided, fixed mode is used).}

\item{min.xi}{Lower support limit in fixed mode (default -1).}

\item{max.xi}{Upper support limit in fixed mode (default 0.5).}

\item{c0}{Half-width of the adaptive support (default 0.35).}

\item{c1}{Scaling hyperparameter for q (default 10).}

\item{c2}{Upper limit hyperparameter for q (default 2).}
}
\value{
A list containing:
\describe{
  \item{pk.one}{Preference function value (scalar)}
  \item{p}{Shape parameter p used}
  \item{q}{Shape parameter q used}
}
}
\description{
Computes the data-adaptive Beta preference (penalty) function for the GEV
shape parameter, used by both the stationary (\code{\link{glme.gev}}) and
the non-stationary (\code{\link{glme.gev11}}) GLME methods (v2.0.0 unified
implementation).

When \code{q} is \code{NULL} (adaptive mode), the support is
\eqn{(\max(-1, \hat\xi - c_0),\; \min(0.3, \hat\xi + c_0))} where
\eqn{\hat\xi} is \code{lme.center[3]}, and
\eqn{q = p + \min(|\hat\xi| c_1, c_2)} for \eqn{\hat\xi \le -0.05}
(otherwise \eqn{q = p}). When \code{q} is given (fixed mode), the support
is \code{(min.xi, max.xi)}; this mode is used for the fixed literature
penalties ("ms", "park", "cannon").

\code{pk.beta.stnary} is kept as a backward-compatible alias. Note that
since v2.0.0 both names use this unified implementation (the v1.x default
hyperparameters \code{c0=0.3, c2=5} changed to \code{c0=0.35, c2=2}, and
out-of-support values now return \code{1e-100} instead of \code{1}).
}
\examples{
# Adaptive beta preference for xi = -0.2 centered at LME xi = -0.15
pk.beta(para = c(100, 20, -0.2),
        lme.center = c(100, 20, -0.15), p = 6, c1 = 3, c2 = 1)

}
\references{
Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
L-moment estimation for stationary and nonstationary extreme value models.
arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
}
\seealso{
\code{\link{pk.norm.stnary}} for the normal penalty,
  \code{\link{MS_pk}} for the Martins-Stedinger penalty,
  \code{\link{glme.gev}}, \code{\link{glme.gev11}}.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{dist.noboot}
\alias{dist.noboot}
\title{Fit submodels and compute distance-based probabilities}
\usage{
dist.noboot(
  data = NULL,
  numk = NULL,
  hosking = NULL,
  boot.lme = TRUE,
  kpar = NULL,
  numom = NULL,
  ntry = 5,
  varcom = NULL,
  cov.lme = NULL,
  trim = NULL,
  cov.type = "lambda"
)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{numk}{Number of candidate submodels.}

\item{hosking}{List containing LME results and bootstrap information.}

\item{boot.lme}{Logical. Whether bootstrap LME was performed (default TRUE).}

\item{kpar}{Numeric vector of candidate xi values.}

\item{numom}{Number of L-moments to use.}

\item{ntry}{Number of optimization attempts. Default is 5.}

\item{varcom}{Logical. Whether to compute variance.}

\item{cov.lme}{Pre-computed LME covariance (default NULL).}

\item{trim}{Left trimming level.}

\item{cov.type}{Covariance type: "ratio" or "lambda" (default "lambda").}
}
\value{
A list containing:
\describe{
  \item{aic}{AIC values for each submodel}
  \item{mle3}{Matrix (numk x 3) of submodel parameter estimates}
  \item{kfix}{List of submodel fitting results}
  \item{cov2}{List of 2x2 covariance matrices (if varcom=TRUE)}
  \item{prob.mtx}{Matrix (numk x 3) of distance-based probabilities}
  \item{gdd}{Matrix (numk x 2) of generalized distances}
}
}
\description{
Internal function that fits GEV submodels with fixed xi for
each candidate and computes generalized L-moment distance or median-based
probabilities for weight construction.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ran.gev.R
\name{quagev.NS}
\alias{quagev.NS}
\title{Quantile function for non-stationary GEV models}
\usage{
quagev.NS(f = NULL, para = NULL, nsample = NULL, model = NULL)
}
\arguments{
\item{f}{Probability (or vector of probabilities) for quantile calculation.}

\item{para}{Parameter vector. For GEV11: (mu0, mu1, sigma0, sigma1, xi).}

\item{nsample}{Number of time points (sample size).}

\item{model}{Model type: "gev11", "gev10", "gev20", "gev01", or "gev00"/"gev".}
}
\value{
A matrix of quantiles (nsample x length(f)) or a vector if f is scalar.
}
\description{
Calculates quantiles for non-stationary GEV models including GEV11,
GEV10, GEV20, and stationary GEV00.
}
\examples{
# GEV11 model: time-varying quantiles
para <- c(84.55, 1.03, 2.91, 0.009, -0.08)  # mu0, mu1, sigma0, sigma1, xi
q99 <- quagev.NS(f = 0.99, para = para, nsample = 53, model = "gev11")
print(q99)

}
\references{
Shin, Y., Shin, Y., Park, J. & Park, J.-S. (2025). Generalized method of
L-moment estimation for stationary and nonstationary extreme value models.
arXiv preprint arXiv:2512.20385. \doi{10.48550/arXiv.2512.20385}
}
\seealso{
\code{\link{glme.gev11}} for non-stationary GEV estimation,
  \code{\link{glme.gev}} for stationary GEV estimation,
  \code{\link{ran.gev_all}} for random number generation.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{remle.gev}
\alias{remle.gev}
\title{Restricted MLE for GEV (Mixed Estimation)}
\usage{
remle.gev(
  xdat,
  ntry = 5,
  rest = "mean",
  quant = c(0.99, 0.995),
  trim = 0,
  CD.mle = NULL,
  mle = NULL,
  second = TRUE,
  w.mpse = FALSE
)
}
\arguments{
\item{xdat}{Numeric vector of data.}

\item{ntry}{Number of random starting points. Default is 5.}

\item{rest}{Restriction type: 'mean' (default) or 'median'.}

\item{quant}{Probabilities for quantile estimation. Default is c(0.99, 0.995).}

\item{trim}{Left trimming level. Default is 0.}

\item{CD.mle}{Coles-Dixon MLE (optional). If NULL, computed internally.}

\item{mle}{Standard MLE (optional). If NULL, computed internally.}

\item{second}{Logical. If TRUE, compute second-stage REMLE. Default is TRUE.}

\item{w.mpse}{Logical. If TRUE, compute MPSE. Default is FALSE.}
}
\value{
A list containing:
  \item{remle1}{First-stage REMLE estimates}
  \item{qua.remle1}{Quantiles from first-stage REMLE}
  \item{remle2}{Second-stage REMLE estimates (if second=TRUE)}
  \item{qua.remle2}{Quantiles from second-stage REMLE}
  \item{rest.method}{Restriction method used}
}
\description{
Computes restricted maximum likelihood estimates for GEV parameters
with constraints on the mean or median matching the sample statistics.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glme.gev.R
\name{pargev.kfix}
\alias{pargev.kfix}
\title{GEV parameter estimation with fixed shape parameter}
\usage{
pargev.kfix(lmom, kfix = 0.1, checklmom = TRUE, ...)
}
\arguments{
\item{lmom}{L-moments object.}

\item{kfix}{Fixed shape parameter value.}

\item{checklmom}{Whether to check L-moment validity.}

\item{...}{Additional arguments.}
}
\value{
A list with components:
\describe{
  \item{type}{Character "gev"}
  \item{para}{Numeric vector of GEV parameters (xi=location, alpha=scale, kappa=shape)}
  \item{source}{Character "pargev"}
}
}
\description{
Estimates GEV location and scale parameters from L-moments while keeping
the shape parameter fixed at a user-specified value. Modified from
\code{lmomco::pargev()}.
}
\examples{
data(haenam)
lmom <- lmomco::lmoms(haenam$X1, nmom = 3)
pargev.kfix(lmom, kfix = -0.1)

}
\references{
Hosking, J. R. M. (1990). L-moments: Analysis and estimation of
distributions using linear combinations of order statistics.
Journal of the Royal Statistical Society, Series B, 52(1), 105-124.
\doi{10.1111/j.2517-6161.1990.tb01775.x}
}
\seealso{
\code{\link{glme.gev}} for GLME estimation,
  \code{\link[lmomco]{pargev}} for the original L-moment GEV fitting.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_helpers.R
\name{wlik.xifix}
\alias{wlik.xifix}
\title{Likelihood-based weights with fixed xi}
\usage{
wlik.xifix(
  data = NULL,
  numk = NULL,
  kpar = NULL,
  weight = NULL,
  pertr = 0.85,
  varcom = FALSE,
  type = NULL,
  prior = NULL,
  trim = NULL
)
}
\arguments{
\item{data}{Numeric vector of data.}

\item{numk}{Number of candidate submodels.}

\item{kpar}{Numeric vector of candidate xi values.}

\item{weight}{Weighting method name.}

\item{pertr}{Perturbation parameter (default 0.85).}

\item{varcom}{Logical. Whether to compute variance (default FALSE).}

\item{type}{Type of computation.}

\item{prior}{Numeric vector of BMA prior values.}

\item{trim}{Left trimming level.}
}
\value{
A list containing:
\describe{
  \item{bmaw}{BMA weight vector}
  \item{wt}{MA weight vector}
  \item{aic}{AIC values for each submodel}
  \item{kfix}{Matrix (3 x numk) of submodel parameters}
  \item{cov2}{List of 2x2 covariance matrices (if varcom=TRUE)}
}
}
\description{
Internal function that computes AIC-based model averaging
weights using the profile likelihood with L-moment submodel estimates
for each candidate xi.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magev_plots.R
\name{magev.ksensplot}
\alias{magev.ksensplot}
\title{K Sensitivity Plot for MAGEV}
\usage{
magev.ksensplot(
  data = NULL,
  q.cut = 0.6,
  mink = 4,
  maxk = 20,
  quant = c(0.99, 0.995)
)
}
\arguments{
\item{data}{A numeric vector of data to be fitted (e.g., annual maxima).}

\item{q.cut}{Quantile cutoff for determining stability (default 0.6).}

\item{mink}{Minimum number of candidate submodels to test (default 4).}

\item{maxk}{Maximum number of candidate submodels to test (default 20).}

\item{quant}{The probabilities for high quantile estimation.
Default is c(0.99, 0.995).}
}
\value{
The optimal K value (integer) selected by the algorithm.
}
\description{
Plots return level estimates, standard errors, and first-order differences
across different numbers of candidate submodels K. This helps identify
stable regions where estimates converge and select an optimal K value.
}
\details{
The function computes MAGEV estimates for K ranging from \code{mink} to
\code{maxk}. For each K, it calculates:
\itemize{
  \item Return level estimates (black points)
  \item Normalized standard errors (blue line)
  \item First-order differences (red line with triangles)
}

The optimal K is selected as the smallest value where both the normalized
standard error and first-order difference are below their respective
\code{q.cut} quantile cutoffs. The selected K is indicated by a purple
vertical line.
}
\examples{
\donttest{
data(haenam)
optimal_k <- magev.ksensplot(haenam$X1)
print(optimal_k)
}

}
\references{
Shin, Y., Shin, Y., & Park, J. S. (2026). Model averaging with mixed criteria
for estimating high quantiles of extreme values: Application to heavy rainfall.
\emph{Stochastic Environmental Research and Risk Assessment}, 40(2), 47.
\doi{10.1007/s00477-025-03167-x}
}
\seealso{
\code{\link{ma.gev}} for the main model averaging function.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lme.gev11.R
\name{lme.gev11}
\alias{lme.gev11}
\title{L-moment parameter estimation for the non-stationary GEV11 model}
\usage{
lme.gev11(xdat, ntry = 5, ftol = 1e-05, show = FALSE)
}
\arguments{
\item{xdat}{A numeric vector of data to be fitted.}

\item{ntry}{Number of attempts (initial points) for the nonlinear solver
(default 5).}

\item{ftol}{Function tolerance for the solver (default 1e-5).}

\item{show}{If TRUE, prints the precision and parameters of each try.}
}
\value{
A list containing:
\itemize{
  \item lme.gev11 - The L-moment estimates (mu0, mu1, sigma0, sigma1, xi).
    If no solution is found, the WLS pre-estimate is returned.
  \item precis - Precision (mean absolute residual of the L-moment
    equations) of the selected solution.
  \item data - The input data (used by the plot method).
}
The object has class \code{"lme11"}; see \code{\link{GLmom-methods}}.
}
\description{
Estimates the parameters of the non-stationary GEV11 model
(mu(t) = mu0 + mu1*t, sigma(t) = exp(sigma0 + sigma1*t), constant xi)
by the pure L-moment method of Shin et al. (2025b, JKSS).
Starting from the WLS pre-estimate of \code{\link{strup.gev11}},
the three L-moment equations (matching the sample L-moments of the
Gumbel-transformed data with the population L-moments of the standard
Gumbel distribution) are solved by Broyden's method for
(mu0, sigma0, xi), and the best solution is chosen by a goodness-of-fit
measure.

This function replaces \code{\link{gado.prop_11}} of GLmom v1.x
(renamed in v2.0.0, with slightly changed input/output).
It is also equivalent to \code{glme.gev11(xdat, pen="no")}.
}
\examples{
\donttest{
data(PhliuAgromet)
result <- lme.gev11(PhliuAgromet$prec, ntry = 5)
print(result$lme.gev11)
}

}
\references{
Shin, Y., Shin, Y. & Park, J.-S. (2025). Building nonstationary extreme value
model using L-moments. Journal of the Korean Statistical Society, 54, 947-970.
\doi{10.1007/s42952-025-00325-3}
}
\seealso{
\code{\link{glme.gev11}} for the penalized (GLME) version,
  \code{\link{strup.gev11}} for the WLS pre-estimation,
  \code{\link{GN16.gev11}} for the GN16 method,
  \code{\link{nsgev}} and \code{\link{gado.prop_11}} for the v1.x
  compatibility wrappers.
}
\author{
Yonggwan Shin, Seokkap Ko, Jihong Park, Yire Shin, Jeong-Soo Park
}
